<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/03/08 16:37
 */
namespace api\controllers;

use common\models\AlternateDate;
use yii\filters\VerbFilter;
use common\classes\Error;

/**
 * Alternate Date controller
 */
class AlternateController extends BaseController
{
    public function actionList() {
        $list = AlternateDate::find()
            ->select(['date' => 'SpecialDate', 'weekday' =>'AlteredScheduleDay'])
            ->orderBy(['date' => SORT_ASC])
            ->asArray()
            ->all();
        return $this->response(Error::STATUS_SUCCESS, $list);
    }

    public function actionUpdate($date, $weekday){
        if(!is_numeric($weekday) || $weekday < 0 || $weekday > 6)
            return $this->response(Error::STATUS_ALTERNATE_WEEKDAY_ERROR);
        $model = AlternateDate::findOne(['SpecialDate' => $date]);
        if(empty($model))
            return $this->response(Error::STATUS_ALTERNATE_NOT_EXISTS);
        $this->logTitle['title1'] = $date;
        $this->logTitle['title2'] = 'ori/weekday' . $weekday;
        $model->AlteredScheduleDay = $weekday;
        if($model->save())
            return $this->response(Error::STATUS_SUCCESS);
        return $this->response(Error::STATUS_FAILED);
    }

    public function actionDelete($date){
        AlternateDate::deleteAll(['SpecialDate' => $date]);
        $this->logTitle['title1'] = $date;
        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionNew($date, $weekday){
        if(!is_numeric($weekday) || $weekday < 0 || $weekday > 6)
            return $this->response(Error::STATUS_ALTERNATE_WEEKDAY_ERROR);
        $model = AlternateDate::findOne(['SpecialDate' => $date]);
        if(!empty($model))
            return $this->response(Error::STATUS_ALTERNATE_EXISTS);
        $model = new AlternateDate();
        $model->SpecialDate = $date;
        $model->AlteredScheduleDay = $weekday;
        $this->logTitle['title1'] = $date;
        $this->logTitle['title2'] = 'ori/weekday' . $weekday;
        if($model->save())
            return $this->response(Error::STATUS_SUCCESS, $model);
        return $this->response(Error::STATUS_FAILED);
    }
}
