<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2017/12/21 11:43
 */
namespace api\controllers;

use common\models\BaseInfo;
use Yii;
use yii\rest\Controller;
use yii\filters\Cors;
use common\classes\Error;
use common\classes\Common;
use common\models\SystemLog;
use yii\filters\auth\QueryParamAuth;

/**
 * Site controller
 */
class BaseController extends Controller
{
    // token expire seconds
    const LOGIN_EXPIRE = 259200;
    // old token expire seconds after refresh
    CONST REFRESH_EXPIRE = 3600;
    public $postRequest;
    public $data;
    public $isAdmin = false;
    public $isSuper = false;
    public $userinfo = null;
    public $serverPri = 0;
    public $priMap = [
        'mode' => 'mode',
        'template' => 'template',
        'session' => 'brd',
        'media' => 'brd',
        'system' => 'system',
        'support' => 'system',
        'monitor' => 'system',
        'sub-network' => 'system',
        'network' => 'system',
        'alternate' => 'system',
        'datetime' => 'system',
        'device' => 'device',
        'device-class' => 'device',
        'projector' => 'device',
        'log' => 'log',
        'statistic' => 'log',
        'message' => 'log',
        'user' => 'ic',
        'event' => 'system',
        'ipcall' => 'ipcall',
        'timetable' => 'device',
        'timetable-sync' => 'system',
    ];
    public $priExempt = [
        'user' => [
            'update-mgr','view-mgr'
        ]
    ];

    public $logTitle = [
        'username' => null,
        'title1' => null,
        'title2' => null
    ];
    public $logDetail = [];
    public $logActions = [
        'site' => ['login', 'oauth-login', 'unionid-login', 'logout'],
        'alternate' => ['update', 'new', 'delete'],
        'datetime' => ['update'],
        'device' => ['set-config', 'control','delete', 'copy', 'cloud-control'],
        'log' => ['clean'],
        'media' => ['update','new','delete','upload', 'move'],
        'message' => ['clean'],
        'mode' => ['new', 'update', 'delete', 'duplicate'],
        'network' => ['update'],
        'projector' => ['new', 'update', 'delete'],
        'session' => ['realtime-update', 'realtime-delete', 'delete','update','new','schedule-new', 'schedule-update','schedule-delete','playtime-delete','playtime-update', 'ctrl'],
        'statistic' => ['clean'],
        'sub-network' => ['new', 'update', 'delete'],
        'support' => ['update'],
        'system' => ['push-certificate', 'iot-update', 'iot-mode-update','reboot','shutdown'],
        'template' => ['active', 'new', 'update', 'delete', 'duplicate'],
        'user' => ['update-mgr', 'auth', 'update','delete','new', 'unionid-bind', 'unionid-unbind', 'wx-unbind', 'cloud-unionid-bind'],
        'monitor' => ['set-dev-offline-monitor-config', 'set-dev-statistics-config', 'set-db-error-config'],
        'timetable-config' => ['set-config', 'section-delete', 'section-new']
    ];
    public $devicePriActions = [];
    public $sessionPriActions = [
        'session' => ['realtime-list', 'realtime-ctrl', 'realtime-media-list', 'realtime-view', 'realtime-update',
            'realtime-delete', 'bw80x-play-ctrl' ],
        'media' => true
    ];
    public $resultCode = Error::STATUS_BLANK;
    public function behaviors()
    {
        $behaviors = parent::behaviors(); // TODO: Change the autogenerated stub
        $behaviors['authenticator'] = [
            'class' => QueryParamAuth::className(),
            'optional' => []
        ];
        $behaviors['corsFilter'] = [
            'class' => Cors::className(),
            'cors' => [
                // restrict access to
                'Origin' => ['*'],
                // restrict access to
                'Access-Control-Request-Method' => ['*'],
                // Allow only POST and PUT methods
                'Access-Control-Request-Headers' => ['*'],
                // Allow only headers 'X-Wsse'
                'Access-Control-Allow-Credentials' => true,
                // Allow OPTIONS caching
                'Access-Control-Max-Age' => 3600,
                // Allow the X-Pagination-Current-Page header to be exposed to the browser.
                'Access-Control-Expose-Headers' => ['X-Pagination-Current-Page'],
            ],
        ];
        return $behaviors;
    }

    public function beforeAction($action)
    {
        //sign data verify
        date_default_timezone_set(Yii::$app->base->timezone);
        $lang = Yii::$app->request->get('lang', 'zh-CN');
        if(!in_array($lang, Common::LANGUAGE_SUPPORT))
            $lang = 'zh-CN';
        Yii::$app->language = $lang;
        $ret = Parent::beforeAction($action);
        if(!$ret)
            return $ret;
        if (isset( $this->postRequest) && in_array( $action->id,  $this->postRequest)) {
             $this->data = Yii:: $app->getRequest()->post('data');
            if (is_null( $this->data)) {
                Yii:: $app->getResponse()->data =  $this->response(Error::STATUS_PARAM_ERROR);
                return false;
            }
        }
        //guest
        if(Yii::$app->user->isGuest)
            return $ret;
        $user = Yii::$app->user;
        $cache = Yii::$app->getCache();
        $uid = $user->id;

        // server priviledge
        $model = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_SCHOOL_PRIVILEGE]);
        if (!empty($model) && !empty($model->InfoValue))
            $this->serverPri = $model->InfoValue;
        $model = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_SCHOOL_UUID]);
        if (empty($model) || empty($model->Description) || strlen($model->Description) < 32)
            $this->serverPri = 7;
        $this->userinfo = $cache->get(Common::USER_INFO_SALT . $uid);
        if(empty($this->userinfo)){
            $this->userinfo = SiteController::packageUserInfo($user->getIdentity());
            if (empty($this->userinfo)) {
                Yii::$app->response->data = $this->response(Error::STATUS_FAILED);
                return false;
            }
            $cache->set(Common::USER_INFO_SALT . $uid, $this->userinfo, 10);
        }
        $this->logTitle['username'] = $user->getIdentity()->User_Name;
        $this->isAdmin = ($uid == 0);
        $this->isSuper = $this->isAdmin || in_array('system', $this->userinfo['pri']);
        // server device pri check
        if (key_exists($this->id, $this->devicePriActions) &&
            (!is_array($this->devicePriActions[$this->id]) || in_array($action->id, $this->devicePriActions[$this->id])) &&
            (($this->serverPri & 0x01) == 0)) {
            Yii::$app->response->data = $this->response(Error::STATUS_PRIVILEGE_ERROR);
            return false;
        }
        // server session pri check
        if (key_exists($this->id, $this->sessionPriActions) &&
            (!is_array($this->sessionPriActions[$this->id]) || in_array($action->id, $this->sessionPriActions[$this->id])) &&
            (($this->serverPri & 0x06) == 0)) {
            Yii::$app->response->data = $this->response(Error::STATUS_PRIVILEGE_ERROR);
            return false;
        }
        // user pri check
        if($this->isSuper || !key_exists($this->id, $this->priMap) ||
            in_array($this->priMap[$this->id], $this->userinfo['pri']) || (
            key_exists($this->id, $this->priExempt) &&
            in_array($action->id, $this->priExempt[$this->id])) ||
            in_array($action->id, $this->behaviors()['authenticator']['optional'])){
            return $ret;
        }
        Yii::$app->response->data = $this->response(Error::STATUS_PRIVILEGE_ERROR);
        return false;
    }

    public function response($status, $data = null)
    {
        $this->resultCode = $status;
        $ret = [
            'status' => $status,
            'message' => Yii::t('app/error', $status)
        ];
        if(!is_null($data))
            $ret['data'] = $data;
        return $ret;
    }

    public function actionError() {
        $error = Yii::$app->errorHandler->exception;
        return $this->response(Error::STATUS_FAILED, $error);
    }

    /*
     * common process for list api
     */
    public function listQuery(&$query, $param = null)
    {
        if(!is_null($param)){
            if (isset($param['order']) && is_array($param['order'])) {
                foreach ($param['order'] as $key => $value) {
                    $key = trim($key);
                    $value = trim($value);
                    if ($value === 'desc')
                        $query->orderBy([$key => SORT_DESC]);
                    else
                        $query->orderBy([$key => SORT_ASC]);
                }
            }
            if (isset($param['search']) && is_array($param['search'])) {
                foreach ($param['search'] as $key => $value) {
                    if(!is_null($value))
                        $query->andWhere(['like', trim($key), trim($value)]);
                }
            }
            if (isset($param['orSearch']) && is_array($param['orSearch'])) {
                $cond = ['or'];
                foreach ($param['orSearch'] as $key => $value) {
                    if(!is_null($value))
                        $cond[] = ['like', trim($key), trim($value)];
                }
                if(count($cond) > 0)
                    $query->andWhere($cond);
            }
            if (isset($param['key']) && is_array($param['key'])) {
                foreach ($param['key'] as $key => $value) {
                    if(is_null($value))
                        $query->andWhere([trim($key) => null]);
                    else if(is_array($value))
                        $query->andWhere(['in', trim($key), $value]);
                    else
                        $query->andWhere([trim($key) => trim($value)]);
                }
            }
            if (isset($param['notkey']) && is_array($param['notkey'])) {
                foreach ($param['notkey'] as $key => $value) {
                    if(is_null($value))
                        $query->andWhere(['not', [trim($key) => null]]);
                    else if(is_array($value))
                        $query->andWhere(['not', ['in', trim($key), $value]]);
                    else
                        $query->andWhere(['not', [trim($key) => trim($value)]]);
                }
            }
            $count = $query->count();
            if( isset($param['limit']) && is_numeric($param['limit'])
                && isset($param['page']) && is_numeric($param['page']))
                $query->offset($param['page'] * $param['limit'])
                ->limit($param['limit']);
            if (isset($param['select']) && is_string($param['select'])) {
                $query = $query->select($param['select']);
            }
        }
        else
            $count = $query->count();
        return $count;
    }

    /**
     * @param $query
     * @param $param
     * @return array
     */
    public function list($query, $param){
        $count = $this->listQuery($query, $param);
        $lists = $query->all();
        return $this->response(Error::STATUS_SUCCESS, ['count' => $count, 'list' => $lists]);
    }

    public $logSended = false;
    public function afterAction($action, $result)
    {
        $ret = parent::afterAction($action, $result); // TODO: Change the autogenerated stub
        header('Cache-Control: no-cache');
        if(!$this->logSended && isset($this->logActions[$this->id]) && in_array($action->id, $this->logActions[$this->id]))
            $this->writeLog($this->resultCode, $this->logDetail);
        return $ret;
    }

    public function getClientIp(){
        if (!empty($_SERVER['HTTP_CLIENT_IP']))
            return $_SERVER['HTTP_CLIENT_IP'];
        elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR']))
            return $_SERVER['HTTP_X_FORWARDED_FOR'];
        else
            return $_SERVER['REMOTE_ADDR'];
    }

    public function writeLog($resultCode, $logDetail) {
        $model = new SystemLog();
        $model->ip = (Yii::$app->getUser()->identity && Yii::$app->getUser()->identity->cloudLogin) ? '127.0.0.1' : $this->getClientIp();
        $model->username = ''.$this->logTitle['username'];
        $model->title1 = ''.$this->logTitle['title1'];
        $model->title2 = ''.$this->logTitle['title2'];
        $model->detail = empty($logDetail) ? null : json_encode($logDetail, JSON_UNESCAPED_UNICODE);
        if(mb_strlen($model->detail) > 1024)
            $model->detail = mb_substr($model->detail, 0, 1024);
        $model->type = $this->id;
        $model->action = $this->action->id;
        $model->result = $resultCode;
        $model->save();
        return $model;
    }

    public function logChange($model, $key, $value) {
        $model->$key = $value;
        $model->save();
    }
}

