<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/5/23 14:18
 */
namespace api\controllers;
use common\models\PlaySessions;
use Yii;
use yii\helpers\ArrayHelper;

/**
 * BW80x controller
 */
class Bw80xController extends BaseController
{
    public $uid;
    public $data;
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return  ArrayHelper::merge (
            parent::behaviors(),[
            'authenticator' => [
                'optional' => ['login','list-real-time-bw80x','list-session-devices','load-play-session', 'edit-session-for-app', 'forbid', 'srshook']
            ]
        ]);
    }

    public function beforeAction($action)
    {
        $this->uid = Yii::$app->getRequest()->post('UserID');
        $session = Yii::$app->getRequest()->post('Session');
        $this->data = Yii::$app->getRequest()->post('DataParam');
        if(!empty($this->data)) {
            $this->data = str_replace("'", '"', $this->data);
            $this->data = json_decode($this->data);
        }
        if($this->action->id != 'login' && $this->action->id != 'srshook'){
            if(empty($session) || strlen($session) < 32) {
                Yii::$app->response->data = json_encode(['ErrCode' => -1, 'ErrMsg' => '参数错误', null]);
                return false;
            }
            $sessionCached = Yii::$app->cache->get('bw80x-adp-ca' . $session);
            if($sessionCached !== $session) {
                Yii::$app->response->data = json_encode(['ErrCode' => -1, 'ErrMsg' => '参数错误', null]);
                return false;
            }
        }
        return parent::beforeAction($action); // TODO: Change the autogenerated stub
    }

    public function actionLogin(){
        if(is_null($this->uid) || empty($this->data) || empty($this->data->Username) || !is_string($this->data->Username) ||
            empty($this->data->Password)|| !is_string($this->data->Password) ||
            !preg_match('/^38/', $this->data->Username) ||
            $this->data->Password != strtoupper(md5('BW803'. $this->data->Username))
        )
            return ['ErrCode' => 101, 'ErrMsg' => '登录失败', 'Data' => ''];
        $session = Yii::$app->security->generateRandomString(32);
        Yii::$app->cache->set('bw80x-adp-ca' . $session, $session, 999999999);
        $ret = [
            'User_ID' => 0,
            'IcCard' => 0,
            'Session' => $session,
            'SignKey' => Yii::$app->security->generateRandomString(32),
            'LanIp' => '127.0.0.1',
            'Version' => 6,
            'Permission' => [
                'DevCtrPermission' => 0,
                'BroadcastPermission' => 1,
                'RfDevCtrPermission' => 0,
                'ICPermission' => 0,
                'ScanPermission' => 0
            ],
            'User_Name' => $this->data->Username
        ];
        return ['ErrCode' => 0, 'ErrMsg' => '登录成功', 'Data' => $ret];
    }

    public function actionListRealTimeBw80x(){
        $models = Yii::$app->db->createCommand('SELECT SessionID, PlayerID as SessionName, SessionType, Priority, CircleLength
			FROM PlaySessions WHERE SessionType IN(10);')
            ->queryAll();
        return ['ErrCode' => 0, 'ErrMsg' => '', 'Data' => $models];
    }

    public function actionListSessionDevices(){
        if(empty($this->data) || !isset($this->data->SessionID))
            return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];
        $session = PlaySessions::findOne(['SessionID' => $this->data->SessionID]);
        if(empty($session))
            return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];

        $devices = self::LoadSessionDevice ( $session->SessionID, $this->uid );
        return ['ErrCode' => 0, 'ErrMsg' => '', 'Data' => $devices];
    }

    public static function LoadDevices($UserID, $SessionID, $Parent) {
        $sql = 'SELECT a.DevID AS id, a.Dev_Name AS text, a.Parent AS Parent, 1 AS IsDevice, "closed" AS state,
					IF(ISNULL(b.SessionID), 0, 1) AS checked
				FROM DeviceList a
				LEFT JOIN (SELECT * FROM SessionDevice WHERE SessionID=' . $SessionID . ') AS b ON a.DevID = b.DeviceID ';
        if (empty ( $Parent )) {
            $sql .= ' WHERE ISNULL(a.Parent)';
        } else {
            $sql .= ' WHERE a.Parent = ' . $Parent;
        }
        $sql .= ' ORDER by a.DevID';

        return Yii::$app->db->createCommand($sql)->queryAll();
    }

    public static function dealDeviceTree($TreeData, &$selectAll) {
        if (empty ( $TreeData )) {
            return $TreeData;
        }
        $out = array ();
        $SelectNum = 0;
        $HasHalf = false;

        foreach ( $TreeData as $key => $item ) {
            $item ['id'] = str_replace ( 'DC_', "", $item ['id'] );
            $item ['Parent'] = str_replace ( 'DC_', "", $item ['Parent'] );
            $item ['name'] = $item ['text'];
            $item ['checked'] = intval ( $item ['checked'] );
            $item ['open'] = false;

            if ($item ['Parent'] == "") {
                $item ['Parent'] = null;
            }
            if ($item ['IsDevice'] != 0) {
                $out [] = $item;

                if ($item ['checked'] == 1) {
                    $SelectNum ++;
                } else {
                    $HasHalf = true;
                }
                continue;
            }

            if (empty ( $item ['children'] )) {
            } else {
                $TempAll = 1;
                $children = self::dealDeviceTree ( $item ['children'], $TempAll );

                if (! empty ( $children )) {
                    $item ['checked'] = $TempAll == 1 || $TempAll == 2 ? 1 : 0;
                    $item ['halfCheck'] = $TempAll == 2 ? 1 : 0;
                    $item ['children'] = $children;
                    $out [] = $item;
                }

                if ($TempAll == 1) {
                    $SelectNum ++;
                } else if ($TempAll == 2) {
                    $HasHalf = true;
                }
            }
        }
        $selectAll = $SelectNum <= 0 ? 0 : ($HasHalf ? 2 : 1);

        return $out;
    }

    public static function LoadSessionDevice($SessionID, $UserID) {
        $DeviceClassArr = self::LoadDevClass ( $UserID, $SessionID, null );
        $Devs = self::LoadDevices ( $UserID, $SessionID, null );
        $DeviceClassArr = empty ( $DeviceClassArr ) ? array () : $DeviceClassArr;
        if (! empty ( $Devs )) {
            $DeviceClassArr = array_merge ( $DeviceClassArr, $Devs );
        }
        $selectAll = 0;
        return self::dealDeviceTree ( $DeviceClassArr, $selectAll );
    }

    private static function LoadDevClass($UserID, $SessionID, $Parent) {
        $sql = 'SELECT ClassID AS id,Name as text, Parent AS Parent, 0 AS IsDevice, "closed" AS state, 0 AS checked FROM DevClasses';
        if (empty ( $Parent )) {
            $sql .= ' WHERE ISNULL(Parent)';
        } else {
            $sql .= ' WHERE Parent = ' . $Parent;
        }
        $Out = Yii::$app->db->createCommand($sql)->queryAll();
        if (! empty ( $Out )) {
            foreach ( $Out as $key => $item ) {
                $Children = self::LoadDevClass ( $UserID, $SessionID, $item ['id'] );
                $Devs = self::LoadDevices ( $UserID, $SessionID, $item ['id'] );

                if (! empty ( $Devs )) {
                    if (empty ( $Children )) {
                        $Children = $Devs;
                    } else {
                        $Children = array_merge ( $Children, $Devs );
                    }
                }
                $Out [$key] ['children'] = $Children;
            }
        }
        return $Out;
    }

    public function actionLoadPlaySession(){
        if(empty($this->data) || !isset($this->data->SessionID))
            return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];
        $PlaySession = PlaySessions::find()->where(['SessionID' => $this->data->SessionID])->asArray()->one();
        if(empty($PlaySession))
            return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];
        $baseInfo = [
            'SessionID' => $PlaySession ['SessionID'],
            'SessionName' => $PlaySession ['SessionName'],
            'SessionType' => $PlaySession ['SessionType'],
            'Priority' => $PlaySession ['Priority'],
            'SteamTypeUp' => round ( $PlaySession ['StreamType'] / 128 ),
            'SteamTypeDown' => $PlaySession ['StreamType'] - round ( $PlaySession ['StreamType'] / 128 ) * 128,
            'Owner' => $PlaySession ['Owner']
        ];
        $taskInfo['BaseInfo'] = $baseInfo;

        if (! is_null ( $PlaySession ['BrdMode'] ) && $PlaySession ['BrdMode'] > 0) {
            $mode = array ();
            $mode ['BrdMode'] = $PlaySession ['BrdMode'];
            $mode ['MemmberMode'] = $PlaySession ['MemberMode'];
            $taskInfo ['Mode'] = $mode;
        }

        if ($baseInfo ['SteamTypeDown'] == 1 || $baseInfo ['SteamTypeDown'] == 2 || $baseInfo ['SteamTypeDown'] == 3) {
            $Brodcast = array ();
            $Brodcast ['AudioGain'] = $PlaySession ['AudioGain'];
            $Brodcast ['CircleLength'] = $PlaySession ['CircleLength'];
            $Brodcast ['PlayMode'] = $PlaySession ['PlayMode'];
            $Brodcast ['PlayFileList'] = Yii::$app->db->createCommand("SELECT a.FileID, a.SequenceID, b.FileName, b.TotalTime FROM PlayListFiles a LEFT JOIN
                    MediaFiles b ON a.FileID=b.MediaID WHERE SessionID='" .  $PlaySession ['SessionID'] . "' ORDER BY SequenceID")
                ->queryAll();

            $taskInfo ['Brodcast'] = $Brodcast;
        }

        $taskInfo ['SessionDevice'] = self::LoadSessionDevice ( $PlaySession ['SessionID'], $this->uid );

        $sql = "SELECT a.ScheduleID, WeekDay, DATE_FORMAT(PlayTime,'%H:%i:%S') as PlayTime, TempFlag, Pause FROM PlaySessionTime a
			LEFT JOIN PlayingSessions b ON a.SessionID=b.SessionID AND a.ScheduleID=b.ScheduleID WHERE a.SessionID='" . $PlaySession ['SessionID'] . "'";
        $taskInfo ['PlayTime'] = Yii::$app->db->createCommand($sql)->queryAll();

        return ['ErrCode' => 0, 'ErrMsg' => '', 'Data' => $taskInfo];
    }

    public static function DealSessionDevice($SessionID, $SessionDevice) {
        if (empty( $SessionDevice )) {
            return;
        }
        $TempArr = array ();
        foreach ( $SessionDevice as $item ) {
            $TempArr [] = $item ->DeviceID;
        }

        if (! empty ( $TempArr )) {
            $sql = "DELETE FROM SessionDevice WHERE SessionID='" . $SessionID . "' AND DeviceID NOT IN(" . implode ( ",", $TempArr ) . ")";
            Yii::$app->db->createCommand($sql)->execute();
        } else {
            $sql = "DELETE FROM SessionDevice WHERE SessionID='" . $SessionID . "'";
            Yii::$app->db->createCommand($sql)->execute();
            return;
        }
        $sql = 'INSERT IGNORE INTO SessionDevice(DeviceID, SessionID, KeyIndex, DevDisabled)VALUES';
        $Values = array ();

        foreach ( $SessionDevice as $item ) {
            $Values [] = '("' . $item ->DeviceID . '", "' . $SessionID . '", "0", "0")';
        }

        if (! empty ( $Values )) {
            $sql .= implode ( ',', $Values );
            Yii::$app->db->createCommand($sql)->execute();
        }
    }

    public function actionEditSessionForApp(){
        if (empty($this->data) || empty($this->data->BaseInfo || empty( $this->data->BaseInfo->SessionID)))
            return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];
        $SessionID = $this->data->BaseInfo->SessionID;
        $SessionModel = PlaySessions::findOne(['SessionID' => $SessionID]);
        if(empty($SessionModel))
            return ['ErrCode' => -1, 'ErrMsg' => '参数错误', null];

        if (!empty($this->data->SessionDevice)) {
            $SessionDevice = $this->data->SessionDevice;
            self::DealSessionDevice ( $SessionID, $SessionDevice );
        }

        if (!empty($this->data->Brodcast)) {
            $Brodcast = $this->data->Brodcast;

            if (isset ( $Brodcast->AudioGain)) {
                $AudioGain = $Brodcast->AudioGain;
                if ($AudioGain != - 100) {
                    $AudioGain = $AudioGain <= - 20 ? - 20 : $AudioGain;
                    $AudioGain = $AudioGain >= 20 ? 20 : $AudioGain;
                }
                $UpdateAttr [] = 'AudioGain="' . $AudioGain . '"';
            }
            if (isset ( $Brodcast ->CircleLength)) {
                $UpdateAttr [] = 'CircleLength="' . $Brodcast->CircleLength . '"';
            }
            if (isset ( $Brodcast ->PlayMode)) {
                $UpdateAttr [] = 'PlayMode="' . $Brodcast->PlayMode . '"';
            }
        }

        return ['ErrCode' => 0, 'ErrMsg' => '参数错误', null];
    }

    public static $srsAction = array('on_publish', 'on_unpublish', 'on_connect', 'on_close');
    public static $srsApp = array('bw803', 'camera');
    public function actionSrshook(){
        $body = Yii::$app->request->post();
        $action = $body['action'];
        $app = $body['app'];
        if(!in_array($app, self::$srsApp)){
            echo '1';
            return;
        }

        if($app == 'bw803'){
            echo '0';
            return;
        }

        if(!in_array($action, self::$srsAction)){
            echo '0';
            return;
        }

        echo '0';
        return;
    }

    public function actionForbid(){
        return ['ErrCode' => 0, 'ErrMsg' => '', null];
    }
}
