<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/1/12 14:37
 */
namespace api\controllers;

use common\classes\Common;
use common\models\DeviceList;
use common\models\VDevClasses;
use Yii;
use yii\db\Exception;
use yii\filters\VerbFilter;
use common\classes\Error;
use common\models\DevClasses;

/**
 * Site controller
 */
class DeviceClassController extends BaseController
{
    public $allPri;
    public $uid;
    public function beforeAction($action)
    {
        $ret = parent::beforeAction($action); // TODO: Change the autogenerated stub
        if(!$ret)
            return $ret;
        $user = Yii::$app->user->identity;
        if ($user->Mgr_Privilege == 1 || $this->isSuper)
            $this->allPri = true;
        else {
            $this->allPri = false;
        }
        $this->uid = $user->User_ID;
        if(!in_array($action->id, ['list'])) {
            $all = true;
            $cacheKey = Common::USER_DEVICE_CLASS_LIST_CACHE . $this->uid . ".all.$all.pri$this->allPri";
            Yii::$app->cache->delete($cacheKey);
            $all = false;
            $cacheKey = Common::USER_DEVICE_CLASS_LIST_CACHE . $this->uid . ".all.$all.pri$this->allPri";
            Yii::$app->cache->delete($cacheKey);
        }
        return $ret;
    }

    /*
     * get All Device Class
     */
    public function actionList(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data))
            $trees = DevClasses::find()->all();
        else {
            $all = (isset($data['all']) && $data['all'] == 'true') ? true: false;
            if(isset($data['getDevice'])) {
                $rootTrees = Yii::$app->getDb()
                    ->createCommand(     "SELECT null as `ClassID`, null as 'Name', null as 'Parent',
                                                                          `DeviceList`.`DevID` AS `DevID`,`DeviceList`.`Dev_Name` AS `Dev_Name`
                                                                            FROM `DeviceList` where `DeviceList`.`Parent` is NULL;")
                    ->queryAll();
                $sugTrees = VDevClasses::find()->orderBy('ClassID')->asArray()->all();
                $trees = array_merge($sugTrees, $rootTrees);

                $user = Yii::$app->user->identity;
                if($user->Mgr_Privilege != 1 && (!$this->isSuper))
                    $devices = DeviceController::userDevices($this->uid, 2);
                $cacheKey = Common::USER_DEVICE_CLASS_LIST_CACHE . $this->uid . ".all.$all.pri$this->allPri";
                $retReal = Yii::$app->cache->get($cacheKey);
                if(!empty($retReal))
                    return $this->response(Error::STATUS_SUCCESS, $retReal);
                $ret = [];
                foreach ($trees as $arr)
                {
                    if(is_object($arr))
                        $arr = $arr->toArray();
                    if(!is_null($arr['DevID']) && ($this->allPri || isset($devices[$arr['DevID']])))
                    {
                        $Device = [
                            'DevID' => $arr['DevID'],
                            'Dev_Name' => $arr['Dev_Name'],
                        ];
                    }
                    else
                    {
                        $Device = null;
                    }
                    if(!isset($ret[$arr['ClassID']])){
                        $ret[$arr['ClassID']] = [
                            'ClassID' => $arr['ClassID'],
                            'Name' => $arr['Name'],
                            'Parent' => is_null($arr['Parent']) ? "root" : $arr['Parent']
                        ];
                    }
                    if(!is_null($Device))
                        $ret[$arr['ClassID']]['Device'][] = $Device;
                }
                if(!$all) {
                    $parent = [];
                    foreach ($ret as $key => $class) {
                        if (!is_null($class['ClassID']))
                            $parent[$class['Parent']][] = $class['ClassID'];
                    }
                    for ($i = 0; $i < 4; $i++) {
                        foreach ($parent as $key => $child) {
                            foreach ($child as $idx => $id) {
                                if (!isset($parent[$id]) && empty($ret[$id]['Device'])) {
                                    unset($ret[$id], $parent[$key][$idx]);
                                }
                            }
                            if (empty($parent[$key]) && (!isset($ret[$key]) || empty($ret[$key]['Device']))) {
                                if (isset($ret[$key]) && !empty($parent[$ret[$key]['Parent']]))
                                    unset($parent[$ret[$key]['Parent']][$key]);
                                unset($parent[$key], $ret[$key]);
                                continue;
                            }
                        }
                    }
                }
                $retReal = [];
                foreach ($ret as $value) {
                    if($value['Parent'] == 'root')
                        $value['Parent'] = null;
                    $retReal[] = $value;
                }
                Yii::$app->cache->set($cacheKey, $retReal, 10);
                return $this->response(Error::STATUS_SUCCESS, $retReal);
            }
            else {
                $query = DevClasses::find();
                if(isset($data['class']))
                    $trees = $query->where(['ClassID' => $data['class']])->all();
                else
                    $trees = $query->all();
            }
        }
        return $this->response(Error::STATUS_SUCCESS, $trees);
    }
    /*
     * create device class
     */
    public function actionNew(){
        $model = new DevClasses();
        try{
            if($model->load(Yii::$app->request->post(), 'data') && $model->save())
                return $this->response(Error::STATUS_SUCCESS, $model);
            else
                return $this->response(Error::STATUS_FAILED, $model->errors);
        }catch (Exception $e){}
        return $this->response(Error::STATUS_FAILED);
    }

    /*
     * update device class
     */
    public function actionUpdate(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['ClassID']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $model = DevClasses::findOne(['ClassID' => $data['ClassID']]);
        if(is_null($model))
            return $this->response(Error::STATUS_FOLDER_NOT_EXISTS);
        try{
            if($model->load(Yii::$app->request->post(), 'data') && $model->save())
                return $this->response(Error::STATUS_SUCCESS, $model);
            else
                return $this->response(Error::STATUS_FAILED, $model->errors);
        }catch (Exception $e){}
        return $this->response(Error::STATUS_FAILED);
    }
    /*
     * update device class
     */
    public function actionDelete(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['ClassID']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $model = DevClasses::findOne(['ClassID' => $data['ClassID']]);
        if(is_null($model))
            return $this->response(Error::STATUS_FOLDER_NOT_EXISTS);
        if($model->delete())
            return $this->response(Error::STATUS_SUCCESS);
        else
            return $this->response(Error::STATUS_FAILED);
    }

    /*
     * add device
     */
    public function actionMove(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['source']) || !isset($data['target']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $src = $data['source'];
        $tgt = $data['target'];
        if(isset($tgt['class'])){
            if($tgt['class'] === -1)
                $tgtClassId = "null";
            else{
                $tgtModel = DevClasses::findOne(['ClassID' => $tgt['class']]);
                if(is_null($tgtModel))
                    return $this->response(Error::STATUS_SRC_FOLDER_NOT_EXISTS);
                $tgtClassId = $tgt['class'];
            }
        }
        else if(isset($tgt['device'])){
            $tgtDev = DeviceList::findOne(['DevID' => $tgt['device']]);
            if(is_null($tgtDev))
                return $this->response(Error::STATUS_TGT_FOLDER_NOT_EXISTS);
            $tgtModel = $tgtDev->getParent();
            if(is_null($tgtModel))
                return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
            $tgtClassId = $tgtModel->ClassID;
        } else{
            return $this->response(Error::STATUS_PARAM_ERROR);
        }
        try{
            if(isset($src['classes']) && !empty($src['classes'])){
                $str = trim($src['classes']);
                if(!preg_match('/^([0-9])|([0-9][0-9 ,]*[0-9])$/', $str))
                    return $this->response(Error::STATUS_PARAM_ERROR);
                Yii::$app->getDb()
                    ->createCommand("update DevClasses set Parent=" . $tgtClassId . " where ClassID in ($str);")
                    ->execute();
            }
            if(isset($src['devices']) && !empty($src['devices']))
            {
                $str = trim($src['devices']);
                if(!preg_match('/^[0-9][0-9 ,]*[0-9]$/', $str))
                    return $this->response(Error::STATUS_PARAM_ERROR);
                Yii::$app->getDb()
                    ->createCommand("update DeviceList set Parent=" . $tgtClassId . " where DevID in ($str);")
                    ->execute();
            }
            return $this->response(Error::STATUS_SUCCESS);
        }catch (\Exception $e){
            return $this->response(Error::STATUS_UNKNOW, $e);}
    }
}
