<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/1/12 14:37
 */
namespace api\controllers;

use common\classes\Common;
use common\classes\MysqlAccess;
use common\models\BaseInfo;
use common\models\DeviceConfig;
use common\models\DeviceList;
use common\models\DeviceStatus;
use common\models\PwrDisplaySet;
use common\models\Timetable;
use common\models\UserDevices;
use common\models\Users;
use common\models\VDevClasses;
use common\models\VDevice;
use Yii;
use yii\db\Exception;
use yii\db\Query;
use yii\filters\VerbFilter;
use common\classes\Error;

/**
 * Site controller
 */
class DeviceController extends BaseController
{
    public $configAction = ['set-config', 'delete'];
    public function beforeAction($action)
    {
        $ret = parent::beforeAction($action); // TODO: Change the autogenerated stub
        if (!$ret) {
            return $ret;
        }
        if (!$this->isSuper && in_array($this->action->id, $this->configAction) && !in_array('dconfig', $this->userinfo['pri'])) {
            Yii::$app->response->data = $this->response(Error::STATUS_PRIVILEGE_ERROR);
            return false;
        }
        return $ret;
    }

    public static function userDevices($uid, $pri) {
        return UserDevices::find()
            ->select(['DevID', 'Pri'])
            ->asArray()
            ->where(['UserID' => $uid])
            ->indexBy('DevID')
            ->andWhere("Pri & $pri = $pri")
            ->all();
    }
    /**
     * get user's mgr devices(db Query)
     * @return $this|null|\yii\db\ActiveQuery
     */
    private function newQuery(){
        $user = Yii::$app->user->identity;
        if($user->Mgr_Privilege == 1 || $this->isSuper) {
            $query = VDevice::find();
        }
        else {
            $devices = self::userDevices($user->User_ID, 2);
            $ids = [];
            foreach ($devices as $dev)
                $ids[] = $dev['DevID'];
            $query = VDevice::find()->where(['in', 'id', $ids]);
        }
        return $query->orderBy(['power'=>SORT_DESC, 'convert(name USING gbk)'=>SORT_ASC]);
    }

    private function mgrPriCheck($id){
        $user = Yii::$app->user->identity;
        if($user->Mgr_Privilege == 1 || $this->isSuper) {
            return true;
        }
        else {
            $count = UserDevices::find()
                ->where(['UserID' => $user->User_ID, 'DevID' => $id])
                ->andWhere('Pri & 2 = 2')
                ->count();
            return ($count == 1);
        }
    }
    /*
     * get All Device Class
     */
    public function actionGetConfig(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['id']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $config = DeviceConfig::readConfig($data['id']);
        if(is_null($config))
            return $this->response(Error::STATUS_FAILED);
        else
            return $this->response(Error::STATUS_SUCCESS, $config);
    }

    public function actionSetConfig(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || empty($data['id']) || !preg_match('/^[0-9][0-9]+[0-9]$/', $data['id']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $id = $data['id'];

        $rtspUrl = isset($data['rtspUrl']) ? $data['rtspUrl'] : null;
        $rtspSub = isset($data['rtspSub']) ? $data['rtspSub'] : null;
        $rtspUrl1 = isset($data['rtspUrl1']) ? $data['rtspUrl1'] : null;
        $rtspSub1 = isset($data['rtspSub1']) ? $data['rtspSub1'] : null;

        // tm03
        if (substr($id, 0, 1) == '5') {
            $vdevice = VDevice::findOne(['id' => $id]);
            $device = DeviceList::findOne(['DevID' => $id]);
            $device->Dev_ConfigurationV2 = json_encode($data['configrationV2'], JSON_UNESCAPED_UNICODE);
            $name = $data['name'];
            $location = $data['location'];
            $device->Dev_Name = $name;
            $device->Dev_Description = iconv('UTF-8', 'GBK', $location);
            $device->ProjUsedTime = $data['ProjUsedTime'];
            $device->ProjWatermark = $data['ProjWatermark'];

            if (empty($rtspUrl))
                $device->RtspUrl = null;
            else if (!preg_match('/^rtsp:\/\/\*\*\*:\*\*\*@/i', $rtspUrl))
                $device->RtspUrl = $rtspUrl;
            if (empty($rtspSub))
                $device->RtspSub = null;
            else if (!preg_match('/^rtsp:\/\/\*\*\*:\*\*\*@/i', $rtspSub))
                $device->RtspSub = $rtspSub;
            if (empty($rtspUrl1))
                $device->RtspUrl1 = null;
            else if (!preg_match('/^rtsp:\/\/\*\*\*:\*\*\*@/i', $rtspUrl1))
                $device->RtspUrl1 = $rtspUrl1;
            if (empty($rtspSub1))
                $device->RtspSub1 = null;
            else if (!preg_match('/^rtsp:\/\/\*\*\*:\*\*\*@/i', $rtspSub1))
                $device->RtspSub1 = $rtspSub1;

            if(array_key_exists('pcMainPasswd', $data) && $data['pcMainPasswd'] !== '***')
                $device->PcMainPasswd = $data['pcMainPasswd'];
            if(array_key_exists('pcViewPasswd', $data) && $data['pcViewPasswd'] !== '***')
                $device->PcViewPasswd = $data['pcViewPasswd'];
            if(array_key_exists('pcIp', $data))
                $device->PcIp = $data['pcIp'];
            if(array_key_exists('pcPort', $data))
                $device->PcPort = $data['pcPort'];
            $tmSql = "call sp_send_dev_cfg_cmd('$id', '$vdevice->ipaddress', '$device->Dev_ConfigurationV2', '$name', '$location')";
            Yii::$app->getDb()->createCommand($tmSql)->execute();
            $devices = Yii::$app->db->createCommand("select Dev_Name from DeviceList where DevID = '$id'")
                ->queryAll();

            $names = [];
            foreach ($devices as $dev) {
                $names[] = $dev['Dev_Name'];
            }
            $this->logDetail['ele/id'] = implode("\n", $names);
            $title1 = implode(",", $names);
            if(mb_strlen($title1) > 64)
                $title1 = mb_substr($title1, 0, 64) . "...";
            $this->logTitle['title1'] = $title1;
            if ($device->save()) {
                return $this->response(Error::STATUS_SUCCESS);
            }
            return $this->response(Error::STATUS_FAILED);
        }

        $config = new DeviceConfig();
        if($config->load(Yii::$app->getRequest()->post(), 'data') && $config->save()) {
            $devices = Yii::$app->db->createCommand('select Dev_Name from DeviceList where DevID in ( ' . $data['id'] .')')
                ->queryAll();
            $names = [];
            foreach ($devices as $dev) {
                $names[] = $dev['Dev_Name'];
            }
            $this->logDetail['ele/id'] = implode("\n", $names);
            $title1 = implode(",", $names);
            if(mb_strlen($title1) > 64)
                $title1 = mb_substr($title1, 0, 64) . "...";
            $this->logTitle['title1'] = $title1;
            return $this->response(Error::STATUS_SUCCESS);
        }
        else
            return $this->response(Error::STATUS_FAILED);
    }

    public function actionList(){
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        try{
            $query = $this->newQuery();
            if($query == null)
                return $this->response(Error::STATUS_SUCCESS, []);
            $query->select(['id', 'name', 'parent'])->orderBy(['Parent' => SORT_ASC]);
            if (!is_null($data)) {
                if (isset($data['class']) && is_numeric($data['class'])) {
                    if (isset($data['recursion']) && $data['recursion'] === true) {
                        $query->andWhere('sp_dev_belong_group(id, ' . $data['class'] . ')');
                    } else {
                        $query->andWhere(['parent' => $data['class']]);
                    }
                } else if (!isset($data['recursion']) || $data['recursion'] !== true) {
                    $query->andWhere(['parent' => null]);
                }
            }
            $devices = $query->all();
            return $this->response(Error::STATUS_SUCCESS, $devices);
        }catch(Exception $e){
            return $this->response(Error::STATUS_UNKNOW);
        }
    }

    public function actionStatus(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $query = $this->newQuery();
        if($query == null)
            return $this->response(Error::STATUS_SUCCESS, []);
        $this->listQuery($query, $data);
        if(isset($data['devices']) && preg_match('/^[0-9][0-9,]+[0-9]$/', $data['devices']))
            $query->andWhere(['in', 'id', explode(',', $data['devices'])]);
        else if(isset($data['class']) && is_numeric($data['class'])) {
            if(isset($data['recursion']) && $data['recursion'] === true){
                $query->andWhere('sp_dev_belong_group(id, ' . $data['class'].')');
            }
            else {
                $query->andWhere(['parent' => $data['class']]);
            }
        }
        else if(!isset($data['recursion']) || $data['recursion'] !== true){
            $query->andWhere(['parent' => NULL]);
        }
        $devices = $query->all();
        if (isset($data['iotList']) && $data['iotList'] == true) {
            if (count($devices) == 1) {
                $dev = $devices[0];
                if (empty($dev['ipaddress']))
                    return $this->response(Error::STATUS_DEVICE_OFFLINE);
                $ret = [];
                $ret['iotStatus'] = empty($dev['iotStatus']) ? '' : $dev['iotStatus'];
                $ret['iots'] = PwrDisplaySet::find()->where(['>', 'PwrID', '1'])->orderBy(['PwrID' => SORT_ASC])->all();
                $ret['mode'] = BaseInfo::find()->where(['in', 'infomationID', [1,2,3,4]])->all();

                $canDisable = true;
                do {
                    $paramUnionid = Yii::$app->getRequest()->get('unionid');
                    $hashUnionid = hash('sha256', Common::UNIONID_MD5_PRESTR . $paramUnionid);
                    $user = Users::findOne(['UnionId' => $hashUnionid]);
                    if (empty($user)) {
                        return $this->response(Error::STATUS_USER_NOT_EXISTS);
                    }
                    if ($user->User_ID == 0)
                        break;
                    $isManagerFlag = !is_null($user->User_Name) && !is_null($user->Pass_Word);
                    $isSuperFlag = $isManagerFlag && !is_null($user->privilege) && ($user->privilege & 0x40000000);
                    if ($isSuperFlag)
                        break;
                    if ($isManagerFlag && ($user->privilege & 0x01)) {
                        if ($user->Mgr_Privilege == 1)
                            break;
                        $userDev = UserDevices::findOne(['UserID' => $user->User_ID, 'DevID' => $dev['id']]);
                        if (!empty($userDev) && ($userDev->Pri & 2) == 2)
                            break;
                    }
                    $canDisable = false;
                } while (false);

                $ret['canDisable'] = $canDisable;
                return $this->response(Error::STATUS_SUCCESS, $ret);
            }
        }
        return $this->response(Error::STATUS_SUCCESS, $devices);
    }

    private function doCtrl ($data, $uid, $devname = null) {
        $vdevice = new VDevice();
        $vdevice->uid = $uid;
        do {
            if ($vdevice->load(['data' => $data], 'data') && !empty($vdevice->devices)) {
                $devs = Yii::$app->db->createCommand("select Dev_Name from DeviceList where DevID in(" . $vdevice->devices . ");")
                    ->queryAll();
                if(count($devs) == 0)
                    break;
                if (!$vdevice->save())
                    break;
                $names = [];
                foreach ($devs as $dev) {
                    $names[] = $dev['Dev_Name'];
                }
                $this->logDetail['ele/id'] = implode("\n", $names);
                $title1 = implode(",", $names);
                if(mb_strlen($title1) > 64)
                    $title1 = mb_substr($title1, 0, 64) . "...";
                $this->logTitle['title1'] = $title1;
                if (isset($vdevice->power))
                    $this->logDetail['ele/power'] = $vdevice->power;
                if (isset($vdevice->pc))
                    $this->logDetail['ele/pc'] = $vdevice->pc;
                if (isset($vdevice->projector))
                    $this->logDetail['ele/projector'] = $vdevice->projector;
                if (isset($vdevice->audioMute))
                    $this->logDetail['ele/audioMute'] = $vdevice->audioMute;
                if (isset($vdevice->micMute))
                    $this->logDetail['ele/micMute'] = $vdevice->micMute;
                if (isset($vdevice->micVol) && is_numeric($vdevice->micVol))
                    $this->logDetail['ele/micVol'] = $vdevice->micVol;
                if (isset($vdevice->audioVol) && is_numeric($vdevice->audioVol))
                    $this->logDetail['ele/audioVol'] = $vdevice->audioVol;
                if (isset($vdevice->projectorSrc))
                    $this->logDetail['ele/projectorSrc'] = $vdevice->projectorSrc;
                if (isset($vdevice->iotStatus) && strlen($vdevice->iotStatus) == 32 && ctype_xdigit($vdevice->iotStatus))
                    $this->logDetail['ele/iotStatus'] = $vdevice->iotStatus;
                return $this->response(Error::STATUS_SUCCESS, $devname);
            }
        }while(false);
        $this->logSended = true;
        return $this->response(Error::STATUS_FAILED, $devname);
    }

    public function actionCloudControl(){
        // Cloud channel
        if (!Yii::$app->getUser()->identity->cloudLogin) {
            return $this->response(Error::STATUS_FAILED);
        }
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['devices']) || !preg_match('/^[0-9][0-9,]+[0-9]$/', $data['devices']))
            return $this->response(Error::STATUS_PARAM_ERROR, $data);

        $device = VDevice::find()->select(['id', 'name', 'power'])->where(['id' => $data['devices']])->one();
        $devname = empty($device) ? $data['device'] : $device->name;
        // Find user
        $user = null;
        if (isset($data['userid'])) {
            $user = Users::findOne(['User_ID' => $data['userid']]);
        }
        if (empty($user)) {
            return $this->response(Error::STATUS_USER_NOT_EXISTS, ['data' => $data['userid']]);
        }

        $this->logTitle['username'] = empty($user->User_Name) ? $user->User_Desc : $user->User_Name;
        $this->logTitle['title1'] = $devname;
        $userDevice = UserDevices::findOne(['UserID' => $user->User_ID, 'DevID' => $device->id]);

        $pcNprjOp = (array_key_exists('pc', $data) || array_key_exists('projector', $data));
        if ($device->power === 'off' && $pcNprjOp) {
            return $this->response(Error::STATUS_DEVICE_POWER_OFF);
        }
        if ($device->power === 'disable' && $pcNprjOp) {
            return $this->response(Error::STATUS_DEVICE_IS_DISABLED);
        }

        // admin & super
        if ($user->User_ID === 0 || ($user->privilege & 0x40000000)) {
            return $this->doCtrl($data, $user->User_ID, $devname);
        }
        // Mgr user
        if ((($user->privilege & 0x01) && !empty($user->User_Name))) {
            if ($user->Mgr_Privilege == 1) {
                return $this->doCtrl($data, $user->User_ID, $devname);
            }
            if (!empty($userDevice) && ($userDevice->Pri & 2) == 2) {
                return $this->doCtrl($data, $user->User_ID, $devname);
            }
        }
        if ($device->power === 'disable') {
            return $this->response(Error::STATUS_DEVICE_IS_DISABLED);
        }
        if ($user->Dev_Privilege == 1) {
            return $this->doCtrl($data, $user->User_ID, $devname);
        }
        if (!empty($userDevice) && ($userDevice->Pri & 1) == 1) {
            return $this->doCtrl($data, $user->User_ID, $devname);
        }

        // Timetable
        $timeTables = Timetable::find()->where(['UserId' => $user->User_ID, 'DevId' => $device->id])
            ->andWhere(['or',
                "(Type = 'normal' and WeekDay = weekday(now()) and time(now()) < time(End) and time(subtime(Start, '00:30:00')) < time(now()))",
                "(Type = 'temporary' AND NOW() < End AND NOW() > SUBTIME(Start, '00:30:00'))"])->count();
        if ($timeTables > 0) {
            return $this->doCtrl($data, $user->User_ID, $devname);
        } else
            return $this->response(Error::STATUS_PRIVILEGE_ERROR, $devname);
        $this->logSended = 1;
        return $this->response(Error::STATUS_FAILED, $devname);
    }


    public function actionControl(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data))
            return $this->response(Error::STATUS_PARAM_ERROR, $data);
        if (isset($data['devices'])){
            if (!preg_match('/^[0-9][0-9,]+[0-9]$/', $data['devices'])) {
                return $this->response(Error::STATUS_PARAM_ERROR, $data);
            }
        } else if (isset($data['allDevice'])) {
            $query = $this->newQuery()->select(['id']);
            $list = $query->all();
            $ids = [];
            foreach ($list as $id) {
                $ids[] = $id->id;
            }
            if (empty($ids))
                return $this->response(Error::STATUS_FAILED);
            $result = [];
            if (isset($data['key']) && !is_null($data['key']['parent'])) {
                $result = $this->findAreaDevice($ids, $data['key']['parent'], $result, true);
            } else {
                $result = $ids;
            }
            $data['devices'] = implode(',', $result);
        } else {
            return $this->response(Error::STATUS_PARAM_ERROR, $data);
        }
        $this->logDetail['ele/id'] = '';
        $device = new VDevice();
        $ret = null;
        if (Yii::$app->getUser()->identity->cloudLogin) {
            $device = VDevice::findOne(['id' => $data['devices']]);
            isset($data['username']) && ($this->logTitle['username'] = $data['username']);
            $ret['name'] = empty($device) ? $data['devices'] : $device->name;
        }
        return $this->doCtrl($data, $this->userinfo['id']);
    }

    /**
     * @param $ids array 用户的管理设备id数组;
     * @param $areaId number 需要查找的分区id;
     * @param $result array 需要返回的查找结果;
     * @param $rootFlag boolean 是否需要获取自身的设备;
     * @return array
     */
    public function findAreaDevice ($ids, $areaId, $result, $rootFlag = false) {
        $query = VDevClasses::find();
        if ($rootFlag) {
            // 获取自己的设备
            $selfDevs = $query->where(['ClassID' => $areaId])->all();
            foreach ($selfDevs as $dev) {
                if (!is_null($dev->DevID)) {
                    if (in_array($dev->DevID, $ids))
                        $result[] = $dev->DevID;
                }
            }
        }
        $areaQuery = $query->where(['Parent' => $areaId]);
        $devArr = $areaQuery->all(); //设备
        foreach ($devArr as $dev) {
            // 分区下的设备
            if (!is_null($dev->DevID)) {
                if (in_array($dev->DevID, $ids))
                    $result[] = $dev->DevID;
            }
        }
        $areaArr = $areaQuery->all();
        foreach ($areaArr as $area) {
            // 分区下的分区,递归
            $result = $this->findAreaDevice($ids, $area->ClassID, $result);
        }
        return $result;
    }

    public function actionBrdMute($id, $mute) {
        $query = $this->newQuery();
        if($query == null)
            return $this->response(Error::STATUS_DEVICE_NOT_EXISTS);
        $ids = preg_split('/,/', $id);
        if(empty($ids))
            return $this->response(Error::STATUS_SUCCESS);
        foreach ($ids as $id) {
            $access = new MysqlAccess($id);
            if(is_null($access))
                continue;
            if ($mute == 'on')
                $access->deviceCtrl(MysqlAccess::CMD_BRD_MUTE, 0);
            else
                $access->deviceCtrl(MysqlAccess::CMD_BRD_MUTE, 1);
        }
        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionTasks(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['id']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        if(!$this->mgrPriCheck($data['id']))
            return $this->response(Error::STATUS_FORBID);
        if(isset($data['date'])){
            $start = $data['date'] . ' 00:00:00';
            $end = $data['date'] . ' 23:59:59';
        }
        else{
            $start = date('Y-m-d 00:00:00');
            $end = date('Y-m-d 23:59:59');
        }
        try{
            $ret = Yii::$app->getDb()
                ->createCommand("call sp_get_dev_scheduled_tasks('" . $data['id'] . "', '$start', '$end');")
                ->queryAll();
            return $this->response(Error::STATUS_SUCCESS, $ret);
        }catch (\Exception $e){
            return $this->response(Error::STATUS_FAILED, $e);}
    }

    public function actionCameraList($id){
        $model = DeviceList::find()->select(['DevID', 'Dev_Name', 'RtspUrl', 'RtspSub', 'RtspUrl1', 'RtspSub1'])
            ->where(['DevID' => $id])
            ->one();
        return $this->response(Error::STATUS_SUCCESS, $model);
    }

    private function deviceDelete($id){
        Yii::$app->getDb()
            ->createCommand("call sp_delete_device('" . $id . "');")
            ->execute();
        return VDevice::find()->where(['id' => $id])->count();
    }
    /*
     * delete offline device
     */
    public function actionDelete(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['id']) ||
            !preg_match('/^[0-9][0-9,]+[0-9]$/', $data['id']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $success = [];
        $names = [];
        try{
            $ids = explode(',', $data['id']);
            foreach($ids as $id){
                if(!$this->mgrPriCheck($id))
                    continue;
                $device = VDevice::findOne(['id' => $id]);
                if(is_null($device))
                    continue;
                if(is_null($device->ipaddress) && $this->deviceDelete($id) == 0)
                    $names[] = empty($device->name) ? $device->description: $device->name;
                    $success[] = $id;
            }
        }catch (\Exception $e){
            return $this->response(Error::STATUS_FAILED);
        }

        $this->logDetail['ele/id'] = implode("\n", $names);
        $title1 = implode(",", $names);
        if(mb_strlen($title1) > 64)
            $title1 = mb_substr($title1, 0, 64) . "...";
        $this->logTitle['title1'] = $title1;
        return $this->response(Error::STATUS_SUCCESS, ['devices' => $success]);
    }

    /**
     * device (config/name+class/task+mode/privilege) copy
     */
    public function actionCopy(){
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data)
            || !isset($data['src']) || !preg_match('/^[0-9]+[0-9]$/', $data['src'])
            || !isset($data['dst'])  || !preg_match('/^[0-9][0-9,]+[0-9]$/', $data['dst'])
            || !isset($data['mask']) || !is_numeric($data['mask']))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $this->logDetail['ele/src'] = $data['src'];
        $this->logDetail['ele/dst'] = $data['dst'];
        if(!$this->mgrPriCheck($data['src']))
            return $this->response(Error::STATUS_FORBID);
        $srcDevice = VDevice::findOne(['id' => $data['src']]);
        if(is_null($srcDevice))
            return $this->response(Error::STATUS_DEVICE_NOT_EXISTS);
        $this->logTitle['title1'] = $srcDevice->name;
        if (($data['mask'] & 0x01) == 0)
            $data['mask'] &= 0xf5;
        $device = explode(',', $data['dst']);
        if($device === false || count($device) == 0)
            return $this->response(Error::STATUS_SUCCESS);
        $sql = "select func_dulicate_device('" . $data['src'] . "', :dst, :mask) as ret;";
        $cmd = Yii::$app->getDb()->createCommand($sql);
        $cmd = $cmd->bindParam(':dst', $dst1);
        $cmd = $cmd->bindParam(':mask', $mask);
        $success = [];
        foreach ($device as $dst) {
            if($dst == $data['src'])
                continue;
            if(!$this->mgrPriCheck($dst))
                continue;
            $dst1 = $dst;
            //Clear bit2, to disable task data copy
            if(substr($data['src'], 0, 1) != substr($dst, 0, 1))
                $mask = $data['mask'] & 0xfb;
            else
                $mask = $data['mask'];
            try {
                $out = $cmd->queryOne();
            } catch (\Exception $e) {
                continue;
            }
            if($out['ret'] !== "0")
                $success[] = $dst;
        }
        //offline and select all bit(0x0f), delete src device
        if (is_null($srcDevice->ipaddress) && ($data['mask'] & 0x1f) == 0x1f && count($success) >= 1)
            $this->deviceDelete($data['src']);
        return $this->response(Error::STATUS_SUCCESS, $success);
    }

    public function actionPublicList () {
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        try{
            $query = $this->newQuery();
            if($query == null)
                return $this->response(Error::STATUS_SUCCESS, []);
            if (!is_null($data)) {
                $this->listQuery($query, $data);
                if (isset($data['class']) && is_numeric($data['class'])) {
                    if (isset($data['recursion']) && $data['recursion'] === true) {
                        $query->andWhere('sp_dev_belong_group(id, ' . $data['class'] . ')');
                    } else {
                        $query->andWhere(['parent' => $data['class']]);
                    }
                } else if (!isset($data['recursion']) || $data['recursion'] !== true) {
                    $query->andWhere(['parent' => null]);
                }
            }
            $devices = $query->all();
            return $this->response(Error::STATUS_SUCCESS, $devices);
        }catch(Exception $e){
            return $this->response(Error::STATUS_UNKNOW);
        }
    }

    public function actionGetFirmwareVersion () {
        $data = Yii::$app->getRequest()->post('data');
        if (empty($data) || empty($data['id']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $device = VDevice::findOne(['id' => $data['id']]);
        if(empty($device))
            return $this->response(Error::STATUS_DEVICE_NOT_EXISTS);
        if(empty($device->ipaddress))
            return $this->response(Error::STATUS_DEVICE_OFFLINE);
        try {
            Yii::$app->getDb()
                ->createCommand("call sp_send_dev_ctrl_cmd('$device->id', '$device->ipaddress', 112, 0, 0, 0, null);")->execute();
            sleep(2);
            $device = VDevice::find()->select(['id', 'name', 'version'])->where(['id' => $device->id])->one();
            if (empty($device))
                return $this->response(Error::STATUS_DEVICE_NOT_EXISTS);
            return $this->response(Error::STATUS_SUCCESS, $device);
        }catch (\Exception $e) {
            return $this->response(Error::STATUS_UNKNOW, $e);
        }
    }

    public function actionFirmwareList() {
        $content = file_get_contents(Common::FIRMWARE_LIST_FILE);
        if(empty($content))
            return $this->response(Error::STATUS_FIRMWARE_LIST_EMPTY);
        $content = unserialize($content);
        if(empty($content) || !is_array($content))
            return $this->response(Error::STATUS_FIRMWARE_LIST_EMPTY);
        return $this->response(Error::STATUS_SUCCESS, $content);
    }

    public function actionFirmwareUpgrade() {
        $data = Yii::$app->request->post('data');
        if (empty($data) || empty($data['id']) || empty($data['filename']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $file = Common::FIRMWARE_DIR . '/' . $data['filename'];
        if(!file_exists($file))
            return $this->response(Error::STATUS_FIRMWARE_NOT_EXISTS);
        $file = '/firmware/' . $data['filename'];
        $ids = explode(',', $data['id']);
        $notExists = [];
        $offline = [];
        foreach ($ids as $id) {
            $device = VDevice::findOne(['id' => $id]);
            if(empty($device)) {
                $notExists[] = [
                    'id' => $id
                ];
                continue;
            }
            if(empty($device->ipaddress)) {
                $offline[] = [
                    'id' => $device->id,
                    'name' => $device->name
                ];
                continue;
            }
            $cmd = "call sp_send_dev_ctrl_cmd('$device->id', '$device->ipaddress', 113, 0, 0, 0, '$file');";
            Yii::$app->getDb()
                ->createCommand($cmd)->execute();
        }
        return $this->response(Error::STATUS_SUCCESS, ['offline' => $offline, 'notExists' => $notExists]);
    }

    public function actionRemoteCtrl(){
        $data = Yii::$app->request->post('data');
        if(empty($data) || !isset($data['id']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $device = DeviceList::find()->select(['PcIp', "PcPort", "PcMainPasswd", "PcViewPasswd"])
            ->where(['DevID' => $data['id']])
            ->one();
        if(empty($device) || empty($device->PcIp) || empty($device->PcPort))
            return $this->response(Error::STATUS_DEVICE_RC_NOT_CONFIG);
        if(isset($data['type']) && $data['type'] == 'view') {
            if(empty($device->PcViewPasswd))
                return $this->response(Error::STATUS_DEVICE_RC_VIEW_PWD_NOT_CONFIG);
            $str = $device->PcIp . "\n" . $device->PcPort . "\n" . $device->PcViewPasswd;
        } else {
            if(empty($device->PcMainPasswd))
                return $this->response(Error::STATUS_DEVICE_RC_VIEW_PWD_NOT_CONFIG);
            $str = $device->PcIp . "\n" . $device->PcPort . "\n" . $device->PcMainPasswd;
        }
        $credit = Yii::$app->security->generateRandomString(16);
        Yii::$app->cache->set(Common::PC_REMOTE_CTRL_SALT . $credit, $str, 60000);
        return $this->response(Error::STATUS_SUCCESS, ['credit' => $credit]);
    }
}
