<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/5/17 17:57
 */
namespace api\controllers;

use common\classes\Common;
use common\classes\Error;
use common\models\CallWaiter;
use common\models\DeviceList;
use common\models\DeviceStatus;
use common\models\IpCall;
use common\models\VDevice;
use common\models\VIpCall;
use Yii;

/**
 * IP Call controller
 */
class IpcallController extends BaseController
{
    public function actionOnhook($id){
        $token = Yii::$app->request->get('access_token');
        $model = IpCall::findOne(['id' => $id]);
        if(empty($model))
            return $this->response(Error::STATUS_IPCALL_NOT_EXISTS);
        $waiter = CallWaiter::findOne(['user' => Yii::$app->user->id, 'session' => $token]);
        if(empty($waiter) || (!is_null($model->waiter) && $model->waiter != $waiter->id))
            return $this->response(Error::STATUS_SUCCESS);
        $model->status = 'idle';
        if($model->save()) {
            return $this->response(Error::STATUS_SUCCESS);
        }
        return $this->response(Error::STATUS_FAILED);
    }

    public function actionOffhook($id){
        $token = Yii::$app->request->get('access_token');
        $model = IpCall::findOne(['id' => $id]);
        if(empty($model))
            return $this->response(Error::STATUS_IPCALL_NOT_EXISTS);
        if(!is_null($model->waiter))
            return $this->response(Error::STATUS_IPCALL_IS_TALKING);
        $waiter = CallWaiter::findOne(['user' => Yii::$app->user->id, 'session' => $token]);
        if(empty($waiter)) {
            $waiter = new CallWaiter();
            $waiter->user = Yii::$app->user->id;
            $waiter->session = $token;
            $waiter->save();
        }
        $model->waiter = $waiter->id;
        if($model->save())
            return $this->response(Error::STATUS_SUCCESS);
        else
            return $this->response(Error::STATUS_FAILED);
    }

    public function actionCall($deviceid){
        if(DeviceList::find()->where(['DevID' => $deviceid])->count() == 0)
            return $this->response(Error::STATUS_DEVICE_NOT_EXISTS);
        $status = VDevice::findOne(['id' => $deviceid]);
        if(empty($status->ipaddress))
            return $this->response(Error::STATUS_DEVICE_OFFLINE);
        if($status->ipcall == 'offline')
            return $this->response(Error::STATUS_DEVICE_IPCALL_NOT_EXISTS);
        if($status->ipcall != 'idle')
            return $this->response(Error::STATUS_IPCALL_IS_TALKING);

        $listen = Yii::$app->request->get('listen');

        $token = Yii::$app->request->get('access_token');
        $waiter = CallWaiter::findOne(['session' => $token]);
        if(empty($waiter)) {
            $waiter = new CallWaiter();
            $waiter->user = Yii::$app->user->id;
            $waiter->session = $token;
            $waiter->save();
        }
        $count = IpCall::find()
            ->where(['waiter' => $waiter->id])
            ->andWhere(['not', ['status' => 'idle']])
            ->count();
        if($count > 0)
            return $this->response(Error::STATUS_IPCALL_BUSY);
        $queryRet = null;
        if($listen == "true")
            $queryRet = Yii::$app->db->createCommand("select func_ipcall_request('$deviceid', 4) as ret;")
                ->queryOne();
        else
            $queryRet = Yii::$app->db->createCommand("select func_ipcall_request('$deviceid', 2) as ret;")
                ->queryOne();
        if(empty($queryRet) || $queryRet['ret'] == 0)
            return $this->response(Error::STATUS_IPCALL_BUSY);
        $model = IpCall::findOne(['device' => $deviceid, 'status' => 'ringing', 'waiter' => null]);
        if(!empty($model)) {
            $model->waiter = $waiter->id;
            $model->direction = 'b2s';
            if($model->save())
                return $this->response(Error::STATUS_SUCCESS);
        }
        return $this->response(Error::STATUS_FAILED);
    }

    public function actionListen() {
        $token = Yii::$app->request->get('access_token');
        Yii::$app->db
            ->createCommand("insert into CallWaiter (user, session) values(" . Yii::$app->user->id . ", '$token') on duplicate key update updateAt = now()")
            ->execute();
        $waiter = CallWaiter::findOne(['user' => Yii::$app->user->id, 'session' => $token]);
        if(empty($waiter))
            return $this->response(Error::STATUS_FAILED);
        $call = VIpCall::find()
            ->select(['id', 'device', 'deviceName', 'waiter', 'waiterName', 'ringing', 'status', 'direction', 'wsPort'])
            ->where("(`waiter` is null and `status`='ringing') or (`waiter`=" . $waiter->id . " and `status` <> 'idle')")
            ->orderBy(['waiter' => SORT_DESC])
            ->asArray()
            ->one();
        return $this->response(Error::STATUS_SUCCESS, [$call]);
    }
}
