<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/5/2 15:34
 */
namespace api\controllers;
use common\classes\Common;
use common\classes\Error;
use common\models\VDeviceLog;
use Yii;

/**
 * Site controller
 */
class LogController extends BaseController
{
    public function actionList($end){
        if(!is_numeric($end))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $startDate = Yii::$app->getRequest()->get('startDate');
        if(empty($startDate) || strptime($startDate, '%Y-%m-%d') == false)
            $startDate = '1000-01-01';

        $endDate = Yii::$app->getRequest()->get('endDate');
        if(empty($endDate) || strptime($endDate, '%Y-%m-%d') == false)
            $endDate = '9999-12-31';

        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data)|| !isset($data['limit'])) {
            $data['limit'] = 200;
        }
        if(!isset($data['page'])) {
            $data['page'] = 0;
        }
        $query = VDeviceLog::find()
            ->andWhere("`DateTime` >= '$startDate 00:00:00' and `DateTime` < '$endDate 23:59:59' and (`EndTime` <= '$endDate 23:59:59' or `EndTime` is null)")
            ->andWhere("DevLogID <= $end");
        return $this->list($query, $data);
    }

    public function actionDownload(){
        $startDate = Yii::$app->getRequest()->get('startDate');
        if(empty($startDate) || strptime($startDate, '%Y-%m-%d') == false)
            $startDate = '1000-01-01';

        $endDate = Yii::$app->getRequest()->get('endDate');
        if(empty($endDate) || strptime($endDate, '%Y-%m-%d') == false)
            $endDate = '9999-12-31';

        $type = Yii::$app->getRequest()->get('type', 'device');
        if($type == 'user') {
            $query = VDeviceLog::find()
                ->select(['User_Sn', 'User_Name', 'DeviceName', 'DeviceID', 'CtrlUser', 'DateTime', 'EndTime', 'Length']);
            $query->andWhere(['not', ['CtrlUser' => null]]);
            $ret =  Yii::t('app/message', 'log/User_Sn') . ',' . Yii::t('app/message', 'log/User_Name') . ',' .
                Yii::t('app/message', 'log/DeviceName') . ',' .
                Yii::t('app/message', 'log/DeviceID') . ',' .Yii::t('app/message', 'log/DateTime') . ',' .
                Yii::t('app/message', 'log/EndTime') . ',' .Yii::t('app/message', 'log/Length') . "\r\n";
            $filename = "userlog-$startDate-$endDate.csv";
        }
        else {
            $query = VDeviceLog::find()
                ->select(['DeviceID', 'DeviceName', 'User_Name', 'CtrlUser','DateTime', 'EndTime', 'Length', 'Event']);
            $ret = Yii::t('app/message', 'log/DeviceID') . ',' .Yii::t('app/message', 'log/DeviceName') . ',' .
                Yii::t('app/message', 'log/User_Name') . ',' .Yii::t('app/message', 'log/DateTime') . ',' .
                Yii::t('app/message', 'log/EndTime') . ',' .Yii::t('app/message', 'log/Length') . ',' .
                Yii::t('app/message', 'log/Event') ."\r\n";
            $filename = "devicelog-$startDate-$endDate.csv";
        }
        $query->andWhere("`DateTime` >= '$startDate 00:00:00' and `DateTime` < '$endDate 23:59:59' and (`EndTime` <= '$endDate 23:59:59' or `EndTime` is null)");
        $event = Yii::$app->getRequest()->get('event');
        if(empty($event))
            $query->andWhere(['Event' => [1,2,3,4,5,6]]);
        else {
            $event = explode(',', $event);
            foreach ($event as $ev) {
                if(!is_numeric($ev))
                    return $this->response(Error::STATUS_PARAM_ERROR);

            }
            $query->andWhere(['Event' => $event]);
        }
        $filepath = Common::MEDIA_TEMPORARA_DIR . Yii::$app->security->generateRandomString(16). ".tmp";
        if(file_exists($filepath)) {
            unlink($filepath);
        }
        $fp = fopen($filepath, "a");
        fwrite($fp, $ret);

        $query->asArray();
        $count = $this->listQuery($query);
        for($i = 0; $i < $count / 50000; $i++) {
            $newQuery = $query;
            $newQuery->offset($i * 50000);
            if($count - $i * 50000 >= 50000)
                $newQuery->limit(50000);
            else
                $newQuery->limit($count - $i * 50000);
            $lists = $newQuery->all();
            $tmp = '';
            foreach ($lists as $list) {
                if(is_null($list['User_Name']) && !is_null($list['CtrlUser'])) {
                    $list['User_Name'] = 'UID:' . $list['CtrlUser'];
                }
                unset($list['CtrlUser']);
                if($type != 'user')
                    $list['Event'] = Yii::t('app/message', 'log/event_type_' . $list['Event']);
                $tmp .= implode(',', $list) . "\r\n";
            }
            fwrite($fp, $tmp);
        }
        fclose($fp);
        Yii::$app->response->sendFile($filepath, $filename,  ["mimeType" => "text/csv"]) && unlink($filepath);;
    }

    public function actionClean() {
        $startDate = Yii::$app->getRequest()->get('startDate');
        if(empty($startDate) || strptime($startDate, '%Y-%m-%d') == false)
            $startDate = '1000-01-01';

        $endDate = Yii::$app->getRequest()->get('endDate');
        if(empty($endDate) || strptime($endDate, '%Y-%m-%d') == false)
            $endDate = '9999-12-31';
        $this->logTitle['title1'] = $startDate;
        $this->logTitle['title2'] = $endDate;

        Yii::$app->db
            ->createCommand("Delete from `DeviceStatistic` where `StatisticDay` >= '$startDate' and `StatisticDay` <= '$endDate';")
            ->execute();
        $cnt = Yii::$app->db
            ->createCommand("DELETE a.* FROM DeviceLog a
                                    WHERE a.`DateTime` >= '$startDate 00:00:00'
                                    AND (a.`EndTime` < '$endDate 23:59:59'
                                        OR (a.`EndTime` IS NULL
                                            AND a.`DevLogID` NOT IN (SELECT
                                                *
                                            FROM
                                                (SELECT
                                                    DevLogID
                                                FROM
                                                    DeviceLog
                                                WHERE
                                                    `DateTime` >= '$endDate 23:59:59') AS tmp)))")
            ->execute();
        return $this->response(Error::STATUS_SUCCESS, ['rows' => $cnt]);
    }


    public function actionChartList(){
        $startDate = Yii::$app->getRequest()->get('startDate');
        if(empty($startDate) || strptime($startDate, '%Y-%m-%d') == false)
            $startDate = '1000-01-01';

        $endDate = Yii::$app->getRequest()->get('endDate');
        if(empty($endDate) || strptime($endDate, '%Y-%m-%d') == false)
            $endDate = '9999-12-31';

        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data)|| !isset($data['limit'])) {
            $data['limit'] = 200;
        }
        if(!isset($data['page'])) {
            $data['page'] = 0;
        }
        $lists = Yii::$app->db
            ->createCommand("SELECT COUNT(User_Name) as users, date_format(DateTime, '%Y-%m-%d') as `date`
                FROM `v_device_log`
                WHERE `DateTime` >= '$startDate 00:00:00' and
                   `DateTime` < '$endDate 23:59:59' and (`EndTime` <= '$endDate 23:59:59' or `EndTime` is null)
                GROUP BY date_format(`DateTime`, '%Y-%m-%d')
                ORDER BY `DateTime` asc;")->queryAll();
        if (empty($lists))
            return $this->response(Error::STATUS_FAILED);
        return $this->response(Error::STATUS_SUCCESS, $lists);
    }
}