<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/03/08 16:37
 */
namespace api\controllers;
use common\classes\Common;
use common\components\Media;
use common\models\MediaFiles;
use common\models\VMediaFiles;
use common\models\VSessionFiles;
use Yii;
use common\models\UploadFile;
use common\classes\Error;
use yii\db\Exception;
use yii\db\mssql\PDO;
use yii\filters\VerbFilter;
use yii\web\UploadedFile;

/**
 * Session controller
 */
class MediaController extends BaseController
{
    public function actionFolderList(){
        if($this->isAdmin)
            $subFolder = VMediaFiles::find()->select(['MediaID', 'FileName', 'Parent', 'Owner', 'Sysfolder'])
                ->where(['AudioCodec' => 0, 'VedioCodec' => 0])
                ->all();
        else {
            $media = VMediaFiles::find()
                ->select(['MediaID', 'FileName', 'Parent', 'Owner', 'Sysfolder'])
                ->where(['Owner' => Yii::$app->user->getId(), 'Sysfolder' => 1])
                ->one();
            if(empty($media)) {
                $media = new MediaFiles();
                $media->FileName = Yii::$app->user->getIdentity()->User_Desc;
                $media->Parent = 1;
                $media->Owner = Yii::$app->user->getId();
                $media->Sysfolder = 1;
                $media->save();
            }
            $subFolder = [$media];
            $parent = [$media->MediaID];
            while(true){
                $temp = VMediaFiles::find()
                    ->select(['MediaID', 'FileName', 'Parent'])
                    ->where(['VedioCodec' => 0, 'AudioCodec' => 0])
                    ->andWhere(['in', 'parent', $parent])
                    ->indexBy('MediaID')
                    ->all();
                if(empty($temp))
                    break;
                $parent = [];
                foreach ($temp as $index => $value) {
                    $subFolder[] = $value;
                    $parent[] = $index;
                }
            }
        }
        return $this->response(Error::STATUS_SUCCESS, $subFolder);
    }

    private function folderView($id) {
        return VMediaFiles::find()->select(['MediaID', 'FileName', 'Parent'])
            ->where(['AudioCodec' => 0, 'VedioCodec' => 0, 'MediaID' => $id])
            ->one();
    }

    public function actionFolderNew(){
        $data = Yii::$app->request->post('data');
        if(is_null($data) || !isset($data['FileName']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $folder = new MediaFiles();
        $folder->FileName = $data['FileName'];

        if(!isset($data['Parent']))
            $folder->Parent = $this->isAdmin ? null: Yii::$app->user->getId();
        else {
            $parentFolder = VMediaFiles::findOne(['MediaID' => $data['Parent']]);
            if(is_null($parentFolder))
                return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
            if($parentFolder->Sysfolder == 2)
                return $this->response(Error::STATUS_FORBID);
            $folder->Parent = $data['Parent'];
        }
        if(!$folder->save())
            return $this->response(Error::STATUS_FAILED);
        return $this->response(Error::STATUS_SUCCESS, $this->folderView($folder->MediaID));
    }

    public function actionFolderUpdate($id){
        $data = Yii::$app->request->post('data');
        if(is_null($data) || empty($data['FileName']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $folder = MediaFiles::findOne(['MediaID' => $id]);
        if(is_null($folder))
            return $this->response(Error::STATUS_FOLDER_NOT_EXISTS);
        if(isset($folder->Sysfolder))
            return $this->response(Error::STATUS_FORBID);
        $folder->FileName = $data['FileName'];
        if(!isset($folder->owner)) {
            if (!isset($data['Parent']))
                $folder->Parent = $this->isAdmin ? null : Yii::$app->user->getId();
            else {
                $parentFolder = VMediaFiles::findOne(['MediaID' => $data['Parent']]);
                if(is_null($parentFolder))
                    return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
                if($parentFolder->Sysfolder == 2)
                    return $this->response(Error::STATUS_FORBID);
                $folder->Parent = $data['Parent'];
            }
        }
        if(!$folder->save())
            return $this->response(Error::STATUS_FAILED);
        return $this->response(Error::STATUS_SUCCESS, $this->folderView($folder->MediaID));
    }

    public function actionLite($type, $folder){
        $data = Yii::$app->request->post('data');
        $query = VMediaFiles::find()
            ->select(['MediaID', 'FileName', 'TotalTime','AudioCodec', 'VedioCodec'])
            ->where(['or', ['Sysfolder' => null], 'Sysfolder >= -1']);
        if($type == 'audio')
            $query = $query->andWhere(['VedioCodec' => -1, 'AudioCodec' => [0,1,2,3]]);
        else if($type == 'video')
            $query = $query->andWhere(['or', ['VedioCodec' => [0,1,2], 'AudioCodec' =>[-1,1,2,3]], ['VedioCodec' => -1, 'AudioCodec' => -1]]);
        else {
            $query = $query->andWhere(['or', ['VedioCodec' => -1, 'AudioCodec' => [-1,0,1,2,3]],['VedioCodec' => [0,1,2], 'AudioCodec' =>[-1,1,2,3]]]);
        }
        if($folder == "true")
            $query = $query->orWhere(['VedioCodec' => 0, 'AudioCodec' => 0]);
        return $this->list($query, $data);
    }

    public function actionList($type, $folder){
        $data = Yii::$app->request->post('data');
        $query = VMediaFiles::find()
            ->where(['or', ['Sysfolder' => null], 'Sysfolder >= 0']);
        if($type == 'audio')
            $query = $query->andWhere(['VedioCodec' => -1, 'AudioCodec' => [0,1,2,3]]);
        else if($type == 'video')
            $query = $query->andWhere(['or', ['VedioCodec' => [0,1,2], 'AudioCodec' =>[-1,1,2,3]], ['VedioCodec' => -1, 'AudioCodec' => -1]]);
        else {
            $query = $query->andWhere(['or', ['VedioCodec' => -1, 'AudioCodec' => [-1,0,1,2,3]],['VedioCodec' => [0,1,2], 'AudioCodec' =>[-1,1,2,3]]]);
        }
        if($folder == "true")
            $query = $query->orWhere(['VedioCodec' => 0, 'AudioCodec' => 0]);
        return $this->list($query, $data);
    }

    /**
     * get file detail info
     * @param $id file id
     * @return array
     */
    public function actionView($id){
        $file = MediaFiles::findOne(['MediaID' => $id]);
        if(is_null($file))
            return $this->response(Error::STATUS_MEDIA_FILE_NOT_EXISTS);
        $sessions = VSessionFiles::find()
            ->select(['SessionName'])
            ->where(['FileID' => $file->MediaID])
            ->all();
        return $this->response(Error::STATUS_SUCCESS, [
            'file' => $file,
            'sessions' => $sessions
        ]);
    }

    /**
     * @param $id
     * @return array
     */
    public function actionUpdate($id){
        $data = Yii::$app->request->post('data');
        if(is_null($data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $file = MediaFiles::findOne(['MediaID' => $id]);
        if(is_null($file))
            return $this->response(Error::STATUS_MEDIA_FILE_NOT_EXISTS);
        if(isset($file->Sysfolder))
            return $this->response(Error::STATUS_FORBID);
        if($file->TotalTime == 10 && $file->FileFmt == -1) {
            if(empty($data['FilePath']))
                return $this->response(Error::STATUS_PARAM_ERROR);
            else
                $file->FilePath = $data['FilePath'];
        }
        if(!empty($data['FileName']))
            $file->FileName = $data['FileName'];
        $this->logTitle['title1'] = $file->FileName;
        if(isset($data['Gain'])) {
            if($data['Gain'] > 10)
                $file->Gain = 10;
            else if($data['Gain'] < -10)
                $file->Gain = -10;
            else
                $file->Gain = $data['Gain'];
        }
        if(isset($data['Parent'])) {
            $parentFolder = VMediaFiles::findOne(['MediaID' => $data['Parent']]);
            if (is_null($parentFolder))
                return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
            if ($parentFolder->Sysfolder == 2)
                return $this->response(Error::STATUS_FORBID);
            $file->Parent = $data['Parent'];
        }
        if($file->load(Yii::$app->request->post(), 'data') && $file->save())
            return $this->response(Error::STATUS_SUCCESS, $this->folderView($file->MediaID));
        return $this->response(Error::STATUS_FAILED);
    }

    public function actionNew(){
        $data = Yii::$app->request->post('data');
        if(is_null($data) || !isset($data['FileName'], $data['Gain'], $data['FilePath']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $isAdmin = (Yii::$app->user->id === 0);
        if(isset($data['Parent'])) {
            $parentFolder = MediaFiles::findOne(['MediaID' => $data['Parent'], 'AudioCodec' => 0, 'VedioCodec' => 0]);
            if(empty($parentFolder))
                return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
            if(!$isAdmin){
                $tryCnt = 20;
                while ($parentFolder->Owner != Yii::$app->user->id) {
                    $parentFolder = MediaFiles::findOne(['MediaID' => $parentFolder->Parent, 'AudioCodec' => 0, 'VedioCodec' => 0]);
                    $tryCnt--;
                    if(empty($parentFolder) || is_null($parentFolder->Parent) || $tryCnt <= 0)
                        return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
                }
            }
        }
        else if(!$isAdmin)
            return $this->response(Error::STATUS_FORBID);
        else
            $parent = null;
        $file = new MediaFiles();
        $file->FileName = $data['FileName'];
        $file->Gain = $data['Gain'];
        $file->FilePath = $data['FilePath'];
        $file->Parent = $data['Parent'];
        if(!$this->isAdmin)
            $file->Parent = isset($data['Parent'])? $data['Parent'] : Yii::$app->user->getId();
        $file->TotalTime = 10;
        $file->FileFmt = -1;
        $file->VedioCodec = -1;
        $file->AudioCodec = -1;
        $this->logTitle['title1'] = $file->FileName;
        if($file->save())
            return $this->response(Error::STATUS_SUCCESS, $file);
        return $this->response(Error::STATUS_FAILED);
    }

    public function actionDelete($id){
        if(!preg_match('/^[0-9,]*$/', $id))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $ids = explode(',', $id);
        if(empty($ids))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $mediaid = [];
        foreach ($ids as $id) {
            if(is_numeric($id))
                $mediaid[] = $id;
        }
        if(empty($mediaid))
            return $this->response(Error::STATUS_SUCCESS, $mediaid);
        $uid = Yii::$app->user->getId();
        $delCmd = Yii::$app->db->createCommand('call sp_del_media_file(:uid, :mediaid);')
            ->bindParam(':uid', $uid, PDO::PARAM_INT)
            ->bindParam(':mediaid', $id, PDO::PARAM_INT);

        $files = [];
        foreach ($mediaid as $id ) {
            $model = MediaFiles::findOne(['MediaID' => $id]);
            if(empty($model))
                continue;
            try{
                $delCmd->execute();
                $files[] = $model->FileName;
            }catch (Exception $e){}
        }
        $this->logDetail['ele/files'] = implode("\n", $files);
        $title1 = implode(",", $files);
        if(mb_strlen($title1) > 64)
            $title1 = mb_substr($title1, 0, 64) . "...";
        $this->logTitle['title1'] = $title1;
        $items = MediaFiles::find()->select(['MediaID'])->where(['in', 'MediaID', $mediaid])->all();
        $ret = [];
        foreach ($items as $item) {
            $ret[] = $item->MediaID;
        }
        return $this->response(Error::STATUS_SUCCESS, ['failed' => $ret]);
    }

    public function actionUpload($parent)
    {
        $isAdmin = (Yii::$app->user->id === 0);
        if(is_numeric($parent)) {
            $parentFolder = MediaFiles::findOne(['MediaID' => $parent, 'AudioCodec' => 0, 'VedioCodec' => 0]);
            if(empty($parentFolder))
                return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
            if(!$isAdmin){
                $tryCnt = 20;
                while ($parentFolder->Owner != Yii::$app->user->id) {
                    $parentFolder = MediaFiles::findOne(['MediaID' => $parentFolder->Parent, 'AudioCodec' => 0, 'VedioCodec' => 0]);
                    $tryCnt--;
                    if(empty($parentFolder) || is_null($parentFolder->Parent) || $tryCnt <= 0)
                        return $this->response(Error::STATUS_FOLDER_PARENT_NOT_EXISTS);
                }
            }
        }
        else if($parent == 'null' && !$isAdmin)
            return $this->response(Error::STATUS_FORBID);
        else
            $parent = null;
        $model = new UploadFile();
        $model->multiple = UploadedFile::getInstancesByName('multi-file');
        if(empty($model->multiple)) {
            return $this->response(Error::STATUS_FAILED, ['error' => $model->errors]);
        }
        $files = $model->uploadMultiple($parent);
        $names = [];
        if(!empty($files)) {
            foreach ($files as $key => $value)
                $names[] = $key;
        }
        $this->logDetail['ele/files']  = implode("\n", $names);
        $title1 = implode(",", $names);
        if(mb_strlen($title1) > 64)
            $title1 = mb_substr($title1, 0, 64) . "...";
        $this->logTitle['title1'] = $title1;
        return $this->response(Error::STATUS_SUCCESS, ['files' => $files]);
    }

    public function actionMove(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['MediaID']) || !preg_match('/^[0-9][0-9,]*$/', $data['MediaID']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        if(isset($data['Parent'])) {
            $parent = MediaFiles::findOne(['MediaID' => $data['Parent']]);
            if(empty($parent))
                return $this->response(Error::STATUS_FOLDER_NOT_EXISTS);
            if($parent->VedioCodec != 0 || $parent->AudioCodec != 0)
                return $this->response(Error::STATUS_FILE_TYPE_ERROR);
            $this->logDetail['ele/target'] = $parent->FileName;
        }
        else {
            $this->logDetail['ele/target'] = "根目录";
            $data['Parent'] = 'null';
        }
        $files = MediaFiles::find()->select(['FileName'])->where('MediaID in (' . $data['MediaID'] .')')
            ->all();
        $names = [];
        foreach ($files as $file) {
            $names[] = $file->FileName;
        }
        $this->logDetail['ele/files'] = implode("\n", $names);
        $title1 = implode(",", $names);
        if(mb_strlen($title1) > 64)
            $title1 = mb_substr($title1, 0, 64) . "...";
        $this->logTitle['title1'] = $title1;
        if(empty($this->logDetail['ele/files'])) {
            $this->logSended = true;
            $this->response(Error::STATUS_FAILED);
        }
        else {
            Yii::$app->db->createCommand('update MediaFiles set Parent=' . $data['Parent'] . ' where MediaID in (' . $data['MediaID'] .');')
                ->execute();
            return $this->response(Error::STATUS_SUCCESS);
        }
    }
}