<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/04/27 16:37
 */
namespace api\controllers;

use common\models\BroadcastMode;
use Yii;
use yii\db\Exception;
use common\classes\Error;
use common\models\VTimedSessionInfo;
use common\models\VSession;
use common\models\BrdModeMember;

/**
 * Task Mode controller
 */
class ModeController extends SessionBaseController
{
    /**
     * @return array
     */
    public function actionView($id) {
        $mode = BroadcastMode::findOne(['idBroadcastMode' => $id]);
        if(is_null($mode))
            return $this->response(Error::STATUS_MODE_NOT_EXIST);
        $members = BrdModeMember::find()
            ->select(['DeviceID'])
            ->where(['BrdModeID' => $id])
            ->all();
        $retMember = [];
        foreach ($members as $member)
            $retMember[] = $member['DeviceID'];
        return $this->response(Error::STATUS_SUCCESS, ['mode' => $mode, 'member' => $retMember]);
    }

    /**
     * @return array
     */
    public function actionNew() {
        $data = Yii::$app->request->post('data');
        return $this->modeNP($data);
    }

    /**
     * @param $id mode id
     * @return array
     */
    public function actionUpdate($id) {
        $data = Yii::$app->request->post('data');
        $shift = Yii::$app->request->get('shift');
        $shift = ($shift == 'yes') ? true : false;
        return $this->modeNP($data, $id, $shift);
    }

    private function modeNP($data, $id = null, $shift =  false){
        if(is_null($data) || !isset($data['mode']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $oldStart = null;
        if(!isset($id)) {
            if(empty($data['mode']['ModeName']))
                return $this->response(Error::STATUS_PARAM_ERROR);
            else
                $this->logTitle['title1'] = $data['mode']['ModeName'];
            $mode = new BroadcastMode();
            $shift = false;
        }
        else {
            $mode = BroadcastMode::findOne(['idBroadcastMode' => $id]);
            if(is_null($mode))
                return $this->response(Error::STATUS_MODE_NOT_EXIST);
            $this->logTitle['title1'] = $mode->ModeName;
            if(empty($mode->StartTime))
                $shift = false;
            else {
                $oldStart = strtotime($mode->StartTime);
            }
        }
        if($mode->load($data, 'mode') && $mode->save()) {
            if($shift && !empty($mode->StartTime)){
                $newStart = strtotime($mode->StartTime);
                if($newStart != $oldStart) {
                    $diff = $newStart - $oldStart;
                    if ($diff > 0) {
                        $timeShifted = "ADDDATE(PlayTime, INTERVAL $diff second)";
                        $order = 'desc';
                    } else {
                        $timeShifted = "SUBDATE(PlayTime, INTERVAL $diff second)";
                        $order = 'asc';
                    }
                    Yii::$app->db->createCommand("update PlaySessionTime set PlayTime = $timeShifted where exists ( select * from PlaySessions
                                      where PlaySessions.BrdMode = " . $mode->idBroadcastMode . " and PlaySessionTime.SessionID = PlaySessions.SessionID)
                                      order by  PlaySessionTime.PlayTime $order;")
                        ->execute();
                }
            }
            do{
                if(!isset($data['member']) || !is_array($data['member']))
                    break;
                if(empty($data['member'])) {
                    BrdModeMember::deleteAll(['BrdModeID' => $mode->idBroadcastMode]);
                    break;
                }
                $members = implode(',', $data['member']);
                if(!preg_match('/^[0-9][0-9,]*[0-9]$/', $members))
                    break;
                Yii::$app->db->createCommand("insert into `BrdModeMember`(`BrdModeID`, `DeviceID`) select ". $mode->idBroadcastMode .
                    ",`DevID` from `DeviceList` where `DevID` in ($members)
                    on duplicate key update `DeviceID`=VALUES(`DeviceID`);")
                    ->query();
                Yii::$app->db->createCommand("delete from `BrdModeMember` where `BrdModeID`=" . $mode->idBroadcastMode ." and `DeviceID` not in ($members);")
                    ->query();
            }while(false);
            $this->logDetail['ele/enable'] = $mode->Enabled ? "enable" : "disable";
            $this->logDetail['ele/active'] = $mode->Active ? "active" : "inactive";
            $this->logDetail['ele/priority'] = $mode->Priority;
            if($mode->TimeStyle != 0) {
                $start = preg_split('/ /', $mode->StartTime)[1];
                $stop = preg_split('/ /', $mode->StopTime)[1];
                for($i = 0; $i < 7; $i++) {
                    if(($mode->TimeStyle & (1 << $i)) > 0) {
                        $this->logDetail['ele/weekday' . $i] = [
                                'start' => $start,
                                'stop' => $stop
                            ];
                    }
                }
            }
            else {
                $this->logDetail['ele/specialdate'] = [
                    'start' => $mode->StartTime,
                    'stop' => $mode->StopTime
                ];
            }
            $this->logDetail['ele/priority'] = $mode->Priority;
            return $this->actionView($mode->idBroadcastMode);
        }
        else
            return $this->response(Error::STATUS_FAILED);
    }

    /**
     * @param $id mode id
     * @return array
     */
    public function actionDelete($id) {
        try{
            $mode = BroadcastMode::find()
                ->select(['ModeName'])
                ->where(['idBroadcastMode' => $id])
                ->one();
            if(empty($mode))
                return $this->response(Error::STATUS_MODE_NOT_EXIST);
            $count = VTimedSessionInfo::find()->where(['BrdMode' => $id])->count();
            if($count > 0) {
                $this->logSended = true;
                return $this->response(Error::STATUS_MODE_NOT_EMPTY);
            }
            $this->logTitle['title1'] = $mode->ModeName;
            Yii::$app->db->createCommand()
                ->delete('BroadcastMode', ['idBroadcastMode' => $id])
                ->execute();
            return $this->response(Error::STATUS_SUCCESS);
        }catch (Exception $e){}
        return $this->response(Error::STATUS_FAILED);
    }

    /**
     * @param $id BroadcastMode id
     * @param $enable
     */
    public function actionDuplicate($id, $enable) {
        $mode = BroadcastMode::find()->select(['ModeName'])->where(['idBroadcastMode' => $id])->one();
        if(empty($mode))
            return $this->response(Error::STATUS_MODE_NOT_EXIST);
        $this->logTitle['title1'] = $mode->ModeName;
        $enable = ($enable == 'yes') ? "true" : "false";
        Yii::$app->db->createCommand("call sp_copy_mode($id, $enable);")
            ->execute();
        $mode = BroadcastMode::find()->orderBy(['idBroadcastMode' => SORT_DESC])
            ->one();
        $this->logTitle['title2'] = $mode->ModeName;
        return $this->response(Error::STATUS_SUCCESS, $mode);
    }

    /**
     * get session list for special mode
     */
    public function actionSessionList($id)
    {
        if(!is_numeric($id))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $data = Yii::$app->getRequest()->post('data');
        $query = VTimedSessionInfo::find()->where(['BrdMode' => $id])
            ->orderBy(['PlayTime' => SORT_ASC]);
        if (($this->serverPri & 0x06) == 0)
            $query = $query->andWhere(['Type' => 'ctrl']);
        return $this->getSessionList($query, $data);
    }
}
