<?php
namespace api\controllers;

use common\classes\Common;
use common\models\BaseInfo;
use common\models\Users;
use Yii;
use common\classes\Error;
use yii\helpers\ArrayHelper;

/**
 * System Monitor controller
 */
class MonitorController extends BaseController
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return  ArrayHelper::merge (
            parent::behaviors(),[
            'authenticator' => [
                'optional' => []
            ]
        ]);
    }

    public function actionGetDevOfflineMonitorConfig() {
        $lists = Yii::$app->db
            ->createCommand('select a.Description as watermark, b.User_Name as user from BaseInfo a
                                    left join Users b on a.InfoValue = b.User_ID where
                                    a.InfomationID >= ' . Common::BASEINFO_OFFLINE_WARNING_START. ' and a.InfomationID <= '. Common::BASEINFO_OFFLINE_WARNING_END . '
                                    order by a.InfomationID;')
            ->queryAll();
        return $this->response(Error::STATUS_SUCCESS, $lists);
    }

    public function actionSetDevOfflineMonitorConfig(){
        $data = Yii::$app->request->post('data');
        if(is_null($data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        if(empty($data) || !is_array($data)) {
            BaseInfo::deleteAll('InfomationID >= ' . Common::BASEINFO_OFFLINE_WARNING_START. ' and InfomationID <= '. Common::BASEINFO_OFFLINE_WARNING_END);
            return $this->response(Error::STATUS_SUCCESS);
        }
        $id = Common::BASEINFO_OFFLINE_WARNING_START;
        foreach ($data as $info) {
            if(!isset($info['watermark'], $info['user']))
                continue;
            if(!is_numeric($info['watermark']) || $info['watermark'] < 0)
                continue;
            $info['watermark'] = floor($info['watermark'] + 0);
            $user = Users::find()->select(['User_ID'])->where(['User_Name' => $info['user']])->one();
            if(empty($user))
                continue;
            Yii::$app->db
                ->createCommand("insert into BaseInfo values($id, '" . $info['watermark'] . "', $user->User_ID) on duplicate key update Description = values(Description), InfoValue=values(InfoValue);")
                ->execute();
            $id++;
            if($id > Common::BASEINFO_OFFLINE_WARNING_END)
                break;
        }
        if($id < Common::BASEINFO_OFFLINE_WARNING_END)
            BaseInfo::deleteAll("InfomationID >= $id and InfomationID <= ". Common::BASEINFO_OFFLINE_WARNING_END);
        return $this->actionGetDevOfflineMonitorConfig();
    }

    public function actionGetDevStatisticsConfig() {
        $lists = Yii::$app->db
            ->createCommand('select a.Description as time, b.User_Name as user from BaseInfo a
                                    left join Users b on a.InfoValue = b.User_ID where
                                    a.InfomationID >= ' . Common::BASEINFO_DEVICE_STATISTICS_START. ' and a.InfomationID <= '. Common::BASEINFO_DEVICE_STATISTICS_END . '
                                    order by a.InfomationID;')
            ->queryAll();
        return $this->response(Error::STATUS_SUCCESS, $lists);
    }

    public function actionSetDevStatisticsConfig() {
        $data = Yii::$app->request->post('data');
        if(is_null($data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        if(empty($data) || !is_array($data)) {
            BaseInfo::deleteAll('InfomationID >= ' . Common::BASEINFO_DEVICE_STATISTICS_START. ' and InfomationID <= '. Common::BASEINFO_DEVICE_STATISTICS_END);
            return $this->response(Error::STATUS_SUCCESS);
        }
        $id = Common::BASEINFO_DEVICE_STATISTICS_START;
        foreach ($data as $info) {
            if(!isset($info['time'], $info['user']))
                continue;
            if(!preg_match('/[0-2][0-9]:[0-5][0-9]:[0-5][0-9]/', $info['time']))
                continue;
            $user = Users::find()->select(['User_ID'])->where(['User_Name' => $info['user']])->one();
            if(empty($user))
                continue;
            Yii::$app->db
                ->createCommand("insert into BaseInfo values($id, '" . $info['time'] . "', $user->User_ID) on duplicate key update Description = values(Description), InfoValue=values(InfoValue);")
                ->execute();
            $id++;
            if($id > Common::BASEINFO_DEVICE_STATISTICS_END)
                break;
        }
        if($id < Common::BASEINFO_DEVICE_STATISTICS_END)
            BaseInfo::deleteAll("InfomationID >= $id and InfomationID <= ". Common::BASEINFO_DEVICE_STATISTICS_END);
        return $this->actionGetDevStatisticsConfig();
    }

    public function actionGetDbErrorConfig(){
        $dbConfig = Yii::$app->db
            ->createCommand('select b.User_Name as user from BaseInfo a
                                    left join Users b on a.InfoValue = b.User_ID where
                                    a.InfomationID = ' . Common::BASEINFO_MYSQL_BACKUP_INFO)
            ->queryOne();
        $config = [];
        if(empty($dbConfig))
            $config['backupUser'] = 'admin';
        else
            $config['backupUser'] = $dbConfig['user'];

        $ttConfig = Yii::$app->db
            ->createCommand('select b.User_Name as user from BaseInfo a
                                    left join Users b on a.InfoValue = b.User_ID where
                                    a.InfomationID = ' . Common::BASEINFO_TIMETABLE_ERROR_RECEIVER)
            ->queryOne();
        if(empty($ttConfig))
            $config['syncUser'] = 'admin';
        else
            $config['syncUser'] = $ttConfig['user'];
        return $this->response(Error::STATUS_SUCCESS, $config);
    }

    public function actionSetDbErrorConfig(){
        $data = Yii::$app->request->post('data');
        if(empty($data) || !isset($data['backupUser'], $data['syncUser']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $user = Users::find()->select(['User_ID'])->where(['User_Name' => $data['backupUser']])->one();
        if(empty($user))
            return $this->response(Error::STATUS_USER_NOT_EXISTS);

        Yii::$app->db
            ->createCommand("insert into BaseInfo values(" . Common::BASEINFO_MYSQL_BACKUP_INFO . ", null, $user->User_ID) on duplicate key update InfoValue=values(InfoValue);")
            ->execute();
        $user = Users::find()->select(['User_ID'])->where(['User_Name' => $data['syncUser']])->one();
        if(empty($user))
            return $this->response(Error::STATUS_USER_NOT_EXISTS);

        Yii::$app->db
            ->createCommand("insert into BaseInfo values(" . Common::BASEINFO_TIMETABLE_ERROR_RECEIVER . ", null, $user->User_ID) on duplicate key update InfoValue=values(InfoValue);")
            ->execute();
        return $this->actionGetDbErrorConfig();
    }
}
