<?php
namespace api\controllers;

use common\classes\Common;
use common\classes\HttpRequest;
use common\models\BaseInfo;
use common\models\CallWaiter;
use common\models\System;
use common\models\Users;
use Yii;
use yii\db\Exception;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;
use common\models\LoginForm;
use common\classes\Error;
use common\models\Cache;
use yii\helpers\ArrayHelper;
use common\models\Users as User;

/**
 * Site controller
 */
class SiteController extends BaseController
{
    const QRCODE_EXIPRE = 600;
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return  ArrayHelper::merge (
            parent::behaviors(),[
            'authenticator' => [
                'optional' => ['login', 'logout', 'stat', 'login-qrcode','login-qrcode-check','flag', 'oauth', 'oauth-login', 'refresh', 'pub-info']
            ]
        ]);
    }

    public static function getUserCacheKey($uid, $action) {
        if($action == 'login')
            return Common::LOGIN_TOKEN_CACHE_PASSWORD . $uid;
        if($action == 'unionid-login')
            return Common::LOGIN_TOKEN_CACHE_UNIONID . $uid;
        if($action == 'oauth')
            return Common::LOGIN_TOKEN_CACHE_OAUTH . $uid;
        return Common::LOGIN_TOKEN_CACHE_OTHER . $uid;
    }

    public static function packageUserInfo($user)
    {
        $ret = [];
        $ret['id'] = $user->User_ID;
        $ret['privilege'] = $user->privilege;
        $ret['desc'] = $user->User_Desc;
        $ret['username'] = $user->User_Name;
        $ret['priority'] = empty($user->Priority) ? 100 : $user->Priority;
        $ret['pri'] = [];
        if($user->User_ID == 0) {
            $ret['pri'] = ['device','dconfig', 'ic','log','brd', 'manual', 'mode', 'template', 'ipcall', 'system'];
        }
        else {
            if($user->privilege & 0x01)
                $ret['pri'][] = 'device';
            if(($user->privilege & 0x03) == 0x03)
                $ret['pri'][] = 'dconfig';
            if($user->privilege & 0x10)
                $ret['pri'][] = 'ic';
            if($user->privilege & 0x20)
                $ret['pri'][] = 'log';
            if($user->privilege & 0x100)
                $ret['pri'][] = 'brd';
            if($user->privilege & 0x200)
                $ret['pri'][] = 'ipcall';
            if(($user->privilege & 0x500) == 0x500)
                $ret['pri'][] = 'mode';
            if(($user->privilege & 0x900) == 0x900)
                $ret['pri'][] = 'template';
            if(($user->privilege & 0x1100) == 0x1100)
                $ret['pri'][] = 'manual';
            if($user->privilege & 0x40000000)
                $ret['pri'][] = 'system';
        }
        return $ret;
    }

    /*
     * login handle
     */
    public static function login($user, $action){
        $token = Yii::$app->security->generateRandomString();
        $now = time();
        $cred = [
            'token' => $token,
            'expire' => $now + self::LOGIN_EXPIRE,      //token expired time
            'user' => $user,
            'action' => $action
        ];

        //save new token and delete cache if old token is exists
        $cache = Yii::$app->getCache();
        $cache->set(Common::LOGIN_TOKEN_CACHE_SALT . $token, $cred, self::LOGIN_EXPIRE);
        $loginInfo = [
            'token' => $token,
            'expire' => self::LOGIN_EXPIRE,
            'datetime' => Yii::$app->base->date,
            'user' => self::packageUserInfo($user)
        ];
        $cache->set(Common::USER_INFO_SALT . $user->User_ID, $loginInfo['user'], 10);
        Yii::$app->udp->login($user, $action);
        //cache logininfo for login-qrcode-check request
        return $loginInfo;
    }
    /**
     * Login action.
     *
     * @return string
     */
    public function actionLogin()
    {
        if (!Yii::$app->user->isGuest) {
            return $this->response(Error::STATUS_FORBID);
        }

        $data = Yii::$app->request->post('data');
        if(is_null($data) || empty($data['username']) || empty($data['password']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $this->logTitle['username'] = $data['username'];
        if(empty($data['source']) || !is_string($data['source']))
            $this->logTitle['title1'] = "log/WEB";
        else
            $this->logTitle['title1'] = 'log/' . strtoupper($data['source']);
        if($data['password'] == 'admin' && (strlen($data['password']) < 8)) {
            $ip = $this->getClientIp();
            $ip = ip2long($ip);
            if($ip == false || ($ip & 0xff000000) != 0x0a000000 && ($ip & 0xfff00000) != 0xac100000 && ($ip & 0xffff0000) != 0xc0a80000)
                return $this->response(Error::STATUS_ADMIN_FORBID);
        }
        $model = new LoginForm();
        if ($model->load(Yii::$app->request->post(), 'data') && $model->login()) {
            $user = $model->getUser();
            return $this->response(Error::STATUS_SUCCESS, self::login($user, $this->action->id));
        } else {
            return $this->response(Error::STATUS_LOGIN_FAILED);
        }
    }

    /**
     * oauth for other platform
     * @param $token Description in baseinfo with InfomationID=0x158
     * @param $usersn
     * @return array
     */
    public function actionOauth($token, $usersn) {
        $base = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_OAUTH_RESOURCE]);
        if(empty($base) || empty($base->Description) || strlen($base->Description) < 64)
            return $this->response(Error::STATUS_FAILED, 1);
        if($token != $base->Description)
            return $this->response(Error::STATUS_FAILED, 2);
        if(!empty($base->InfoValue)) {
            if(!isset($_SERVER['HTTP_X_FORWARDED_FOR']))
                return $this->response(Error::STATUS_FAILED, 3);
            $client = ip2long($_SERVER['HTTP_X_FORWARDED_FOR']);
            if(false == $client)
                return $this->response(Error::STATUS_FAILED, 4);
            if($base->InfoValue != ($client & 0x7fffffff))
                return $this->response(Error::STATUS_FAILED, 5);
        }
        $user = User::findOne(['User_Sn' => $usersn]);
        if(empty($user) || empty($user->User_Name) || empty($user->Pass_Word) || empty($user->privilege))
            return $this->response(Error::STATUS_FAILED);
        $userinfo = self::login($user, $this->action->id);
        $this->logTitle['username'] = $user->User_Name;
        $credit = Yii::$app->security->generateRandomString(64);
        Yii::$app->cache->set(Common::OAUTH_CREDIT_CACHE_OAUTH . $credit, $userinfo, 30);
        return $this->response(Error::STATUS_SUCCESS, ['credit' =>$credit, 'expire' => 30]);
    }

    public function actionOauthLogin($credit) {
        $userinfo = Yii::$app->cache->get(Common::OAUTH_CREDIT_CACHE_OAUTH . $credit);
        Yii::$app->cache->delete(Common::OAUTH_CREDIT_CACHE_OAUTH . $credit);
        if($userinfo === false)
            return $this->response(Error::STATUS_FAILED);
        else
            return $this->response(Error::STATUS_SUCCESS, $userinfo);
    }

    /**
     * Logout action.
     *
     * @return string
     */
    public function actionLogout() {
        $token = Yii::$app->getRequest()->get('access_token');
        $cache = Yii::$app->getCache();
        $cred = $cache->get(Common::LOGIN_TOKEN_CACHE_SALT . $token);
        if(empty($cred)) {
            $this->logSended = true;
            return $this->response(Error::STATUS_SUCCESS);
        }
        $this->logTitle['username'] = $cred['user']->User_Name;
        $this->logTitle['title1'] = $cred['user']->User_Name;
        Yii::$app->db->createCommand("update CallWaiter, IpCall set IpCall.status='idle' where CallWaiter.session = '$token' and IpCall.waiter = CallWaiter.id;")
            ->execute();
        $cache->delete(Common::LOGIN_TOKEN_CACHE_SALT . $token);
        //cache logininfo for login-qrcode-check request
        return $this->response(Error::STATUS_SUCCESS);
    }

    /**
     * create new access_token with 30 minutes duration
     * old access_token only valid in 5 minutes
     *
     * @return string
     */

    public function actionRefresh()
    {
        $oldToken = Yii::$app->getRequest()->get('access_token');
        $cred = Yii::$app->getCache()->get(Common::LOGIN_TOKEN_CACHE_SALT . $oldToken);
        if($cred === false)
            return $this->response(Error::STATUS_ERROR_401);
        try{
            $user = Users::findOne(['User_ID' => $cred['user']->getId()]);
            if(empty($user))
                return $this->response(Error::STATUS_ERROR_401);
        }catch (Exception $e){
            return $this->response(Error::STATUS_FAILED);
        }
        $now = time();
        $cred = [
            'token' => Yii::$app->security->generateRandomString(),
            'expire' => $now + self::LOGIN_EXPIRE,      //token expired time
            'user' => $user,
        ];

        Yii::$app->getCache()->set(Common::LOGIN_TOKEN_CACHE_SALT . $cred['token'], $cred, self::LOGIN_EXPIRE);
        $loginInfo = [
            'token' => $cred['token'],
            'expire' => self::LOGIN_EXPIRE,
            'datetime' => Yii::$app->base->date,
            'user' => self::packageUserInfo($cred['user'])
        ];
        return $this->response(Error::STATUS_SUCCESS, $loginInfo);
    }

    /**
     * get server stat
     */
    public function actionStat(){
        //get uuid
        try{
            $models = BaseInfo::find()->where(['in', 'InfomationID', [0, 312, 313, 314]])->indexBy('InfomationID')->all();
            if(count($models) == 0 || !isset($models[313]) || empty($models[313]->Description))
                return $this->response(Error::STATUS_UNREGISTERED);
        }catch (Exception $e){
            $ret['dbsever'] = 'stop';
            return $this->response(Error::STATUS_SUCCESS, $ret);
        }
        $ret['dbsever'] = 'running';
        $ret['cloud'] = 'https://' . Yii::$app->base->cloudServer . '/api';
        $ret['fullName'] = $models[0]->Description;
        $ret['shortName'] = $models[312]->Description;
        $ret['uuid'] =md5(\common\classes\Common::UUID_MD5_PRESTR. $models[313]->Description);
        $ret['version'] = Yii::$app->base->version;
        $ret['weekday'] = Yii::$app->base->weekday;
        $ret['privilege'] = null;
        //timetable enable flag
        $ttEn = Yii::$app->base->timetableState;
        if($ttEn == 1)
            $ret['timetable'] = 'enable';
        $ret['time'] = Yii::$app->base->date;
        if($models[314]->InfoValue & 0x01)
            $ret['privilege'][] = 'device';
        if($models[314]->InfoValue & 0x02)
            $ret['privilege'][] = 'audioBroadcast';
        if($models[314]->InfoValue & 0x04)
            $ret['privilege'][] = 'videoBroadcast';
        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    /**
     * login with unionid and usrename
     */
    public function actionUnionidLogin(){
        $data = Yii::$app->getRequest()->post("data");
        if(!Yii::$app->getUser()->identity->cloudLogin)
            return $this->response(Error::STATUS_FAILED);
        if(is_null($data) || !isset($data['username']))
            return $this->response(Error::STATUS_LOGIN_FAILED);
        if(!isset($data['unionid']))
            return $this->response(Error::STATUS_USER_UNIONID_NOT_BOUND);
        $this->logTitle['username'] = $data['username'];
        $this->logTitle['title1'] = $data['username'];
        $user = User::findByUsername($data['username']);
        if(is_null($user))
            return $this->response(Error::STATUS_USER_NOT_EXISTS);
        $hash = hash('sha256', Common::UNIONID_MD5_PRESTR . $data['unionid']);

        if($hash != $user->UnionId)
            return $this->response(Error::STATUS_LOGIN_FAILED);
        return $this->response(Error::STATUS_SUCCESS, self::login($user, $this->action->id));
    }

    /**
     * get server flag
     */
    public function actionFlag()
    {
        $ret = [
            'flag' => '9026849',
            'cloud' => 'https://' . Yii::$app->base->cloudServer . '/api',
            'version' => Yii::$app->base->version
        ];
        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    public function actionPubInfo() {
        return $this->response(Error::STATUS_SUCCESS, [
            'info' => [
                'title' => '服务器新版本发布!',
                'subTitle' => '「D06.00.09 2018-09-04」',
                'titleIcon' => 'el-icon-info',
                'content' => "版本说明\n\n1.修复Bug\n2.新增使用人数统计曲线\n3.新增投影仪通道切换功能",
                'img' => 'https://timgsa.baidu.com/timg?image&quality=80&size=b9999_10000&sec=1536230296924&di=9b501fc2d5e6e0df7e8dbbadb137aacd&imgtype=0&src=http%3A%2F%2Fatt.bbs.duowan.com%2Fforum%2F201801%2F28%2F2050073n37e33p3empekb9.jpg',
                'more' => [
                    'url' => 'https://www.baidu.com',
                    'label' => '查看更多'
                ]
            ]
        ]);
    }
    /**
     * @return array
     * return 401 if token is invalid.
     */
    public function actionLoginValid(){
        return $this->response(Error::STATUS_SUCCESS);
    }
}
