<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/5/8 19:20
 */
namespace api\controllers;

use common\classes\Common;
use common\classes\Error;
use common\models\SystemLog;
use Yii;

/**
 * System log controller
 */
class SlogController extends BaseController
{
    public function actionTypes(){
        $types = Yii::$app->getCache()->get(Common::MESSAGE_TYPE_CACHE);
        if(!empty($types))
            return $this->response(Error::STATUS_SUCCESS, $types);
        $tmp = [];
        foreach ($this->logActions as $type => $actions) {
            $v = Yii::t('app/message', 'type/' . $type);
            $tmp[$v][] = $type;
        }
        $types = [];
        foreach ($tmp as $key => $value) {
            $types[] = [
                'key' => $key,
                'types' => $value
            ];
        }
        Yii::$app->getCache()->set(Common::MESSAGE_TYPE_CACHE, $types, 30);
        return $this->response(Error::STATUS_SUCCESS, $types);
    }

    public function actionList(){
        $query = SystemLog::find()->orderBy(['id' => SORT_DESC]);

        $startDate = Yii::$app->getRequest()->get('startDate');
        if(strptime($startDate, '%Y-%m-%d') !== false)
            $query->andWhere("`at` >= '$startDate 00:00:00'");

        $endDate = Yii::$app->getRequest()->get('endDate');
        if(strptime($endDate, '%Y-%m-%d') !== false)
            $query->andWhere("`at` <= '$endDate 23:59:59'");

        $end = Yii::$app->getRequest()->get('end');
        if(!empty($end) && is_numeric($end))
            $query->andWhere("id <= $end");

        $file = Yii::$app->getRequest()->get('file');

        $getError = Yii::$app->getRequest()->get('get-error');
        $getError = (!empty($getError) && $getError=='yes') ? true : false;

        $data = Yii::$app->getRequest()->post('data');
        $count = $this->listQuery($query, $data);
        if(!$file)
            $ret = [];
        else
            $ret = Yii::t('app/message', 'log/at') . ',' . Yii::t('app/message', 'log/ip') . ','.
                Yii::t('app/message', 'log/username') . ',' . Yii::t('app/message', 'log/type') . ',' .
                Yii::t('app/message', 'log/title') . ',' . Yii::t('app/message', 'log/result') . "\r\n";
        $fp = null;
        $filepath = '';
        if($file) {
            $filepath = Common::MEDIA_TEMPORARA_DIR . Yii::$app->security->generateRandomString(16) . ".tmp";
            $fp = fopen($filepath, "a");
            fwrite($fp, $ret);
        }
        $limit = (isset($data['limit'])) ? $data['limit'] : $count;
        for($i = 0; $i < $limit / 10000; $i++) {
            $newQuery = $query;
            $newQuery->offset($i * 10000);
            if($limit - $i * 10000 >= 10000)
                $newQuery->limit(10000);
            else
                $newQuery->limit($limit - $i * 10000);
            if($file)
                $ret = '';

            $lists = $newQuery->all();
            foreach ($lists as $list) {
                $tmp['id'] = $list->id;
                $tmp['at'] = $list->at;
                $tmp['ip'] = $list->ip;
                $tmp['username'] = $list->username;
                $tmp['type'] = Yii::t('app/message', 'type/' . $list->type);
                $key = $list->type . '/' . $list->action;
                if(!empty($list->title1)) {
                    $pre = null;
                    if(preg_match('/^ori\//', $list->title1)) {
                        $title = (preg_split('/\//', $list->title2, 2))[1];
                        $pre = 'ele/' . $title . '/label';
                    }
                    else if(preg_match('/^ele\//', $list->title1)) {
                        $title = (preg_split('/\//', $list->title2, 2))[1];
                        $pre = 'ele/' . $list->type . '/' . $list->action . '/' . $title . '/label';
                    }
                    if(!empty($pre)) {
                        $title = Yii::t('app/message', $pre);
                        if($title != $pre)
                            $list->title1 = $title;
                    }
                }
                if(!empty($list->title2)) {
                    $pre = null;
                    if(preg_match('/^ori\//', $list->title2)) {
                        $title = (preg_split('/\//', $list->title2, 2))[1];
                        $pre = 'ele/' . $title . '/label';
                    }
                    else if(preg_match('/^ele\//', $list->title2)) {
                        $title = (preg_split('/\//', $list->title2, 2))[1];
                        $pre = 'ele/' . $list->type . '/' . $list->action . '/' . $title . '/label';
                    }
                    if(!empty($pre)) {
                        $title = Yii::t('app/message', $pre);
                        if($title != $pre)
                            $list->title2 = $title;
                    }
                }
                $tmp['title'] = Yii::t('app/message', 'title/' . $key,
                    ['username' => $list->username, 'title1' => Yii::t('app/message', $list->title1), 'title2' => Yii::t('app/message', $list->title2)]);
                $tmp['detail'] = empty($list->detail) ? 'no':'yes';
                $tmp['result'] = ($list->result == Error::STATUS_SUCCESS) ? Yii::t('app', 'success') : Yii::t('app', 'failed');
                $tmp['title'] = str_replace('「」', '', $tmp['title']);
                if(!$file)
                    $ret[] = $tmp;
                else {
                    unset($tmp['detail'], $tmp['id']);
                    $ret .= implode(',', $tmp) . "\r\n";
                }
            }
            if($file)
                fwrite($fp, $ret);
        }
        if(!$file)
            return $this->response(Error::STATUS_SUCCESS, ['count' => $count, 'list' => $ret]);
        return Yii::$app->response->sendFile($filepath, "systemlog-$startDate-$endDate.csv",  ["mimeType" => "text/csv"]);
    }

    public function actionView($id){
        $getError = Yii::$app->getRequest()->get('get-error');
        $getError = (!empty($getError) && $getError=='yes') ? true : false;
        $model = SystemLog::find()
            ->select(['detail', 'type', 'action'])
            ->where(['id' => $id])
            ->one();
        if(empty($model))
            return $this->response(Error::STATUS_MESSAGE_NOT_EXISTS);
        if(empty($model->detail))
            return $this->response(Error::STATUS_SUCCESS);
        $details = json_decode($model->detail, true);
        if(empty($details) && is_array($details))
            return $this->response(Error::STATUS_SUCCESS);
        $ret = [];
        foreach ($details as $key => $value) {
            if(empty($value))
                continue;
            if(preg_match('/^ele\//', $key)) {
                $key = (preg_split('/\//', $key, 2))[1];
                $pre = 'ele/' . $model->type . '/' . $model->action . '/' . $key;
            }
            else {
                $pre = 'ele/' . $key;
            }
            $tmp['k'] = Yii::t('app/message', $pre . '/label', ['label' => $key]);
            if(is_array($value)) {
                $tmp['v'] = '';
                $replaceDirect = false;
                foreach ($value as $item) {
                    if(is_array($item))
                        $tmp['v'] .= Yii::t('app/message', $pre . '/value', $item) . "\n";
                    else {
                        $replaceDirect = true;
                        break;
                    }
                }
                if($replaceDirect)
                    $tmp['v'] = Yii::t('app/message', $pre . '/value', $value);
            }
            else
                $tmp['v'] = Yii::t('app/message', $pre . '/value', ['value' => $value]);
            $tmp['v'] = trim($tmp['v']);
            if($getError || (!preg_match('/\{[a-z0-9]+\}/', $tmp['k']) && !preg_match('/\{[a-z0-9]+\}/', $tmp['v'])))
                $ret[] = $tmp;
        }
        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    public function actionClean() {
        $startDate = Yii::$app->getRequest()->get('startDate');
        if(empty($startDate) || strptime($startDate, '%Y-%m-%d') == false)
            $startDate = '1000-01-01';

        $endDate = Yii::$app->getRequest()->get('endDate');
        if(empty($endDate) || strptime($endDate, '%Y-%m-%d') == false)
            $endDate = '9999-12-31';

        $this->logTitle['title1'] = $startDate;
        $this->logTitle['title2'] = $endDate;
        $cnt = Yii::$app->db
            ->createCommand("delete from SystemLog where at >= '$startDate 00:00:00' and at <= '$endDate 23:59:59';")
            ->execute();
        return $this->response(Error::STATUS_SUCCESS, ['rows' => $cnt]);

    }
}
