<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/5/3 10:34
 */
namespace api\controllers;
use common\classes\Error;
use common\models\PwrDisplaySet;
use common\models\VDeviceStatistic;
use Yii;

/**
 * Device Statistic controller
 */
class StatisticController extends BaseController
{
    public function actionList(){
        $file = Yii::$app->getRequest()->get('file');
        $data = Yii::$app->getRequest()->post('data');

        $startDate = Yii::$app->getRequest()->get('startDate');
        if(empty($startDate) || strptime($startDate, '%Y-%m-%d') == false)
            $startDate = '1000-01-01';

        $endDate = Yii::$app->getRequest()->get('endDate');
        if(empty($endDate) || strptime($endDate, '%Y-%m-%d') == false)
            $endDate = '9999-12-31';

        $query = VDeviceStatistic::find()
            ->andWhere("`StatisticDay` >= '$startDate 00:00:00'")
            ->andWhere("`StatisticDay` <= '$endDate 23:59:59'");

        unset($data['limit'], $data['page']);
        $count = $this->listQuery($query, $data);

        $ret = [];
        for($i = 0; $i < $count / 10000; $i++) {
            $newQuery = $query;
            $newQuery->offset($i * 10000);
            $newQuery->limit(10000);
            $lists = $newQuery->all();
            foreach ($lists as $item){
                if(!isset($ret[$item->DeviceID]))
                    $ret[$item->DeviceID] = $item;
                else {
                    for($i = 0; $i < 16; $i++) {
                        $key = 'power' . $i;
                        $ret[$item->DeviceID]->$key += $item->$key;
                        $key = 'time' . $i;
                        $ret[$item->DeviceID]->$key += $item->$key;
                    }
                }
            }
        }
        if(!$file)
            return $this->response(Error::STATUS_SUCCESS, ['count' => count($ret), 'list' => $ret]);

        $col = [
            Yii::t('app/message', 'log/DeviceID'),
            Yii::t('app/message', 'log/DeviceName'),
            Yii::t('app/message', 'log/Day'),
            Yii::t('app/message', 'log/Controller') . Yii::t('app/message', 'log/Power'),
            Yii::t('app/message', 'log/Controller') . Yii::t('app/message', 'log/Interval'),
            Yii::t('app/message', 'log/Projector') . Yii::t('app/message', 'log/Power'),
            Yii::t('app/message', 'log/Projector') . Yii::t('app/message', 'log/Interval')
        ];
        $select = ['DeviceID', 'DeviceName', 'StatisticDay',
            'power0' => 'cast(power0/1000 as decimal(10,4))', 'time0' => 'cast(time0/3600 as decimal(10,4))',
            'power1' => 'cast(power1/1000 as decimal(10,4))', 'time1' => 'cast(time1/3600 as decimal(10,4))'];
        $iots = PwrDisplaySet::find()->all();
        foreach ($iots as $iot) {
            if(!empty($iot->Name) && !is_null($iot->Type)) {
                $select[] = 'cast(power' . $iot->PwrID . '/1000 as decimal(10,4))';
                $select[] = 'cast(time' . $iot->PwrID . '/3600 as decimal(10,4))';
                $col[] = $iot->Name . Yii::t('app/message', 'log/Power');
                $col[] = $iot->Name . Yii::t('app/message', 'log/Interval');
            }
        }
        $ret = implode(',', $col) . "\r\n";
        $models = VDeviceStatistic::find()
            ->select($select)
            ->andWhere("`StatisticDay` >= '$startDate 00:00:00'")
            ->andWhere("`StatisticDay` <= '$endDate 23:59:59'")
            ->asArray()
            ->all();
        foreach ($models as $model) {
            $ret .= implode(',', $model) . "\r\n";
        }
        return Yii::$app->response->sendContentAsFile($ret, "device-statistic-$startDate-$endDate.csv",  ["mimeType" => "text/csv"]);
    }

    public function actionClean($startDate, $endDate) {
        if($startDate == 'null')
            $startDate = '1000-01-01';
        else if(strptime($startDate, '%Y-%m-%d') == false)
            return $this->response(Error::STATUS_PARAM_ERROR);
        if($endDate == 'null')
            $endDate = '9999-12-31';
        else if(strptime($endDate, '%Y-%m-%d') == false)
            return $this->response(Error::STATUS_PARAM_ERROR);
        $this->logTitle['title1'] = $startDate;
        $this->logTitle['title2'] = $endDate;
        $cnt = Yii::$app->db
            ->createCommand("delete from DeviceStatistic where StatisticDay >= '$startDate 00:00:00' and StatisticDay <= '$endDate 23:59:59';")
            ->execute();
        return $this->response(Error::STATUS_SUCCESS, ['rows' => $cnt]);
    }

    public function actionChartList () {
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data)|| !isset($data['limit'])) {
            $data['limit'] = 200;
        }
        if(!isset($data['page'])) {
            $data['page'] = 0;
        }
        $startDate = Yii::$app->getRequest()->get('startDate');
        if(empty($startDate) || strptime($startDate, '%Y-%m-%d') == false)
            $startDate = '1000-01-01';

        $endDate = Yii::$app->getRequest()->get('endDate');
        if(empty($endDate) || strptime($endDate, '%Y-%m-%d') == false)
            $endDate = '9999-12-31';

        $query = VDeviceStatistic::find()->select([
            'StatisticDay',
            'sum(power0) as power0',
            'sum(power1) as power1',
            'sum(power2) as power2',
            'sum(power3) as power3',
            'sum(power4) as power4',
            'sum(power5) as power5',
            'sum(power6) as power6',
            'sum(power7) as power7',
            'sum(power8) as power8',
            'sum(power9) as power9',
            'sum(power10) as power10',
            'sum(power11) as power11',
            'sum(power12) as power12',
            'sum(power13) as power13',
            'sum(power14) as power14',
            'sum(power15) as power15',
            'sum(time0) as time0',
            'sum(time1) as time1',
            'sum(time2) as time2',
            'sum(time3) as time3',
            'sum(time4) as time4',
            'sum(time5) as time5',
            'sum(time6) as time6',
            'sum(time7) as time7',
            'sum(time8) as time8',
            'sum(time9) as time9',
            'sum(time10) as time10',
            'sum(time11) as time11',
            'sum(time12) as time12',
            'sum(time13) as time13',
            'sum(time14) as time14',
            'sum(time15) as time15'])
            ->andWhere("`StatisticDay` >= '$startDate 00:00:00'")
            ->andWhere("`StatisticDay` <= '$endDate 23:59:59'")
            ->groupBy(['StatisticDay']);

        $this->listQuery($query, $data);
        $lists = $query->all();

        return $this->response(Error::STATUS_SUCCESS, ['count' => count($lists), 'list' => $lists]);
    }
}