<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/04/18 17:59
 */
namespace api\controllers;

use common\models\SubNetwork;
use yii\db\Exception;
use yii\filters\VerbFilter;
use common\classes\Error;
use Yii;

/**
 * Sub NetWork controller
 */
class SubNetworkController extends BaseController
{
    public function actionList() {
        $list = SubNetwork::find()
            ->where('idSubNetwork > 0')
            ->all();
        return $this->response(Error::STATUS_SUCCESS, $list);
    }

    public function actionUpdate($id, $mask, $ip){
        if(ip2long($mask) === false || ip2long($ip) === false)
            return $this->response(Error::STATUS_PARAM_ERROR);
        $model = SubNetwork::findOne(['idSubNetwork' => $id]);
        if(empty($model))
            return $this->response(Error::STATUS_SUBNETWORK_NOT_EXISTS);
        $model->NetMask = $mask;
        $model->IpAddress = $ip;
        $this->logTitle['title1'] = $model->IpAddress;
        $this->logTitle['title2'] = $model->NetMask;
        try{
            if($model->save())
                return $this->response(Error::STATUS_SUCCESS);
        }catch (Exception $e){}
        return $this->response(Error::STATUS_FAILED);
    }

    public function actionDelete($id){
        $model = SubNetwork::findOne(['idSubNetwork' => $id]);
        if(empty($model))
            return $this->response(Error::STATUS_SUCCESS);
        $this->logTitle['title1'] = $model->IpAddress;
        $this->logTitle['title2'] = $model->NetMask;
        if($model->delete()) {
            $items = SubNetwork::find()->where("idSubNetwork > $id")
                ->all();
            if(count($items) > 0) {
                $mask = null;
                $ip = null;
                $cmd = Yii::$app->db->createCommand('insert into `SubNetwork` (`idSubNetwork`, `NetMask`, `IpAddress`) VALUES (:id, :mask, :ip);')
                    ->bindParam(':id', $id)
                    ->bindParam(':mask', $mask)
                    ->bindParam(':ip', $ip);
                foreach ($items as $item) {
                    $item->delete();
                    $mask = $item->NetMask;
                    $ip = $item->IpAddress;
                    $cmd->execute();
                    $id++;
                }
            }
            return $this->response(Error::STATUS_SUCCESS);
        }
        return $this->response(Error::STATUS_FAILED);
    }

    public function actionNew($mask, $ip){
        if(ip2long($mask) === false || ip2long($ip) === false)
            return $this->response(Error::STATUS_PARAM_ERROR);
        $subnetworks = SubNetwork::find()
            ->select(['idSubNetwork'])
            ->where('idSubNetwork > 0')
            ->indexBy('idSubNetwork')
            ->all();
        if(count($subnetworks) >= 32)
            return $this->response(Error::STATUS_SUBNETWORK_TOO_MANY);
        for ($i = 1; $i < 32; $i++) {
            if(!isset($subnetworks[$i])) {
                $id = $i;
                break;
            }
        }
        $model = new SubNetwork();
        $model->idSubNetwork = $id;
        $model->NetMask = $mask;
        $model->IpAddress = $ip;
        $this->logTitle['title1'] = $model->IpAddress;
        $this->logTitle['title2'] = $model->NetMask;
        try{
            if($model->save())
                return $this->response(Error::STATUS_SUCCESS, $model);
        }catch (Exception $e){}
        return $this->response(Error::STATUS_FAILED);
    }
}
