<?php
namespace api\controllers;

use common\classes\Common;
use common\models\BaseInfo;
use common\models\DeviceList;
use common\models\DeviceStatus;
use common\models\VEvent;
use common\models\PwrDisplaySet;
use common\models\System;
use common\models\SystemLog;
use common\models\TaskTemplate;
use common\models\VDeviceLog;
use common\models\UploadFile;
use Yii;
use yii\db\Exception;
use common\classes\Error;
use common\models\Cache;
use yii\helpers\ArrayHelper;
use yii\web\UploadedFile;

/**
 * Site controller
 */
class SystemController extends BaseController
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return  ArrayHelper::merge (
            parent::behaviors(),[
            'authenticator' => [
                'optional' => ['iot-list','info']
            ]
        ]);
    }

    public function actionPushCertificate(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['certificate']))
            return $this->response(Error::STATUS_FAILED);
        try{
            $str = str_replace(["\r\n","\n","\r"," ", "\t"], "", $data['certificate']);
            if(!ctype_xdigit($str)){
                return $this->response(Error::STATUS_PARAM_ERROR);
            }
            $fp = fopen("/home/upload/temp/idste_server.crt", "w+");
            $str = hex2bin($str);
            fwrite($fp, $str);
            fclose($fp);
            return $this->response(Error::STATUS_SUCCESS, $data);
        }catch(Exception $e){
            return $this->response(Error::STATUS_FAILED);
        }
    }

    public function actionIotList() {
        $ret = [];
        $iots = PwrDisplaySet::find()->where(['>', 'PwrID', '1'])->orderBy(['PwrID' => SORT_ASC])->all();
        $rf0 = PwrDisplaySet::findOne(['PwrID' => 0]);
        if(is_null($rf0)) {
            $rf0 = new PwrDisplaySet();
            $rf0->PwrID = 0;
            $rf0->Type = NULL;
        }
        if(empty($rf0->Name))
            $rf0->Name = Yii::t('app', "center_controller");

        $rf1 = PwrDisplaySet::findOne(['PwrID' => 1]);
        if(is_null($rf1)) {
            $rf1 = new PwrDisplaySet();
            $rf1->PwrID = 1;
            $rf1->Type = NULL;
        }
        if(empty($rf1->Name))
            $rf1->Name = Yii::t('app', "projector");
        $ret['iots'] = array_merge([$rf0, $rf1], $iots);
        $ret['mode'] = BaseInfo::find()->where(['in', 'infomationID', [1,2,3,4]])->all();

        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    public function actionIotUpdate(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !is_array($data))
            return $this->response(Error::STATUS_FAILED);
        $success = [];
        foreach ($data as $iot){
            if(!isset($iot['PwrID']) || !is_numeric($iot['PwrID']) || $iot['PwrID'] < 0)
                continue;
            $model = PwrDisplaySet::findOne(['PwrID' => $iot['PwrID']]);
            if(is_null($model)) {
                $model = new PwrDisplaySet();
                $model->PwrID = $iot['PwrID'];
            }
            if(!empty($iot['Name']))
                $model->Name = mb_strcut($iot['Name'], 0, 15);
            else
                $model->Name = null;
            if(isset($iot['Type']) && is_numeric($iot['Type']) && $iot['Type'] >= 0)
                $model->Type = $iot['Type'];
            else
                $model->Type = null;
            if($model->save())
                $success[] = $model;
        }
        $update = [];
        $delete = [];
        foreach ($success as $item) {
            if(is_null($item->Type))
                $delete[] = $item->PwrID;
            else
                $update[] = [
                    'name' => $item->Name,
                    'type' => $item->Type,
                    'id' => $item->PwrID
                ];
        }
        if(!empty($update))
            $this->logDetail['ele/update'] = $update;
        if(!empty($delete))
            $this->logDetail['ele/delete'] = $delete;
        return $this->response(Error::STATUS_SUCCESS, $success);
    }

    public function actionIotModeUpdate(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !is_array($data))
            return $this->response(Error::STATUS_FAILED);
        $success = [];
        foreach ($data as $key => $mode){
            if(!in_array($key, ["1", "2", "3", "4"]))
                continue;
            $model = BaseInfo::findOne(['InfomationID' => $key]);
            if(is_null($model))
                continue;
            if($model->load($data, $key) && $model->save()) {
                $success[$key] = $model->Description;
            }
        }

        $tmp = [];
        foreach ($success as $key => $value) {
            $tmp[] = [
                'id' => $key,
                'name' => $value,
            ];
        }
        $this->logDetail['ele/mode'] = $tmp;

        return $this->response(Error::STATUS_SUCCESS, array_keys($success));
    }

    /**
     * system reboot
     * @return array
     */
    public function actionReboot(){
        $log = $this->writeLog(Error::STATUS_SUCCESS, null);
        $this->logSended = true;
        $ret = Yii::$app->udp->system($this->id, $this->action->id, null);
        $this->logChange($log, 'result', Error::STATUS_FAILED);
        return $ret;
    }

    /**
     * system shutdown
     * @return array
     */
    public function actionShutdown(){
        $log = $this->writeLog(Error::STATUS_SUCCESS, null);
        $this->logSended = true;
        $ret = Yii::$app->udp->system($this->id, $this->action->id, null);
        $this->logChange($log, 'result', Error::STATUS_FAILED);
        return $ret;
    }

    /**
     * idste-service restart
     * @return array
     */
    public function actionServiceRestart(){
        $log = $this->writeLog(Error::STATUS_SUCCESS, null);
        $this->logSended = true;
        $now = time();
        Yii::$app->udp->system($this->id, $this->action->id, null);
        $duration = time() - $now;
        if($duration > 5) {
            $this->logChange($log, 'result', Error::STATUS_SUCCESS);
            return $this->response(Error::STATUS_SUCCESS);
        }
        else {
            $this->logChange($log, 'result', Error::STATUS_FAILED);
            return $this->response(Error::STATUS_FAILED);
        }
    }

    private function systemBaseInfo(){

        $tmp = json_decode(stripcslashes(shell_exec(  "systeminfo general_info")),true);
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/os/label'),
            'v' => $tmp['OS']
        ];
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/hostname/label'),
            'v' => $tmp['Hostname']
        ];
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/uptime/label'),
            'v' => $tmp['Uptime']
        ];
        $tmp = json_decode(stripcslashes(shell_exec(  "systeminfo cpu_info")),true);
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/cpuinfo/label'),
            'v' => Yii::t('app/message', 'ele/system/info/cpuinfo/value', [
                'architecture' => trim($tmp['Architecture']),
                'vendor' => trim($tmp['Vendor ID']),
                'cpus' => trim($tmp['CPU(s)']),
                'frequence' => trim($tmp['CPU MHz']),
            ])
        ];
        $tmp = json_decode(stripcslashes(shell_exec(  "systeminfo load_avg")),true);
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/avarate/label'),
            'v' => Yii::t('app/message', 'ele/system/info/avarate/value', [
                '1_min_avg' => trim($tmp['1_min_avg']),
                '5_min_avg' =>  trim($tmp["5_min_avg"]),
                '15_min_avg' =>  trim($tmp["15_min_avg"])
            ])
        ];
        $tmp = json_decode(stripcslashes(shell_exec(  "systeminfo current_ram")),true);
        $tmp['used%'] = floor($tmp['used'] * 100 / $tmp['total']);
        $tmp['used'] = floor($tmp['used']);
        $tmp['total'] = floor($tmp['total']);
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/memory/label'),
            'v' => Yii::t('app/message', 'ele/system/info/memory/value', $tmp)
        ];
        $tmp = json_decode(stripcslashes(shell_exec(  "systeminfo disk_partitions")),true);
        $disks = [];
        foreach ($tmp as $item) {
            if(!preg_match('/\/dev\//', $item['file_system']))
                continue;
            $disks[] = Yii::t('app/message', 'ele/system/info/disk/value', $item);
        }
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/disk/label'),
            'v' => $disks
        ];
        $tmp = json_decode(stripcslashes(shell_exec(  "systeminfo io_stats")),true);
        $iostat = [];
        foreach ($tmp as $item) {
            if(preg_match('/[0-9]$/', $item['device']))
                continue;
            $item['reads'] = floor($item['reads'] / 1024);
            $item['writes'] = floor($item['writes'] / 1024);
            $iostat[] = Yii::t('app/message', 'ele/system/info/iostat/value', $item);
        }
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/iostat/label'),
            'v' => $iostat
        ];
        $tmp = json_decode(stripcslashes(shell_exec(  "systeminfo swap")),true);
        foreach ($tmp as $item) {
            $item['size'] = floor($item['size'] / 1024);
            $item['used'] = floor($item['used'] / 1024);
            $item['used%'] = floor($item['used'] * 100 / $item['size']);
            $ret[] = [
                'k' => Yii::t('app/message', 'ele/system/info/swap/label'),
                'v' => Yii::t('app/message', 'ele/system/info/swap/value', $item)
            ];
            break;
        }
        $tmp = json_decode(stripcslashes(shell_exec(  "systeminfo logged_in_users")),true);
        $logged_in_users = [];
        foreach ($tmp as $item) {
            $logged_in_users[] = Yii::t('app/message', 'ele/system/info/loggeduser/value', $item);
        }
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/loggeduser/label'),
            'v' => $logged_in_users
        ];
        return $ret;
    }

    private function idsteServerInfo(){
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/version/label'),
            'v' => Yii::$app->base->version
        ];
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/timezone/label'),
            'v' => Yii::$app->base->timezone
        ];

        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/servertime/label'),
            'v' => trim(shell_exec('date'))
        ];

        $tmp = [];
        $model = TaskTemplate::findOne(['idTaskTemplate' => Yii::$app->base->activeTemplateId]);
        if(empty($model)) {
            $template = Yii::t('app', 'unset');
            $tmp['level'] = 'error';
        }
        else if(empty($model->TemplateName)) {
            if($model->idTaskTemplate == 0)
                $template = Yii::t('app', 'default');
            else
                $template = Yii::t('app', 'default') . " copy";
        }
        else
            $template = $model->TemplateName;
        $tmp['k'] = Yii::t('app/message', 'ele/system/info/template/label');
        $tmp['v'] = $template;
        $ret[] = $tmp;

        $cloudInfo = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_CLOUD_STATUS]);
        if(empty($cloudInfo) || empty($cloudInfo->Description)) {
            $cloudStatus = 'error';
            $cloudValue = time();
        }
        else {
            $cloudStatus = $cloudInfo->Description;
            $cloudValue = $cloudInfo->InfoValue;
        }
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/cloud/label'),
            'v' => Yii::t('app/message', 'ele/system/info/cloud/value', [
                'status' => Yii::t('app', 'cloud-' . $cloudStatus),
                'time' => date('Y-m-d H:i:s', $cloudValue)
            ]),
        ];

        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/support/label'),
            'v' => is_null(Yii::$app->base->support) ? Yii::t('app', 'close') : Yii::t('app', 'open')
        ];
        $bi = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_MYSQL_BACKUP_INFO]);
        if(!empty($bi) && !empty($bi->Description))
            $ret[] = [
                'k' => Yii::t('app/message', 'ele/system/info/dbbackup/label'),
                'v' => Yii::t('app', 'db_backup_success') . pathinfo($bi->Description, PATHINFO_FILENAME) . ")"
            ];
        else
            $ret[] = [
                'k' => Yii::t('app/message', 'ele/system/info/dbbackup/label'),
                'v' => Yii::t('app', 'db_backup_failed')
            ];

        $tmp = [];
        $events = VEvent::find()->all();
        if(count($events) == 0)
            $tmp[] = Yii::t('app', 'unset');
        else {
            foreach ($events as $event) {
                $item['Name'] = $event->Name;
                $item['StartTaskName'] = is_null($event->StartTaskName) ?  '「' . Yii::t('app', 'unset') .'」': $event->StartTaskName;
                $item['EndTaskName'] = is_null($event->EndTaskName) ?  '「' . Yii::t('app', 'unset') .'」': $event->EndTaskName;
                $tmp[] = Yii::t('app/message', 'ele/system/info/event/value', $item);
            }
        }
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/event/label'),
            'v' => $tmp
        ];

        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/logcnt/label'),
            'v' => Yii::t('app/message', 'ele/system/info/logcnt/value', [
                'dLogCnt' => VDeviceLog::find()->count(),
                'sLogCnt' => SystemLog::find()->count()
            ]),
        ];

        $query = DeviceStatus::find()->where(['not', ['IpAddress' => NULL]]);
        $ret[] = [
            'k' => Yii::t('app/message', 'ele/system/info/devcnt/label'),
            'v' => Yii::t('app/message', 'ele/system/info/devcnt/value', [
                'all' => DeviceList::find()->count(),
                'online' => $query->count(),
                'poweroff' => DeviceStatus::find()->where(['not', ['IpAddress' => NULL]])->andWhere(['State_SysPower' => 0])->count(),
                'poweron' => DeviceStatus::find()->where(['not', ['IpAddress' => NULL]])->andWhere(['State_SysPower' => 1])->count(),
                'disabled' => DeviceStatus::find()->where(['not', ['IpAddress' => NULL]])->andWhere(['State_SysPower' => 2])->count(),
            ]),
        ];
        return $ret;
    }

    public function actionInfo(){
        $ret = array_merge($this->idsteServerInfo(), $this->systemBaseInfo());
        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    public function actionDbFileInfo() {
        $bi = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_MYSQL_BACKUP_INFO]);
        if(empty($bi) || empty($bi->Description))
            return $this->response(Error::STATUS_MYSQL_BACKUP_NOT_EXISTS);
        $filepath = Common::MYSQL_BACKUP_DIR . '/' . $bi->Description;
        if(!file_exists($filepath))
            return $this->response(Error::STATUS_SUCCESS, ['filename' => null]);
        return $this->response(Error::STATUS_SUCCESS, [
            'filename' => $bi->Description,
            'datetime' => date('Y-m-d H:i:s', filectime($filepath))
        ]);
    }

    public function actionDbFileDownload() {
        if (Yii::$app->getUser()->identity->cloudLogin) {
            return $this->response(Error::STATUS_FAILED);
        }
        $bi = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_MYSQL_BACKUP_INFO]);
        if(empty($bi) || empty($bi->Description))
            return $this->response(Error::STATUS_MYSQL_BACKUP_NOT_EXISTS);
        $filepath = Common::MYSQL_BACKUP_DIR . '/' . $bi->Description;
        if(!file_exists($filepath))
            return $this->response(Error::STATUS_MYSQL_BACKUP_NOT_EXISTS);
        return Yii::$app->response->sendFile($filepath);
    }

    public function actionIconUpload () {
        $location = Yii::$app->request->get('location');
        if (empty($location))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $model = new uploadFile();
        $model->imageFile = UploadedFile::getInstanceByName('image-file');
        if(empty($model->imageFile)) {
            return $this->response(Error::STATUS_FAILED, ['error' => $model->errors]);
        }
        $ret = $model->uploadImage($location);
        return $this->response($ret);
    }

    public function actionIconReset () {
        $data = Yii::$app->request->post('data');
        if (empty($data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $location = $data['location'];
        if (empty($location))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $folder = Common::IMAGE_STORAGE_DIR;
        $srcPath = '';
        $targetPath = '';
        switch ($location) {
            case 'header':
                $srcPath = $folder . 'logo__default.png';
                $targetPath = $folder . 'logo.png';
                break;
            case 'login':
                $srcPath = $folder . 'idste_logo__default.png';
                $targetPath = $folder . 'idste_logo.png';
                break;
            case 'tab':
                $srcPath = $folder . 'favicon__default.ico';
                $targetPath = $folder . 'favicon.ico';
                break;
        }
        $result = shell_exec("cp $srcPath $targetPath");
        if (!empty($result)) {
            return $this->response(Error::STATUS_FAILED, ['data' => $result]);
        }
        return $this->response(Error::STATUS_SUCCESS);
    }
}
