<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/04/27 15:11
 */
namespace api\controllers;

use common\models\PlaySessions;
use common\models\TaskTemplate;
use Yii;
use yii\db\Exception;
use common\classes\Error;
use common\models\VTimedSessionInfo;
use common\models\VSession;
use common\models\BaseInfo;
use common\classes\Common;

/**
 * Task Template controller
 */
class TemplateController extends SessionBaseController
{
    /**
     * @param $id string template's id
     * @return array
     */
    public function actionActive($id, $active)
    {
        $template = TaskTemplate::findOne(['idTaskTemplate' => $id]);
        if (is_null($template))
            return $this->response(Error::STATUS_TEMPLATE_NOT_EXIST);
        $this->logTitle['title1'] = 'ON';
        $this->logTitle['title2'] = ($id == 0 && empty($template->TemplateName)) ? "ID:0" : $template->TemplateName;
        if($active == 1)
            Yii::$app->base->activeTemplateId = $id;
        if($active == 0 && Yii::$app->base->activeTemplateId == $id){
            Yii::$app->base->activeTemplateId = -1;
            $this->logTitle['title1'] = 'OFF';
        }
        return $this->response(Error::STATUS_SUCCESS, ['active' => Yii::$app->base->activeTemplateId]);
    }

    /**
     * @return array
     */
    public function actionNew() {
        $template = new TaskTemplate();
        if($template->load(Yii::$app->request->post(), 'data') && $template->save()) {
            $this->logTitle['title1'] = $template->TemplateName;
            return $this->response(Error::STATUS_SUCCESS, $template);
        }
        else
            return $this->response(Error::STATUS_FAILED);
    }

    /**
     * @param $id template id
     * @return array
     */
    public function actionUpdate($id) {
        $template = TaskTemplate::findOne(['idTaskTemplate' => $id]);
        if(is_null($template))
            return $this->response(Error::STATUS_MODE_NOT_EXIST);
        if($template->load(Yii::$app->request->post(), 'data') && $template->save()) {
            $this->logTitle['title1'] = $template->TemplateName;
            return $this->response(Error::STATUS_SUCCESS, $template);
        }
        else
            return $this->response(Error::STATUS_FAILED);
    }

    /**
     * @param $id template id
     * @return array
     */
    public function actionDelete($id) {
        try{
            if($id == 0)
                return $this->response(Error::STATUS_FORBID);
            $template = TaskTemplate::findOne(['idTaskTemplate' => $id]);
            if(empty($template))
                return $this->response(Error::STATUS_TEMPLATE_NOT_EXIST);
            $this->logTitle['title1'] = $template->TemplateName;
            $count = VTimedSessionInfo::find()
                ->where(['Template' => $id, 'BrdMode' => null])
                ->count();
            if($count > 0)
                return $this->response(Error::STATUS_TEMPLATE_NOT_EMPTY);
            Yii::$app->db->createCommand("update `PlaySessions` set `Template`=0 where `BrdMode` is not null and `Template`=$id;")
                ->execute();
            Yii::$app->db->createCommand()
                ->delete('TaskTemplate', ['idTaskTemplate' => $id])
                ->execute();
            if(Yii::$app->base->activeTemplateId == $id)
                Yii::$app->base->activeTemplateId = 0;
            return $this->response(Error::STATUS_SUCCESS);
        }catch (Exception $e){}
        return $this->response(Error::STATUS_FAILED);
    }


    /**
     * @param $id TaskTemplate id
     */
    public function actionDuplicate($id) {
        $old = TaskTemplate::findOne(['idTaskTemplate' => $id]);
        if(empty($old))
            return $this->response(Error::STATUS_TEMPLATE_NOT_EXIST);
        $this->logTitle['title1'] = $old->TemplateName;
        Yii::$app->db->createCommand("call sp_copy_template($id);")
            ->execute();
        $new = TaskTemplate::find()->orderBy(['idTaskTemplate' => SORT_DESC])
            ->one();
        $this->logTitle['title2'] = $new->TemplateName;
        return $this->response(Error::STATUS_SUCCESS, $new);
    }

    /**
     * get session list for special template
     */
    public function actionSessionList($id)
    {
        if (!is_numeric($id))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $data = Yii::$app->getRequest()->post('data');
        $query = VTimedSessionInfo::find()->where(['Template' => $id, 'BrdMode' => null])
            ->orderBy(['PlayTime' => SORT_ASC]);
        if (($this->serverPri & 0x06) == 0)
            $query = $query->andWhere(['Type' => 'ctrl']);
        return $this->getSessionList($query, $data);
    }
}
