<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/08/16 09:23
 */
namespace api\controllers;

use common\classes\Common;
use common\models\BaseInfo;
use common\models\SectionTable;
use Yii;
use common\classes\Error;

/**
 * Timetable controller
 */
class TimetableConfigController extends BaseController
{
    public function actionSectionList(){
        $data = Yii::$app->getRequest()->post('data');
        $query = SectionTable::find();
        if($query == null)
            return $this->response(Error::STATUS_SUCCESS, []);
        return $this->list($query, $data);
    }

    public function actionSectionDelete(){
        $data = Yii::$app->getRequest()->post('data');
        if(empty($data) || !array_key_exists('name', $data) || empty($data['zone']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $this->logTitle['title1'] = $data['zone'];
        $this->logTitle['title2'] = $data['name'];
        $ret = SectionTable::deleteAll(['zone' => $data['zone'], 'name' => $data['name']]);
        if($ret > 0)
            return $this->response(Error::STATUS_SUCCESS);
        return $this->response(Error::STATUS_FAILED);
    }

    public function actionSectionNew(){
        $data = Yii::$app->getRequest()->post('data');
        if(empty($data) || !array_key_exists('name', $data) || empty($data['zone']) || empty($data['start']) || empty($data['end']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $model = new SectionTable();
        $this->logTitle['title1'] = $data['zone'];
        $this->logTitle['title2'] = $data['name'];
        $this->logDetail['ele/start'] = $data['start'];
        $this->logDetail['ele/end'] = $data['end'];
        if($model->load(Yii::$app->request->post(), 'data') && $model->save()) {
            return $this->response(Error::STATUS_SUCCESS, $model);
        }
        return $this->response(Error::STATUS_FAILED);
    }

    public function actionGetConfig() {
        $ttEn = Yii::$app->base->timetableState;
        $ret = ['enable' => $ttEn];
        do{
            $bi = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_TIMETABLE_OPEN_DATE]);
            if(empty($bi) || empty($bi->Description))
                $ret['openDate'] = null;
            else
                $ret['openDate'] = $bi->Description;
            $bi = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_TIMETABLE_TERM_NAME]);
            if(empty($bi) || empty($bi->Description))
                $ret['termName'] = null;
            else
                $ret['termName'] = $bi->Description;

            $bi = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_TIMETABLE_AUTO_PWRCTRL]);
            if(empty($bi) || $bi->InfoValue != 1) {
                $ret['autoPwrCtrl'] = 0;
                $ret['pwronAhead'] = 0;
                $ret['pwroffDelay'] = 0;
                break;
            }
            else
                $ret['autoPwrCtrl'] = 1;

            $bi = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_TIMETABLE_PWRON_AHEAD]);
            if(empty($bi) || $bi->InfoValue == 0)
                $ret['pwronAhead'] = 0;
            else
                $ret['pwronAhead'] = $bi->InfoValue;

            $bi = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_TIMETABLE_PWROFF_DELAY]);
            if(empty($bi) || $bi->InfoValue == 0)
                $ret['pwroffDelay'] = 0;
            else
                $ret['pwroffDelay'] = $bi->InfoValue;
        }while(false);
        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    public function actionSetConfig() {
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !array_key_exists('enable', $data) || !is_numeric($data['enable']))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $this->logTitle['title1'] = $data['enable'] == 1 ? 'ENABLE' : 'DISABLE';
        $ttEn = Yii::$app->base->timetableState;
        if($ttEn == 1 && $data['enable'] != 1) {
            Yii::$app->base->timetableState = 0;
            return $this->actionGetConfig();
        }
        Yii::$app->base->timetableState = 1;
        if(array_key_exists('termName', $data) && !empty($data['termName']) && !empty($data['termName']))
            $tmp = $data['termName'];
        else
            $tmp = null;
        Yii::$app->base->baseSet(Common::BASEINFO_TIMETABLE_TERM_NAME, $tmp, 0);
        $this->logDetail['ele/TermName'] = $tmp;

        if(array_key_exists('autoPwrCtrl', $data) && $data['autoPwrCtrl'] === 1)
            $tmp = 1;
        else
            $tmp = 0;
        Yii::$app->base->baseSet(Common::BASEINFO_TIMETABLE_AUTO_PWRCTRL, "EnableAutoPwrCtrl", $tmp);
        $this->logDetail['ele/EnableAutoPwrCtrl'] = ($tmp == 1 ? "ON" : "OFF");

        if(array_key_exists('openDate', $data) && !empty($data['openDate']) && preg_match('/[0-9]{4}-[0-9]{1,2}-[0-9]{1,2}/', $data['openDate']))
            $tmp = $data['openDate'];
        else
            $tmp = null;
        Yii::$app->base->baseSet(Common::BASEINFO_TIMETABLE_OPEN_DATE, $tmp, empty($tmp) ? 0 : 1);
        $this->logDetail['ele/OpenDate'] = $tmp;

        if(array_key_exists('pwronAhead', $data) && is_numeric($data['pwronAhead']))
            $tmp = $data['pwronAhead'];
        else
            $tmp = 0;
        Yii::$app->base->baseSet(Common::BASEINFO_TIMETABLE_PWRON_AHEAD, "PwronAhead", $tmp);
        $this->logDetail['ele/PwronAhead'] = $tmp;

        if(array_key_exists('pwroffDelay', $data) && is_numeric($data['pwroffDelay']))
            $tmp = $data['pwroffDelay'];
        else
            $tmp = 0;
        Yii::$app->base->baseSet(Common::BASEINFO_TIMETABLE_PWROFF_DELAY, "PwroffDelay", $tmp);
        $this->logDetail['ele/PwroffDelay'] = $tmp;
        return $this->actionGetConfig();
    }
}
