<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/4/3 11:58
 */

namespace common\components;

use common\classes\Common;
use common\models\AlternateDate;
use common\models\BaseInfo;
use Yii;

/**
 * called by workerman deamon progress only
 *
 * @property string $uuid
 * @property string $cloudServer
 * @property string $accessToken
 * @property string $secretToken
 * @property string $support
 * @property string $timezone
 * @property string $version
 * @property string $activeTemplateId
 * @property string $weekday
 * @property string $date
 * @property string $baseSet
 * @property string $timetableState
 */
class Base extends yii\base\Component
{
    public function getCloudServer(){
        $info = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_CLOUD_DOMAIN]);
        if (isset($info) && !empty($info->Description)) {
            $ret = $info->Description;
        }
        else
            $ret = 'www.idste.net';
        return $ret;
    }

    public function getAccessToken(){
        $info = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_CLOUD_ACCESS_TOKEN]);
        if (isset($info) && !empty($info->Description))
            return $info->Description;
        else
            return null;
    }

    public function getSecretToken(){
        $info = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_CLOUD_SECRET_TOKEN]);
        if (isset($info) && !empty($info->Description))
            return $info->Description;
        else
            return null;
    }

    public function getUuid() {
        $info = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_SCHOOL_UUID]);
        if (isset($info) && !empty($info->Description) && strlen($info->Description) >= 32) {
            $ret = $info->Description;
        }
        else
            $ret = null;
        return $ret;
    }

    public function getSupport(){
        $info = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_SUPPORT_STATE]);
        if (empty($info))
            return null;
        if(isset($info->Description))
            return $info->Description;
        BaseInfo::deleteAll(['InfomationID' => Common::BASEINFO_SUPPORT_STATE]);
        return null;
    }

    public function getActiveTemplateId() {
        $info = BaseInfo::findOne(['InfomationID' => Common::BASE_ENABLED_TEMPLATE_ID]);
        if(!is_null($info) && isset($info->InfoValue) && is_numeric($info->InfoValue))
            return $info->InfoValue;
        return -1;
    }

    public function setActiveTemplateId($value){
        $model = BaseInfo::findOne(['InfomationID' => Common::BASE_ENABLED_TEMPLATE_ID]);
        if(empty($model)) {
            $model = new BaseInfo();
            $model->InfomationID = Common::BASE_ENABLED_TEMPLATE_ID;
        }
        $model->InfoValue = $value;
        return $model->save();
    }

    public function getTimezone(){
        $zone = shell_exec('cat /etc/timezone');
        $zone = empty($zone) ? null : trim($zone);
        if(empty($zone) || !file_exists(Common::ZONEINFO_PATH . $zone)) {
            $zone = 'Asia/Chongqing';
            $this->setTimezone($zone);
        }
        return $zone;
    }

    public function setTimezone($value) {
        if(!file_exists(Common::ZONEINFO_PATH . $value))
            return false;
        system('timedatectl set-timezone ' . $value, $ret);
        return ($ret == 0);
    }

    public function getVersion() {
        $info = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_SERVER_VERSION]);
        if(!is_null($info) && !empty($info->Description))
            return $info->Description;
        return "unknow";
    }

    public function getWeekday(){
        $alterData = AlternateDate::findOne(['SpecialDate' => date('Y-n-j')]);
        if(!empty($alterData))
            $weekday = $alterData->AlteredScheduleDay;
        else{
            $weekday = `date +%u` - 1;
        }
        return $weekday;
    }

    public function getTimetableState(){
        $info = BaseInfo::findOne(['InfomationID' => Common::BASEINFO_ENABLE_TIMETABLE]);
        if(is_null($info) || empty($info->InfoValue))
            return 0;
        return 1;
    }

    public function setTimetableState($state){
        if($state == 1)
            $on = 1;
        else
            $on = 0;
        $this->baseSet(Common::BASEINFO_ENABLE_TIMETABLE, "EnableTimetable", $on);
    }

    public function getDate(){
        return trim(`date +'%Y-%m-%d %T'`);
    }

    public function baseSet($id, $description, $value) {
        $model = BaseInfo::findOne(['InfomationID' => $id]);
        if(empty($model)) {
            $model = new BaseInfo();
            $model->InfomationID = $id;
        }
        $model->Description = $description;
        $model->InfoValue = $value;
        $model->save();
    }

    public function __get($name)
    {
        if(in_array($name, get_class_vars(self::className())))
            return $this->$name;
        $method = 'get' . ucfirst($name);
        if(!method_exists($this, $method))
            throwException('method or parameter not exists');
        return $this->$method();
    }


    public function __set($name, $value)
    {
        if(in_array($name, get_class_vars(self::className())))
            return $this->$name;
        $method = 'set' . ucfirst($name);
        if(!method_exists($this, $method))
            throwException('method or parameter not exists');
        return $this->$method($value);
    }
}
