<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/4/3 11:58
 */

namespace common\components;

use common\classes\Error;
use common\classes\Common;
use common\models\MediaFiles;
use Yii;

class Media extends yii\base\Component
{
    public function info($filename) {
        if(!file_exists($filename))
            return null;
        $cmd = 'ffprobe -v quiet -print_format json -show_format -show_streams ' . $filename;
        $output = shell_exec($cmd);
        if(empty($output))
            return null;
        $streams = json_decode($output, true);
        if(empty($streams))
            return null;
        $ret = [
            'audio' => null,
            'video' => null,
            'format' => [
                'duration' => 0,
                'bit_rate' => 0
            ]
        ];
        try {
            if (isset($streams['format'])) {
                $ret['format']['duration'] = isset($streams['format']['duration']) ? floor($streams['format']['duration']) : 0;
                $ret['format']['bit_rate'] = isset($streams['format']['bit_rate']) ? floor($streams['format']['bit_rate'] / 1000) : 0;
            }
            foreach ($streams['streams'] as $stream) {
                if ($stream['codec_type'] == 'audio') {
                    if(isset($stream['codec_name']))
                        $codec = $stream['codec_name'];
                    else if(isset($stream['codec_tag_string']))
                        $codec = $stream['codec_tag_string'];
                    else
                        $codec = 'unknow';
                    $ret['audio'] = [
                        'codec_name' => strtolower(trim($codec)),
                        'sample_rate' => trim($stream['sample_rate']),
                        'channels' => trim($stream['channels'])
                    ];
                } else if ($stream['codec_type'] == 'video') {
                    $tmp = explode('/', $stream['avg_frame_rate']);
                    $frame_rate = floor($tmp[0] / $tmp[1]);
                    if(isset($stream['codec_name']))
                        $codec = $stream['codec_name'];
                    else if(isset($stream['codec_long_name'])) {
                        $stream['codec_long_name'] = strtolower($stream['codec_long_name']);
                        if(preg_match('/264/', $stream['codec_long_name']))
                            $codec = 'h264';
                        else if(preg_match('/mpeg-4/', $stream['codec_long_name']))
                            $codec = 'mpeg4';
                    }
                    else {
                        $codec = 'unknow';
                    }
                    $ret['video'] = [
                        'codec_name' => strtolower(trim($codec)),
                        'width' => trim($stream['width']),
                        'height' => trim($stream['height']),
                        'frame_rate' => trim($frame_rate)
                    ];
                }
            }
        }catch (\Exception $e){}
        return $ret;
    }

    /**
     * audio codec check
     * @param $source
     * @param $streams
     * @return int
     */
    private function audioCodecCheck($source, &$streams) {
        if(empty($streams['audio']) || !file_exists($source))
            return Error::STATUS_FAILED;
        if(array_key_exists($streams['audio']['codec_name'], Common::AUDIO_CODEC))
            return Error::STATUS_SUCCESS;

        if($streams['audio']['codec_name'] !== 'mp3')
            return Error::STATUS_MEDIA_CODEC_NOT_SUPPORT;
        if($streams['audio']['sample_rate'] < 32000 || $streams['audio']['sample_rate'] > 96000) {
            //convert to mp3 first
            return Error::STATUS_MEDIA_SAMPLER_RATE_ERROR;
        }
        $tmpFile = Common::MEDIA_TEMPORARA_DIR . Yii::$app->security->generateRandomString(16);
        $cmd = "audio_convert $source $tmpFile";//convert to celt codec
        system($cmd, $ret);
        if($ret != 0)
            return Error::STATUS_MEDIA_FILE_SAVE_ERROR;
        shell_exec("mv $tmpFile $source");
        $streams['audio']['codec_name'] = 'celt';
        return Error::STATUS_SUCCESS;
    }

    /**
     * video codec check
     * @param $source
     * @param $streams
     * @return int
     */
    private function videoCodecCheck($source, &$streams){
        if(empty($streams['video']) || !file_exists($source))
            return Error::STATUS_FAILED;
        if(!empty($streams['audio']) && !array_key_exists($streams['audio']['codec_name'], Common::VIDEO_CODEC['audio']))
            return Error::STATUS_MEDIA_CODEC_NOT_SUPPORT;
        if(!array_key_exists($streams['video']['codec_name'], Common::VIDEO_CODEC['video']))
            return Error::STATUS_MEDIA_CODEC_NOT_SUPPORT;
        return Error::STATUS_SUCCESS;
    }

    public function codecCheck($source, &$streams){
        $ret = Error::STATUS_FAILED;
        try {
            if (isset($streams['video']))
                $ret = $this->videoCodecCheck($source, $streams);
            else if (isset($streams['audio']))
                $ret = $this->audioCodecCheck($source, $streams);
        }catch (\Exception $e){}
        return $ret;
    }

    /**
     * save file and create MediaFiles model
     * @param string $filename
     * @param string $filepath
     * @param $streams
     * @return MediaFiles
     */
    public function saveAs(string $filename, string $filepath, &$streams, $parent){
        $media = new MediaFiles();
        $arr = explode('.', $filename);
        if(count($arr) < 2)
            $media->FileName = $filename;
        else {
            $media->FileName = implode('.', array_slice($arr, 0, -1));
        }
        $media->TotalTime = $streams['format']['duration'];
        $media->BitRate = $streams['format']['bit_rate'];
        $media->Parent = $parent;
        if (isset($streams['video'])) {
            $media->Width = $streams['video']['width'];
            $media->Height = $streams['video']['height'];
            $media->FrameRate = $streams['video']['frame_rate'];
            $media->FileFmt = 3;
            $media->VedioCodec = Common::VIDEO_CODEC['video'][$streams['video']['codec_name']];
            if (isset($streams['audio'])) {
                $media->AudioCodec = Common::VIDEO_CODEC['audio'][$streams['audio']['codec_name']];
                $media->Channels = $streams['audio']['channels'];
                $media->SampleRate = $streams['audio']['sample_rate'];
            }
            else {
                $media->AudioCodec = -1;
                $media->Channels = 0;
                $media->SampleRate = 0;
            }
        }
        else if (isset($streams['audio'])) {
            $media->Width = 0;
            $media->Height = 0;
            $media->FrameRate = 0;
            $media->FileFmt = 0;
            $media->VedioCodec = -1;
            $media->AudioCodec = Common::AUDIO_CODEC[$streams['audio']['codec_name']];
            $media->Channels = $streams['audio']['channels'];
            $media->SampleRate = $streams['audio']['sample_rate'];
        }
        $media->FileSize = filesize($filepath);
        if($media->save()) {
            $media->FilePath = Common::MEDIA_STORAGE_DIR . $media->MediaID;
            shell_exec("mv $filepath " . $media->FilePath);
            if($media->save())
                return $media;
        }
        return null;
    }
}