<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/4/3 11:58
 */

namespace common\components;

use common\classes\Common;
use common\classes\Error;
use common\models\Users;
use Yii;

class UdpService extends yii\base\Component
{
    const LOGIN_ACTION_NAME = [
        'login' => '账号密码',
        'unionid-login' => '微信登录',
        'oauth' => 'oauth单点登录'
    ];

    /**
     * @param $port udp port
     * @param $msg msg body encoded by json_encode
     * @param bool $waitRet
     * @param int $timeout
     * @return bool|string
     */
    public static function send($port, $msg, $waitRet = false, $timeout = 5){
        $fp = stream_socket_client("udp://127.0.0.1:" . $port, $errno, $errstr, 30);
        if(is_array($msg))
            $msg = json_encode($msg, JSON_UNESCAPED_UNICODE);
        fwrite($fp, $msg);
        $ret = true;
        if($waitRet) {
            stream_set_timeout($fp, $timeout);
            $ret = fread($fp, 4096);
            if(empty($ret))
                return Error::jsonResponse(Error::STATUS_FAILED);
            $ret = unserialize($ret);
            if(empty($ret) || !isset($ret['status']))
                return Error::jsonResponse(Error::STATUS_FAILED);
            $ret['message'] = Yii::t('app/error', $ret['status']);
        }
        fclose($fp);
        return $ret;
    }

    private function getUserName($user) {
        if(isset($user->User_Desc))
            return $user->User_Desc;
        if(isset($user->User_Name))
            return $user->User_Name;
        return "UNKNOW";
    }
    /**
     * send login message
     * @param Users $user
     * @param string $action
     */
    public function login(Users $user, $action){
        if(!isset(self::LOGIN_ACTION_NAME[$action]))
            return;
        $body = [
            'type' => 'login_info',
            'uid' => $user->User_ID,
            'msg' => [
                'keyword1' => $this->getUserName($user),
                'keyword2' => 'iDste控制台',
                'keyword3' => Yii::$app->base->date,
                'remark' => "登录方式：". self::LOGIN_ACTION_NAME[$action]
            ],
            'level' => Common::WECHAT_MSG_LEVEL_INFO
        ];
        $port =  Common::MSG_WORKER_PORT_START + rand(0,  Common::MSG_WORKER_COUNT - 1);
        self::send($port, $body);
        //new login message
        $body = [
            'uid' => $user->User_ID,
            'action' => $action
        ];
        for($i = 0; $i < Common::WS_WORKER_COUNT; $i++) {
            $port =  Common::WS_WORKER_PORT_START + $i;
            self::send($port, $body);
        }

    }

    public function bind($user, $state){
        $body = [
            'type' => 'bind_info',
            'uid' => $user->User_ID,
            'msg' => [
                'keyword1' => $this->getUserName($user),
                'keyword2' => Yii::$app->base->date,
                'remark' => "状态：". $state
            ],
            'level' => Common::WECHAT_MSG_LEVEL_INFO
        ];
        $port =  Common::MSG_WORKER_PORT_START + rand(0,  Common::MSG_WORKER_COUNT - 1);
        self::send($port, $body);
    }

    /**
     * @param $state "reboot" or "shutdown"
     */
    public function systemStateChange($state) {
        $user = Users::findOne(['User_ID' => 0]);
        if(is_null($user) || empty($user->UnionId))
            return false;
        if($state == 'shutdown') {
            $msg = '关闭操作系统';
            $notice = '如果不是您本人执行的"关机"操作，请关注系统是否出现故障';
        }
        else if($state == 'reboot'){
            $msg = '重启操作系统';
            $notice = '如果不是您本人执行的"重启"操作，请关注系统是否出现故障';
        }
        else{
            $msg = '重启iDste服务';
            $notice = '如果不是您本人执行的"服务重启"操作，请关注系统是否出现故障';
        }
        $body = [
            'type' => 'service_error',
            'uid' => 0,
            'msg' => [
                'keyword1' => $msg,
                'keyword2' => Yii::$app->base->date,
                'keyword3' => $notice
            ],
            'level' => Common::WECHAT_MSG_LEVEL_INFO
        ];
        $port =  Common::MSG_WORKER_PORT_START + rand(0,  Common::MSG_WORKER_COUNT - 1);
        self::send($port, $body);
    }

    /**
     * wechat message
     * @param $state "on" or "off"
     * @param $initiator "client" or "remote"
     */
    public function supportCtrlMsg($state, $initiator = 'remote') {
        $user = Users::findOne(['User_ID' => 0]);
        if(is_null($user) || empty($user->UnionId))
            return false;
        $remark = "\n请关注系统是否出现故障";
        if($state == 'on') {
            $msg = '发起远程维护';
            $notice = "由客户端发起";
        }
        else{
            $msg = '停止远程维护';
            if($initiator ==  'remote')
                $notice = "操作由远程维护者发起";
            else
                $notice = '由客户端发起';
        }
        $body = [
            'type' => 'service_error',
            'uid' => 0,
            'msg' => [
                'keyword1' => $msg,
                'keyword2' => Yii::$app->base->date,
                'keyword3' => $notice,
                'remark' => $remark
            ],
            'level' => Common::WECHAT_MSG_LEVEL_WARN
        ];
        $port =  Common::MSG_WORKER_PORT_START + rand(0,  Common::MSG_WORKER_COUNT - 1);
        self::send($port, $body);
    }

    public function projectorMsg ($deviceName, $watermark, $usedtime) {
        $user = Users::findOne(['User_ID' => 0]);
        if(is_null($user) || empty($user->UnionId))
            return false;
        $remark = "\n请及时更换投影灯泡。投影仪灯泡配置：\n灯泡寿命：" . floor($watermark / 3600) . "小时\n已用时长：" . floor($usedtime / 3600) . "小时\n";
        $body = [
            'type' => 'device_notice',
            'uid' => 0,
            'msg' => [
                'keyword1' => $deviceName,
                'keyword2' => '投影灯泡告警',
                'keyword3' => '默认',
                'keyword4' => Yii::$app->base->date,
                'remark' => $remark
            ],
            'level' => Common::WECHAT_MSG_LEVEL_WARN
        ];
        $port =  Common::MSG_WORKER_PORT_START + rand(0,  Common::MSG_WORKER_COUNT - 1);
        self::send($port, $body);
    }

    public function mysqldumpErrorMsg ($uid) {
        $user = Users::findOne(['User_ID' => 0]);
        if(is_null($user) || empty($user->UnionId))
            return false;
        $body = [
            'type' => 'mysql_notice',
            'uid' => $uid,
            'msg' => [
                'keyword1' => '数据库备份失败',
                'keyword2' =>  Yii::$app->base->date,
                'keyword3' => "请检查网络或数据库状态",
                'remark' => ""
            ],
            'level' => Common::WECHAT_MSG_LEVEL_WARN
        ];
        $port =  Common::MSG_WORKER_PORT_START + rand(0,  Common::MSG_WORKER_COUNT - 1);
        self::send($port, $body);
    }

    public function system($controller, $action, $param = null, $timeout = 10) {
        $req = [
            'controller' => $controller,
            'action' => $action,
            'param' => $param
        ];
        $msg = json_encode($req, JSON_UNESCAPED_UNICODE);
        $waitRet = ($timeout > 0);
        return self::send(Common::SYSTEM_DEAMON_PORT, $msg, $waitRet, $timeout);
    }

    //$type can be ['offline_recovery_warning', 'offline_warning']
    public function devOfflineWarningMsg ($type, $uid, $offlineCnt, $onlineCnt, $watermark)
    {
        $user = Users::findOne(['User_ID' => 0]);
        if (is_null($user) || empty($user->UnionId))
            return false;
        $body = [
            'type' => $type,
            'uid' => $uid,
            'msg' => [
                'keyword1' => Yii::$app->base->date,
                'keyword2' => "自动检测",
                'keyword3' => "离线$offlineCnt 在线$onlineCnt 阈值$watermark",
                'remark' => ""
            ],
            'level' => Common::WECHAT_MSG_LEVEL_WARN
        ];
        $port =  Common::MSG_WORKER_PORT_START + rand(0,  Common::MSG_WORKER_COUNT - 1);
        self::send($port, $body);
    }

    //$type can be ['offline_recovery_warning', 'offline_warning']
    public function devStatisticMsg ($uid, $poweronCnt, $poweroffCnt, $disableCnt, $offlineCnt)
    {
        $user = Users::findOne(['User_ID' => 0]);
        if (is_null($user) || empty($user->UnionId))
            return false;
        $body = [
            'type' => 'device_statistics',
            'uid' => $uid,
            'msg' => [
                'keyword1' => "开机$poweronCnt 关机$poweroffCnt 禁用$disableCnt 离线$offlineCnt",
                'keyword2' => Yii::$app->base->date,
                'remark' => ""
            ],
            'level' => Common::WECHAT_MSG_LEVEL_WARN
        ];
        $port =  Common::MSG_WORKER_PORT_START + rand(0,  Common::MSG_WORKER_COUNT - 1);
        self::send($port, $body);
    }
    /**
     * wechat message
     * @param $state "on" or "off"
     * @param $initiator "client" or "remote"
     */
    public function timetableSyncError() {
        $user = Users::findOne(['User_ID' => 0]);
        if(is_null($user) || empty($user->UnionId))
            return false;
        $remark = "\n请关注系统是否出现故障";
        $body = [
            'type' => 'service_error',
            'uid' => 0,
            'msg' => [
                'keyword1' => "课表同步失败",
                'keyword2' => Yii::$app->base->date,
                'keyword3' => "请检查网络或数据库状态",
                'remark' => $remark
            ],
            'level' => Common::WECHAT_MSG_LEVEL_WARN
        ];
        $port =  Common::MSG_WORKER_PORT_START + rand(0,  Common::MSG_WORKER_COUNT - 1);
        self::send($port, $body);
    }
}
