<?php
/**
 * Created by PhpStorm.
 * User: haijun(xu_415@126.com)
 * Date: 2018/1/15
 * Time: 22:11
 */

namespace common\models;

use common\classes\Common;
use Yii;
use yii\db\Exception;

/**
 * Created by PhpStorm.
 * User: haijun(xu_415@126.com)
 * Date: 2017/7/10
 * Time: 23:44
 */
class DeviceConfig extends \yii\base\Model
{
    public $id; //device id
    public $name; //device name
    public $position; //position/description
    public $enable; //enabled
    public $analogBroadcast; //anlog broad cast
    public $prjOn;
    public $pcOn;
    public $micOn;
    public $vgaResolution;
    public $brdTurnoffDelay;
    public $scrnScrollTime;
    public $poweroffDelay;
    public $prjModel;
    public $parityBit;
    public $sendTimes;
    public $sendPeriod;
    public $baudRate;
    public $prjCmdOpen;
    public $prjCmdClose;
    public $prjCmdFromVideo;
    public $prjCmdFromPc;
    public $defaultAvSrc;
    public $prjChannelPc;
    public $prjChannelExtend;
    public $prjChannelPlatform;
    public $prjChannelVideo;
    public $defaultAudioVol;
    public $defaultMicVol;
    public $audioVolKeep;
    public $micVolKeep;
    public $audioAmp;
    public $audioBass;
    public $audioTreble;
    public $micAmp;
    public $micBass;
    public $micTreble;
    public $broadVol;

    public $openMode;
    public $innerHdmiAudioOn;
    public $videoEnhanceOn;
    public $micEnhanceOn;
    public $micEnhanceSrc;
    public $mic1EnhanceAmp;
    public $mic2EnhanceAmp;
    public $mic1MagicPowerOn;
    public $mic2MagicPowerOn;
    public $vgaInnerEDID;
    public $rtspUrl;
    public $rtspSub;
    public $rtspUrl1;
    public $rtspSub1;

    public $MaintainQrcode;
    public $pcIp;
    public $pcPort;
    public $pcMainPasswd;
    public $pcViewPasswd;

    public $ProjWatermark;
    public $ProjUsedTime;

    public $configrationV2;

    public function rules()
    {
        return [
            [['id'], 'required'],
            [['defaultAvSrc', 'openMode', 'vgaResolution', 'prjOn', 'pcOn', 'micOn', 'enable',
                'analogBroadcast', 'innerHdmiAudioOn', 'videoEnhanceOn', 'micEnhanceOn', 'mic1EnhanceAmp', 'mic2EnhanceAmp'], 'required'],  //system config
            [['defaultAudioVol', 'defaultMicVol', 'audioAmp', 'audioBass', 'audioTreble', 'micAmp', 'micBass', 'micTreble', 'broadVol', 'audioVolKeep', 'micVolKeep'], 'required'], //audio config
            [['prjChannelPc', 'prjChannelExtend', 'prjChannelPlatform', 'prjChannelVideo', 'parityBit', 'sendTimes', 'sendPeriod',
                'baudRate', 'scrnScrollTime', 'poweroffDelay'], 'required'], //video config
            [['prjCmdOpen', 'prjCmdClose', 'prjCmdFromVideo', 'prjCmdFromPc', 'prjModel'], 'string',],
            [['prjCmdOpen', 'prjCmdClose', 'prjCmdFromVideo', 'prjCmdFromPc'], 'match', 'pattern' => '/^[0-9A-F]*$/i'],
            [['name', 'position'], 'string', 'max' => 96],
            [['prjModel'], 'string', 'max' => 64],
            [['name', 'position'], 'trim'],
            ['pcIp', 'ip'],
            ['pcPort', 'integer', 'max' => 65535, 'min' => 1],
            [['pcMainPasswd', 'pcViewPasswd'], 'string', 'max' => 32],
            [['brdTurnoffDelay', 'scrnScrollTime', 'poweroffDelay'], 'integer'],
            [['prjOn', 'pcOn', 'micOn', 'enable', 'analogBroadcast', 'innerHdmiAudioOn', 'videoEnhanceOn', 'micEnhanceOn', 'mic1MagicPowerOn', 'mic2MagicPowerOn',
                'prjChannelPc', 'prjChannelExtend', 'prjChannelPlatform', 'prjChannelVideo', 'audioVolKeep', 'micVolKeep', 'vgaInnerEDID'], 'integer', 'max' => 1, 'min' => 0],
            [['openMode', 'audioAmp', 'micAmp', 'defaultAvSrc', 'sendTimes', 'sendPeriod', 'micEnhanceSrc'], 'integer', 'max' => 3, 'min' => 0],
            [['vgaResolution', 'mic1EnhanceAmp', 'mic2EnhanceAmp'], 'integer', 'max' => 7, 'min' => 0],
            [['parityBit'], 'integer', 'max' => 2, 'min' => 0],
            [['defaultAudioVol', 'defaultMicVol', 'broadVol'], 'integer', 'max' => 64, 'min' => 0],
            [['audioBass', 'audioTreble', 'micBass', 'micTreble'], 'integer', 'max' => 15, 'min' => 0],
            [['rtspUrl', 'rtspSub', 'rtspUrl1', 'rtspSub1'], 'string', 'max' => 128],  //camera config
            [['rtspUrl', 'rtspSub', 'rtspUrl1', 'rtspSub1'], 'match', 'pattern' => '/^rtsp:\/\//i'],  //camera config
            [['ProjWatermark', 'ProjUsedTime'], 'integer', 'min' => 0],
        ];
    }

    public function load($data, $formName = null)
    {
        $ret = parent::load($data, $formName); // TODO: Change the autogenerated stub
        if($ret === false)
            return $ret;
        $this->name = mb_strcut($this->name, 0, 32);
        return $this->validate();
    }

    public function save(){
        $ids = explode(',', $this->id);
        if(empty($ids))
            return false;
        $config = $this->formatConfig();
        foreach($ids as $id){
            $vdevice = VDevice::findOne(['id' => $id]);
            $device = DeviceList::findOne(['DevID' => $id]);

            if (empty($this->rtspUrl))
                $device->RtspUrl = null;
            else if (!preg_match('/^rtsp:\/\/\*\*\*:\*\*\*@/i', $this->rtspUrl))
                $device->RtspUrl = $this->rtspUrl;
            if (empty($this->rtspSub))
                $device->RtspSub = null;
            else if (!preg_match('/^rtsp:\/\/\*\*\*:\*\*\*@/i', $this->rtspSub))
                $device->RtspSub = $this->rtspSub;
            if (empty($this->rtspUrl1))
                $device->RtspUrl1 = null;
            else if (!preg_match('/^rtsp:\/\/\*\*\*:\*\*\*@/i', $this->rtspUrl1))
                $device->RtspUrl1 = $this->rtspUrl1;
            if (empty($this->rtspSub1))
                $device->RtspSub1 = null;
            else if (!preg_match('/^rtsp:\/\/\*\*\*:\*\*\*@/i', $this->rtspSub1))
                $device->RtspSub1 = $this->rtspSub1;

            if(is_null($vdevice) || is_null($device))
                continue;
            if ($device->ProjWatermark != $this->ProjWatermark || $device->ProjUsedTime != $this->ProjUsedTime) {
                $device->ProjUsedTime = $this->ProjUsedTime;
                $device->ProjWatermark = $this-> ProjWatermark;
                $device->save();
            }
            $tmp = bin2hex($device->Dev_Configuration);
            if(count($ids) > 1){
                $realCfg = substr($tmp, 0, 256) . substr($config, 256);
                $sql = "call sp_send_dev_cfg_cmd('" . $vdevice->id . "', '" . $vdevice->ipaddress . "','" . $realCfg . "','','');";
            }
            else
                $sql = "call sp_send_dev_cfg_cmd('" . $vdevice->id . "', '" . $vdevice->ipaddress . "','" . $config . "','','');";
            Yii::$app->getDb()->createCommand($sql)->execute();
        }

        if(count($ids) == 1 && !is_null($vdevice) && !is_null($device)){
            $device->Dev_Description = iconv('UTF-8', 'GBK', $this->position);
            $device->Dev_Name = $this->name;

            if($this->pcMainPasswd !== '***')
                $device->PcMainPasswd = $this->pcMainPasswd;
            if($this->pcViewPasswd !== '***')
                $device->PcViewPasswd = $this->pcViewPasswd;
            $device->PcIp = $this->pcIp;
            $device->PcPort = $this->pcPort;
            return $device->save();
        }
        return true;
    }

    private function formatConfig()
    {
        $name = bin2hex(iconv('UTF-8', 'GBK', mb_strcut($this->name, 0, 32)));
        $configArr = $name;
        for ($i = strlen($name); $i < 64; $i += 2)
            $configArr .= '00';
        $position = bin2hex(iconv('UTF-8', 'GBK', mb_strcut($this->position, 0, 96)));
        $configArr .= $position;
        for ($i = strlen($position); $i < 192; $i += 2)
            $configArr.= '00';
        //position 128
        $val = ($this->openMode << 4) + $this->enable;
        $configArr .= sprintf("%02x", $val);

        $val = $this->analogBroadcast & 1;
        $val += ($this->mic1EnhanceAmp & 0x07) <<2;
        $val += ($this->mic2EnhanceAmp & 0x07) <<5;
        $configArr .= sprintf("%02x", $val);
        $val = ($this->prjOn & 1) << 4;
        $val += ($this->pcOn & 1) << 5;
        $val += ($this->micOn & 1) << 6;
        $configArr .= sprintf("%02x", $val);
        $configArr .= sprintf("%02x", $this->vgaResolution);

        //position 132;
        for ($j = 0; $j < 32; $j += 8) {
            $val = ($this->brdTurnoffDelay >> $j) % 256;
            $configArr .= sprintf("%02x", $val);
        }
        for ($j = 0; $j < 32; $j += 8) {
            $val = ($this->scrnScrollTime >> $j) % 256;
            $configArr .= sprintf("%02x", $val);
        }
        for ($j = 0; $j < 32; $j += 8) {
            $val = ($this->poweroffDelay >> $j) % 256;
            $configArr .= sprintf("%02x", $val);
        }
        $mod = bin2hex(iconv('UTF-8', 'GBK', mb_strcut($this->prjModel, 0, 64)));
        $configArr .= $mod;
        for ($j = strlen($mod); $j < 128; $j += 2)
            $configArr .= '00';

        //position 208 proj_cmd_t
        $configArr .= sprintf("%02x", $this->parityBit);
        $configArr .= sprintf("%02x", $this->sendTimes);
        $configArr .= sprintf("%02x", $this->sendPeriod);
        $configArr .= sprintf("%02x", $this->baudRate);
        $len = strlen($this->prjCmdOpen) >> 1;
        $configArr .= sprintf("%02x", $len);
        $configArr .= $this->prjCmdOpen;
        for ($j = strlen($this->prjCmdOpen); $j < 126; $j += 2)
            $configArr .= '00';
        $len = strlen($this->prjCmdClose) >> 1;
        $configArr .= sprintf("%02x", $len);
        $configArr .= $this->prjCmdClose;
        for ($j = strlen($this->prjCmdClose); $j < 126; $j += 2)
            $configArr .= '00';
        $len = strlen($this->prjCmdFromVideo) >> 1;
        $configArr .= sprintf("%02x", $len);
        $configArr .= $this->prjCmdFromVideo;
        for ($j = strlen($this->prjCmdFromVideo); $j < 126; $j += 2)
            $configArr .= '00';
        $len = strlen($this->prjCmdFromPc) >> 1;
        $configArr .= sprintf("%02x", $len);
        $configArr .= $this->prjCmdFromPc;
        for ($j = strlen($this->prjCmdFromPc); $j < 126; $j += 2)
            $configArr .= '00';
        //position 468
        if($this->audioVolKeep === 1)
            $this->defaultAudioVol = 255;
        if($this->micVolKeep == 1)
            $this->defaultMicVol = 255;
        $configArr .= sprintf("%02x", $this->defaultAudioVol);

        $configArr .= sprintf("%02x", $this->defaultMicVol);
        // position 470;
        $configArr .= sprintf("%02x", $this->defaultAvSrc);
        $val = $this->prjChannelPc & 3;
        $val += ($this->prjChannelExtend & 3) << 2;
        $val += ($this->prjChannelPlatform & 3) << 4;
        $val += ($this->prjChannelVideo & 3) << 6;
        $configArr .= sprintf("%02x", $val);
        $configArr .= sprintf("%02x", $this->audioAmp);
        $configArr .= sprintf("%02x", $this->audioBass);
        $configArr .= sprintf("%02x", $this->audioTreble);
        $configArr .= sprintf("%02x", $this->micAmp);
        $configArr .= sprintf("%02x", $this->micBass);
        $configArr .= sprintf("%02x", $this->micTreble);
        $configArr .= sprintf("%02x", $this->broadVol);

        //position 479
        $val = ($this->innerHdmiAudioOn ^ 1) & 1;;
        $val += ($this->videoEnhanceOn & 1) << 1;
        $val += ($this->micEnhanceOn & 1) << 2;
        $val += ($this->micEnhanceSrc & 3) << 5;
        $val += ($this->mic1MagicPowerOn & 1) << 3;
        $val += ($this->mic2MagicPowerOn & 1) << 4;
        $val += ($this->vgaInnerEDID & 1) << 7;
        $configArr .= sprintf("%02x", $val);

        return $configArr;
    }

    public static function readConfig($id)
    {
        Yii::$app->db->createCommand("select device_ctrl(IpAddress, '$id', 55, 0, 0, 0) from DeviceStatus where DeviceID='$id';")->execute();
        sleep(1);
        $device = DeviceList::findOne(['DevID' => $id]);
        if(is_null($device))
            return null;
        $config = new DeviceConfig();
        $config->id = $device->DevID;
        $config->rtspUrl = preg_replace('/rtsp:\/\/(\w+):(\w+)@/i', 'rtsp://***:***@', $device->RtspUrl);
        $config->rtspUrl1 = preg_replace('/rtsp:\/\/(\w+):(\w+)@/i', 'rtsp://***:***@', $device->RtspUrl1);
        $config->rtspSub = preg_replace('/rtsp:\/\/(\w+):(\w+)@/i', 'rtsp://***:***@', $device->RtspSub);
        $config->rtspSub1 = preg_replace('/rtsp:\/\/(\w+):(\w+)@/i', 'rtsp://***:***@', $device->RtspSub1);
        $config->ProjWatermark = $device->ProjWatermark;
        $config->ProjUsedTime = $device->ProjUsedTime;
        $uuid = Yii::$app->base->uuid;
        $identify = "u=$uuid&i=" . $device->DevID;
        $config->MaintainQrcode = 'https://' . Yii::$app->base->getCloudServer() . "/s/oq?$identify";
        $config->pcIp = $device->PcIp;
        $config->pcPort = $device->PcPort;
        $config->pcMainPasswd = empty($device->PcMainPasswd) ? "" : "***";
        $config->pcViewPasswd = empty($device->PcViewPasswd) ? "" : "***";
        if (substr($id, 0, 1) === '5') {
            $config->setScenario('v2');
            $confV2 = str_replace(["\n", ",}", ',]'], ['', '}', ']'], $device->Dev_ConfigurationV2);
            $config->configrationV2 = json_decode($confV2);
            return $config;
        }
        if(empty($device->Dev_Configuration) || strlen($device->Dev_Configuration) < 480)
            return $config;
        $configArr = $device->Dev_Configuration;
        //0-131共132字节为设备配置字段
        for ($i = 0; $i < 32; $i++) {
            if (bin2hex($configArr[$i]) == '00')
                break;
        }
        $config->name = iconv('GBK', 'UTF-8', substr($configArr, 0, $i));
        if (is_null($config->name))
            $config->name = $device->Dev_Name;
        for ($i = 32; $i < 128; $i++) {
            if (bin2hex($configArr[$i]) == '00')
                break;
        }
        $config->position = iconv('GBK', 'UTF-8', substr($configArr, 32, $i - 32));
        if (is_null($config->position))
            $config->position= $device->Dev_Description;
        $i = 128;
        $val = hexdec(bin2hex($configArr[$i++]));
        $config->enable = $val & 0x0f;
        $config->openMode = ($val >> 4);
        $val = hexdec(bin2hex($configArr[$i++]));
        $config->analogBroadcast = $val & 0x01;
        $config->mic1EnhanceAmp = ($val >> 2) & 0x07;
        $config->mic2EnhanceAmp = ($val >> 5) & 0x07;
        $val = hexdec(bin2hex($configArr[$i++]));
        $config->prjOn = ($val >> 4) & 0x01;
        $config->pcOn = ($val >> 5) & 0x01;
        $config->micOn = ($val >> 6) & 0x01;
        $config->vgaResolution = hexdec(bin2hex($configArr[$i++]));
        $i = 132;
        //132-467共336字节为视频配置字段
        $config->brdTurnoffDelay = hexdec(bin2hex($configArr[$i++]));
        $config->brdTurnoffDelay += hexdec(bin2hex($configArr[$i++])) << 8;
        $config->brdTurnoffDelay += hexdec(bin2hex($configArr[$i++])) << 16;
        $config->brdTurnoffDelay += hexdec(bin2hex($configArr[$i++])) << 24;
        $config->scrnScrollTime = hexdec(bin2hex($configArr[$i++]));
        $config->scrnScrollTime += hexdec(bin2hex($configArr[$i++])) << 8;
        $config->scrnScrollTime += hexdec(bin2hex($configArr[$i++])) << 16;
        $config->scrnScrollTime += hexdec(bin2hex($configArr[$i++])) << 24;
        $config->poweroffDelay = hexdec(bin2hex($configArr[$i++]));
        $config->poweroffDelay += hexdec(bin2hex($configArr[$i++])) << 8;
        $config->poweroffDelay += hexdec(bin2hex($configArr[$i++])) << 16;
        $config->poweroffDelay += hexdec(bin2hex($configArr[$i++])) << 24;
        for ($j = 0; $j < 64; $j++) {
            if (bin2hex($configArr[$i + $j]) == '00')
                break;
        }
        $config->prjModel = iconv('GBK', 'UTF-8', substr($configArr, $i, $j));
        $i += 64;
        $config->parityBit = hexdec(bin2hex($configArr[$i++]));
        $config->sendTimes = hexdec(bin2hex($configArr[$i++]));
        $config->sendPeriod = hexdec(bin2hex($configArr[$i++]));
        $config->baudRate = hexdec(bin2hex($configArr[$i++]));
        $len = hexdec(bin2hex($configArr[$i++]));
        $config->prjCmdOpen = strtoupper(bin2hex(substr($configArr, $i, $len)));
        $i += 63;
        $len = hexdec(bin2hex($configArr[$i++]));
        $config->prjCmdClose = strtoupper(bin2hex(substr($configArr, $i, $len)));
        $i += 63;
        $len = hexdec(bin2hex($configArr[$i++]));
        $config->prjCmdFromVideo = strtoupper(bin2hex(substr($configArr, $i, $len)));
        $i += 63;
        $len = hexdec(bin2hex($configArr[$i++]));
        $config->prjCmdFromPc = strtoupper(bin2hex(substr($configArr, $i, $len)));
        $i = 468;
        //468-478为音频配置字段
        $config->defaultAudioVol = hexdec(bin2hex($configArr[$i++]));
        $config->defaultMicVol = hexdec(bin2hex($configArr[$i++]));
        if ($config->defaultAudioVol > 63) {
            $config->audioVolKeep = 1;
            $config->defaultAudioVol = 0;
        } else {
            $config->audioVolKeep = 0;
        }
        if ($config->defaultMicVol > 63) {
            $config->micVolKeep = 1;
            $config->defaultMicVol = 0;
        } else {
            $config->micVolKeep = 0;
        }
        $config->defaultAvSrc = hexdec(bin2hex($configArr[$i++]));
        $val = hexdec(bin2hex($configArr[$i++]));
        $config->prjChannelPc = $val & 0x01;
        $config->prjChannelExtend = ($val >> 2) & 0x03;
        $config->prjChannelPlatform = ($val >> 4) & 0x03;
        $config->prjChannelVideo = ($val >> 6) & 0x03;
        $config->audioAmp = hexdec(bin2hex($configArr[$i++]));
        $config->audioBass = hexdec(bin2hex($configArr[$i++]));
        $config->audioTreble = hexdec(bin2hex($configArr[$i++]));
        $config->micAmp = hexdec(bin2hex($configArr[$i++]));
        $config->micBass = hexdec(bin2hex($configArr[$i++]));
        $config->micTreble = hexdec(bin2hex($configArr[$i++]));
        $config->broadVol = hexdec(bin2hex($configArr[$i++]));
        $i = 479;
        //479为设备配置字段
        $val = hexdec(bin2hex($configArr[$i]));
        $config->innerHdmiAudioOn = ($val ^ 1) & 0x01;
        $config->videoEnhanceOn = ($val >> 1) & 0x01;
        $config->micEnhanceOn = ($val >> 2) & 0x01;
        $config->micEnhanceSrc = ($val >> 5) & 0x03;
        $config->mic1MagicPowerOn = ($val >> 3) & 0x01;
        $config->mic2MagicPowerOn = ($val >> 4) & 0x01;
        $config->vgaInnerEDID = ($val >> 7) & 0x01;
        // tm-03
        $config->configrationV2 = $device->Dev_ConfigurationV2;
        return $config;
    }

    public function fields()
    {
        if ($this->getScenario() == 'v2')
            return ['MaintainQrcode', 'configrationV2', 'ProjUsedTime', 'ProjWatermark', 'id', 'name', 'position',
                'pcIp', 'pcMainPasswd', 'pcPort', 'pcViewPasswd',
                'rtspUrl', 'rtspUrl1', 'rtspSub', 'rtspSub1'];
        else {
            $fields = parent::fields(); // TODO: Change the autogenerated stub
            unset($fields['configrationV2']);
            return $fields;
        }
    }
}
