<?php

namespace common\models;

use Yii;
use common\classes\MysqlAccess;

/**
 * This is the model class for table "v_device".
 *
 * @property string $id
 * @property string $name
 * @property integer $parent
 * @property string $description
 * @property string $ipaddress
 * @property integer $shouldSession
 * @property integer $actualSession
 * @property string $sessionName
 * @property string $power
 * @property string $projector
 * @property string $projectorLamp
 * @property string $exDevice1
 * @property string $exDevice2
 * @property string $projectorSrc
 * @property string $audioSrc
 * @property integer $audioVol
 * @property integer $micVol
 * @property integer $fault
 * @property string $micMute
 * @property string $audioMute
 * @property string $mute
 * @property string $pc
 * @property string $iotStatus
 * @property string $userName
 * @property string $ipcall
 * @property string $ipcallStatus
 */
class VDevice extends \yii\db\ActiveRecord
{
    public $devices;
    public $uid;
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'v_device';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['audioVol', 'micVol'], 'integer', 'max' => 63, 'min' => 0],
            [['devices'], 'match', 'pattern' => '/^[0-9][0-9, ]*[0-9]$/i'],
            [['iotStatus','power', 'projector', 'exDevice1', 'exDevice2', 'micMute', 'projectorSrc', 'audioSrc', 'audioMute', 'pc'], 'string', 'max' => 32],
            [['iotStatus','power', 'projector', 'exDevice1', 'exDevice2', 'micMute', 'projectorSrc', 'audioSrc', 'audioMute', 'pc'], 'trim']
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'parent' => 'Parent',
            'description' => 'Description',
            'ipaddress' => 'Ipaddress',
            'shouldSession' => 'Should Session',
            'actualSession' => 'Actual Session',
            'sessionName' => 'Session Name',
            'power' => 'Power',
            'projector' => 'Projector',
            'projectorLamp' => 'Projector Lamp',
            'exDevice1' => 'Ex Device1',
            'exDevice2' => 'Ex Device2',
            'projectorSrc' => 'Projector Src',
            'audioSrc' => 'Audio Src',
            'audioVol' => 'Audio Vol',
            'micVol' => 'Mic Vol',
            'fault' => 'Fault',
            'micMute' => 'Mic Mute',
            'audioMute' => 'Audio Mute',
            'pc' => 'Pc',
            'iotStatus' => 'Iot Status',
            'userName' => 'User Name',
            'ipcall' => 'Ipcall',
            'ipcallStatus' => 'Ipcall Status',
        ];
    }

    public function save($runValidation = true, $attributeNames = null)
    {
        $access = new MysqlAccess($this->devices);
        if(is_null($access))
            return false;
        try {
            if (isset($this->power)) {
                if ($this->power == 'on') {
                    $access->deviceCtrl(MysqlAccess::CMD_POWER, 1);
                    $ids = [];
                    foreach ($access->models as $model) {
                        $ids[] = $model->DeviceID;
                    }
                    Yii::$app->db->createCommand('update DeviceStatus set User_ID=' . $this->uid . ' where State_SysPower != 1 and DeviceID in (' . implode(",", $ids) . ')')
                        ->execute();
                } else if ($this->power == 'off') {
                    $access->deviceCtrl(MysqlAccess::CMD_POWER, 0);
                } else if ($this->power == 'disable') {
                    $access->deviceCtrl(MysqlAccess::CMD_POWER, 2);
                }
            }

            if (isset($this->pc)) {
                if ($this->pc == 'on') {  //power on
                    $access->deviceCtrl(MysqlAccess::CMD_PC, 1);
                } else if ($this->pc == 'off') { //power off
                    $access->deviceCtrl(MysqlAccess::CMD_PC, 0);
                } else if ($this->pc == 'force') { //force power off
                    $access->deviceCtrl(MysqlAccess::CMD_PC, 2);
                }
            }

            if (isset($this->projector)) {
                if ($this->projector == 'on') {
                    $access->deviceCtrl(MysqlAccess::CMD_PROJ, 1);
                } else if ($this->projector == 'off') {
                    $access->deviceCtrl(MysqlAccess::CMD_PROJ, 0);
                }
            }
            if (isset($this->audioMute)) {
                if ($this->audioMute == 'on') {
                    $access->deviceCtrl(MysqlAccess::CMD_AUDIO_MUTE, 1);
                } else if ($this->audioMute == 'off') {
                    $access->deviceCtrl(MysqlAccess::CMD_AUDIO_MUTE, 0);
                }
            }
            if (isset($this->micMute)) {
                if ($this->micMute == 'on') {
                    $access->deviceCtrl(MysqlAccess::CMD_MIC_MUTE, 0);
                } else if ($this->micMute == 'off') {
                    $access->deviceCtrl(MysqlAccess::CMD_MIC_MUTE, 1);
                }
            }

            if (isset($this->micVol) && is_numeric($this->micVol)) {
                $access->deviceCtrl(MysqlAccess::CMD_SET_VOL, 1, $this->micVol);

            }
            if (isset($this->audioVol) && is_numeric($this->audioVol)) {
                $access->deviceCtrl(MysqlAccess::CMD_SET_VOL, 0, $this->audioVol);
            }
            if (isset($this->projectorSrc)) {
                if ($this->projectorSrc == 'pc') {
                    $access->deviceCtrl(MysqlAccess::CMD_PROJ_SRC, 1);
                } else if ($this->projectorSrc == 'extend') {
                    $access->deviceCtrl(MysqlAccess::CMD_PROJ_SRC, 2);
                } else if ($this->projectorSrc == 'visualizer') {
                    $access->deviceCtrl(MysqlAccess::CMD_PROJ_SRC, 3);
                } else if ($this->projectorSrc == 'broadcast') {
                    $access->deviceCtrl(MysqlAccess::CMD_PROJ_SRC, 4);
                }
            }
            if (isset($this->iotStatus) && strlen($this->iotStatus) == 32 && ctype_xdigit($this->iotStatus)) {
                $access->deviceIotCtrl($this->iotStatus);
            }
        }catch (\Exception $e){
            return $e;
        }
        return true;
    }
}
